/*----------------------------------------------------------------------------------------
  Database dv
----------------------------------------------------------------------------------------*/

#include "dvdatabase.h"

union dvTempType_ dvTemp_;
struct dvRootType_ dvRootData;
uint8 dvModuleID;
struct dvRootFields dvRoots;
struct dvModpathFields dvModpaths;
struct dvModuleFields dvModules;
struct dvLinkFields dvLinks;
struct dvSchemaFields dvSchemas;
struct dvEnumFields dvEnums;
struct dvEntryFields dvEntrys;
struct dvTypedefFields dvTypedefs;
struct dvClassFields dvClasss;
struct dvPropertyFields dvPropertys;
struct dvRelationshipFields dvRelationships;
struct dvUnionFields dvUnions;
struct dvCaseFields dvCases;

/*----------------------------------------------------------------------------------------
  Constructor/Destructor hooks.
----------------------------------------------------------------------------------------*/
void(*dvRootConstructorCallback)(dvRoot);
void(*dvModpathConstructorCallback)(dvModpath);
void(*dvModuleConstructorCallback)(dvModule);
void(*dvLinkConstructorCallback)(dvLink);
void(*dvSchemaConstructorCallback)(dvSchema);
void(*dvEnumConstructorCallback)(dvEnum);
void(*dvEntryConstructorCallback)(dvEntry);
void(*dvTypedefConstructorCallback)(dvTypedef);
void(*dvClassConstructorCallback)(dvClass);
void(*dvPropertyConstructorCallback)(dvProperty);
void(*dvRelationshipConstructorCallback)(dvRelationship);
void(*dvUnionConstructorCallback)(dvUnion);
void(*dvUnionDestructorCallback)(dvUnion);
void(*dvCaseConstructorCallback)(dvCase);
void(*dvCaseDestructorCallback)(dvCase);

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocRoot(void)
{
    dvRoot Root = dvRootAlloc();

    return dvRoot2Index(Root);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Root.
----------------------------------------------------------------------------------------*/
static void allocRoots(void)
{
    dvSetAllocatedRoot(2);
    dvSetUsedRoot(0);
    dvRoots.FirstModpath = utNewA(dvModpath, (dvAllocatedRoot()));
    dvRoots.LastModpath = utNewA(dvModpath, (dvAllocatedRoot()));
    dvRoots.ModpathTableIndex = utNewA(uint32, (dvAllocatedRoot()));
    dvRoots.NumModpathTable = utNewA(uint32, (dvAllocatedRoot()));
    dvSetUsedRootModpathTable(0);
    dvSetAllocatedRootModpathTable(2);
    dvSetFreeRootModpathTable(0);
    dvRoots.ModpathTable = utNewA(dvModpath, dvAllocatedRootModpathTable());
    dvRoots.NumModpath = utNewA(uint32, (dvAllocatedRoot()));
    dvRoots.FirstModule = utNewA(dvModule, (dvAllocatedRoot()));
    dvRoots.LastModule = utNewA(dvModule, (dvAllocatedRoot()));
    dvRoots.ModuleTableIndex = utNewA(uint32, (dvAllocatedRoot()));
    dvRoots.NumModuleTable = utNewA(uint32, (dvAllocatedRoot()));
    dvSetUsedRootModuleTable(0);
    dvSetAllocatedRootModuleTable(2);
    dvSetFreeRootModuleTable(0);
    dvRoots.ModuleTable = utNewA(dvModule, dvAllocatedRootModuleTable());
    dvRoots.NumModule = utNewA(uint32, (dvAllocatedRoot()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Root.
----------------------------------------------------------------------------------------*/
static void reallocRoots(
    uint32 newSize)
{
    utResizeArray(dvRoots.FirstModpath, (newSize));
    utResizeArray(dvRoots.LastModpath, (newSize));
    utResizeArray(dvRoots.ModpathTableIndex, (newSize));
    utResizeArray(dvRoots.NumModpathTable, (newSize));
    utResizeArray(dvRoots.NumModpath, (newSize));
    utResizeArray(dvRoots.FirstModule, (newSize));
    utResizeArray(dvRoots.LastModule, (newSize));
    utResizeArray(dvRoots.ModuleTableIndex, (newSize));
    utResizeArray(dvRoots.NumModuleTable, (newSize));
    utResizeArray(dvRoots.NumModule, (newSize));
    dvSetAllocatedRoot(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Roots.
----------------------------------------------------------------------------------------*/
void dvRootAllocMore(void)
{
    reallocRoots(dvAllocatedRoot() + (dvAllocatedRoot() >> 1));
}

/*----------------------------------------------------------------------------------------
  Compact the Root.ModpathTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactRootModpathTables(void)
{
    uint32 elementSize = sizeof(dvModpath);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvModpath *toPtr = dvRoots.ModpathTable;
    dvModpath *fromPtr = toPtr;
    dvRoot Root;
    uint32 size;

    while(fromPtr < dvRoots.ModpathTable + dvUsedRootModpathTable()) {
        Root = *(dvRoot *)(void *)fromPtr;
        if(Root != dvRootNull) {
            /* Need to move it to toPtr */
            size = utMax(dvRootGetNumModpathTable(Root) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvRootSetModpathTableIndex(Root, toPtr - dvRoots.ModpathTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvRoot *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedRootModpathTable(toPtr - dvRoots.ModpathTable);
    dvSetFreeRootModpathTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Root.ModpathTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreRootModpathTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedRootModpathTable() - dvUsedRootModpathTable();

    if((dvFreeRootModpathTable() << 2) > dvUsedRootModpathTable()) {
        dvCompactRootModpathTables();
        freeSpace = dvAllocatedRootModpathTable() - dvUsedRootModpathTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedRootModpathTable(dvAllocatedRootModpathTable() + spaceNeeded - freeSpace +
            (dvAllocatedRootModpathTable() >> 2));
        utResizeArray(dvRoots.ModpathTable, dvAllocatedRootModpathTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Root.ModpathTable array.
----------------------------------------------------------------------------------------*/
void dvRootAllocModpathTables(
    dvRoot Root,
    uint32 numModpathTables)
{
    uint32 freeSpace = dvAllocatedRootModpathTable() - dvUsedRootModpathTable();
    uint32 elementSize = sizeof(dvModpath);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numModpathTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvRootGetNumModpathTable(Root) == 0);
#endif
    if(numModpathTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreRootModpathTables(spaceNeeded);
    }
    dvRootSetModpathTableIndex(Root, dvUsedRootModpathTable() + usedHeaderSize);
    dvRootSetNumModpathTable(Root, numModpathTables);
    *(dvRoot *)(void *)(dvRoots.ModpathTable + dvUsedRootModpathTable()) = Root;
    memset(dvRootGetModpathTables(Root), 0xff, numModpathTables*elementSize);
    dvSetUsedRootModpathTable(dvUsedRootModpathTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvRootGetModpathTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getRootModpathTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvRoot Root = dvIndex2Root(objectNumber);

    *numValues = dvRootGetNumModpathTable(Root);
    return dvRootGetModpathTables(Root);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvRootAllocModpathTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocRootModpathTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvRoot Root = dvIndex2Root(objectNumber);

    dvRootSetModpathTableIndex(Root, 0);
    dvRootSetNumModpathTable(Root, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvRootAllocModpathTables(Root, numValues);
    return dvRootGetModpathTables(Root);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Root.ModpathTable array.
----------------------------------------------------------------------------------------*/
void dvRootFreeModpathTables(
    dvRoot Root)
{
    uint32 elementSize = sizeof(dvModpath);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvRootGetNumModpathTable(Root) + usedHeaderSize, freeHeaderSize);
    dvModpath *dataPtr = dvRootGetModpathTables(Root) - usedHeaderSize;

    if(dvRootGetNumModpathTable(Root) == 0) {
        return;
    }
    *(dvRoot *)(void *)(dataPtr) = dvRootNull;
    *(uint32 *)(void *)(((dvRoot *)(void *)dataPtr) + 1) = size;
    dvRootSetNumModpathTable(Root, 0);
    dvSetFreeRootModpathTable(dvFreeRootModpathTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Root.ModpathTable array.
----------------------------------------------------------------------------------------*/
void dvRootResizeModpathTables(
    dvRoot Root,
    uint32 numModpathTables)
{
    uint32 freeSpace = dvAllocatedRootModpathTable() - dvUsedRootModpathTable();
    uint32 elementSize = sizeof(dvModpath);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numModpathTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvRootGetNumModpathTable(Root) + usedHeaderSize, freeHeaderSize);
    dvModpath *dataPtr;

    if(numModpathTables == 0) {
        if(dvRootGetNumModpathTable(Root) != 0) {
            dvRootFreeModpathTables(Root);
        };
        return;
    }
    if(dvRootGetNumModpathTable(Root) == 0) {
        dvRootAllocModpathTables(Root, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreRootModpathTables(newSize);
    }
    dataPtr = dvRootGetModpathTables(Root) - usedHeaderSize;
    memcpy((void *)(dvRoots.ModpathTable + dvUsedRootModpathTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvRoots.ModpathTable + dvUsedRootModpathTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvRoot *)(void *)dataPtr = dvRootNull;
    *(uint32 *)(void *)(((dvRoot *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeRootModpathTable(dvFreeRootModpathTable() + oldSize);
    dvRootSetModpathTableIndex(Root, dvUsedRootModpathTable() + usedHeaderSize);
    dvRootSetNumModpathTable(Root, numModpathTables);
    dvSetUsedRootModpathTable(dvUsedRootModpathTable() + newSize);
}

/*----------------------------------------------------------------------------------------
  Compact the Root.ModuleTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactRootModuleTables(void)
{
    uint32 elementSize = sizeof(dvModule);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvModule *toPtr = dvRoots.ModuleTable;
    dvModule *fromPtr = toPtr;
    dvRoot Root;
    uint32 size;

    while(fromPtr < dvRoots.ModuleTable + dvUsedRootModuleTable()) {
        Root = *(dvRoot *)(void *)fromPtr;
        if(Root != dvRootNull) {
            /* Need to move it to toPtr */
            size = utMax(dvRootGetNumModuleTable(Root) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvRootSetModuleTableIndex(Root, toPtr - dvRoots.ModuleTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvRoot *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedRootModuleTable(toPtr - dvRoots.ModuleTable);
    dvSetFreeRootModuleTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Root.ModuleTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreRootModuleTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedRootModuleTable() - dvUsedRootModuleTable();

    if((dvFreeRootModuleTable() << 2) > dvUsedRootModuleTable()) {
        dvCompactRootModuleTables();
        freeSpace = dvAllocatedRootModuleTable() - dvUsedRootModuleTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedRootModuleTable(dvAllocatedRootModuleTable() + spaceNeeded - freeSpace +
            (dvAllocatedRootModuleTable() >> 2));
        utResizeArray(dvRoots.ModuleTable, dvAllocatedRootModuleTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Root.ModuleTable array.
----------------------------------------------------------------------------------------*/
void dvRootAllocModuleTables(
    dvRoot Root,
    uint32 numModuleTables)
{
    uint32 freeSpace = dvAllocatedRootModuleTable() - dvUsedRootModuleTable();
    uint32 elementSize = sizeof(dvModule);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numModuleTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvRootGetNumModuleTable(Root) == 0);
#endif
    if(numModuleTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreRootModuleTables(spaceNeeded);
    }
    dvRootSetModuleTableIndex(Root, dvUsedRootModuleTable() + usedHeaderSize);
    dvRootSetNumModuleTable(Root, numModuleTables);
    *(dvRoot *)(void *)(dvRoots.ModuleTable + dvUsedRootModuleTable()) = Root;
    memset(dvRootGetModuleTables(Root), 0xff, numModuleTables*elementSize);
    dvSetUsedRootModuleTable(dvUsedRootModuleTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvRootGetModuleTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getRootModuleTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvRoot Root = dvIndex2Root(objectNumber);

    *numValues = dvRootGetNumModuleTable(Root);
    return dvRootGetModuleTables(Root);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvRootAllocModuleTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocRootModuleTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvRoot Root = dvIndex2Root(objectNumber);

    dvRootSetModuleTableIndex(Root, 0);
    dvRootSetNumModuleTable(Root, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvRootAllocModuleTables(Root, numValues);
    return dvRootGetModuleTables(Root);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Root.ModuleTable array.
----------------------------------------------------------------------------------------*/
void dvRootFreeModuleTables(
    dvRoot Root)
{
    uint32 elementSize = sizeof(dvModule);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvRootGetNumModuleTable(Root) + usedHeaderSize, freeHeaderSize);
    dvModule *dataPtr = dvRootGetModuleTables(Root) - usedHeaderSize;

    if(dvRootGetNumModuleTable(Root) == 0) {
        return;
    }
    *(dvRoot *)(void *)(dataPtr) = dvRootNull;
    *(uint32 *)(void *)(((dvRoot *)(void *)dataPtr) + 1) = size;
    dvRootSetNumModuleTable(Root, 0);
    dvSetFreeRootModuleTable(dvFreeRootModuleTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Root.ModuleTable array.
----------------------------------------------------------------------------------------*/
void dvRootResizeModuleTables(
    dvRoot Root,
    uint32 numModuleTables)
{
    uint32 freeSpace = dvAllocatedRootModuleTable() - dvUsedRootModuleTable();
    uint32 elementSize = sizeof(dvModule);
    uint32 usedHeaderSize = (sizeof(dvRoot) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvRoot) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numModuleTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvRootGetNumModuleTable(Root) + usedHeaderSize, freeHeaderSize);
    dvModule *dataPtr;

    if(numModuleTables == 0) {
        if(dvRootGetNumModuleTable(Root) != 0) {
            dvRootFreeModuleTables(Root);
        };
        return;
    }
    if(dvRootGetNumModuleTable(Root) == 0) {
        dvRootAllocModuleTables(Root, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreRootModuleTables(newSize);
    }
    dataPtr = dvRootGetModuleTables(Root) - usedHeaderSize;
    memcpy((void *)(dvRoots.ModuleTable + dvUsedRootModuleTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvRoots.ModuleTable + dvUsedRootModuleTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvRoot *)(void *)dataPtr = dvRootNull;
    *(uint32 *)(void *)(((dvRoot *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeRootModuleTable(dvFreeRootModuleTable() + oldSize);
    dvRootSetModuleTableIndex(Root, dvUsedRootModuleTable() + usedHeaderSize);
    dvRootSetNumModuleTable(Root, numModuleTables);
    dvSetUsedRootModuleTable(dvUsedRootModuleTable() + newSize);
}

static void addRootModpathToHashTable(dvRoot Root, dvModpath _Modpath);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeRootModpathHashTable(
    dvRoot Root)
{
    dvModpath _Modpath;
    dvModpath *Modpaths;
    uint32 numModpaths = dvRootGetNumModpathTable(Root) << 1;

    if(numModpaths == 0) {
        numModpaths = 2;
        dvRootAllocModpathTables(Root, 2);
    } else {
        dvRootResizeModpathTables(Root, numModpaths);
    }
    Modpaths = dvRootGetModpathTables(Root);
    /* Zero out the table */
    while(numModpaths-- != 0) {
        *Modpaths++ = dvModpathNull;
    }
    dvRootSetNumModpath(Root, 0);
    dvForeachRootModpath(Root, _Modpath) {
        if(dvModpathGetSym(_Modpath) != utSymNull) {
            addRootModpathToHashTable(Root, _Modpath);
        }
    } dvEndForeachRootModpath;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Root.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addRootModpathToHashTable(
    dvRoot Root,
    dvModpath _Modpath)
{
    dvModpath nextModpath;
    uint32 index;

    if(dvRootGetNumModpath(Root) >= dvRootGetNumModpathTable(Root)) {
        resizeRootModpathHashTable(Root);
        return;
    }
    index = (dvRootGetNumModpathTable(Root) - 1) & utSymGetHashValue(dvModpathGetSym(_Modpath));
    nextModpath = dvRootGetiModpathTable(Root, index);
    dvModpathSetNextTableRootModpath(_Modpath, nextModpath);
    dvRootSetiModpathTable(Root, index, _Modpath);
    dvRootSetNumModpath(Root, dvRootGetNumModpath(Root) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Modpath from the hash table.
----------------------------------------------------------------------------------------*/
static void removeRootModpathFromHashTable(
   dvRoot Root,
   dvModpath _Modpath)
{
    uint32 index = (dvRootGetNumModpathTable(Root) - 1) & utSymGetHashValue(dvModpathGetSym(_Modpath));
    dvModpath prevModpath, nextModpath;
    
    nextModpath = dvRootGetiModpathTable(Root, index);
    if(nextModpath == _Modpath) {
        dvRootSetiModpathTable(Root, index, dvModpathGetNextTableRootModpath(nextModpath));
    } else {
        do {
            prevModpath = nextModpath;
            nextModpath = dvModpathGetNextTableRootModpath(nextModpath);
        } while(nextModpath != _Modpath);
        dvModpathSetNextTableRootModpath(prevModpath, dvModpathGetNextTableRootModpath(_Modpath));
    }
    dvRootSetNumModpath(Root, dvRootGetNumModpath(Root) - 1);
    dvModpathSetNextTableRootModpath(_Modpath, dvModpathNull);
}

/*----------------------------------------------------------------------------------------
  Find the Modpath from the Root and its name.
----------------------------------------------------------------------------------------*/
dvModpath dvRootFindModpath(
    dvRoot Root,
    utSym sym)
{
    uint32 mask = dvRootGetNumModpathTable(Root) - 1;
    dvModpath _Modpath;

    if(mask + 1 != 0) {
        _Modpath = dvRootGetiModpathTable(Root, utSymGetHashValue(sym) & mask);
        while(_Modpath != dvModpathNull) {
            if(dvModpathGetSym(_Modpath) == sym) {
                return _Modpath;
            }
            _Modpath = dvModpathGetNextTableRootModpath(_Modpath);
        }
    }
    return dvModpathNull;
}

/*----------------------------------------------------------------------------------------
  Find the Modpath from the Root and its name.
----------------------------------------------------------------------------------------*/
void dvRootRenameModpath(
    dvRoot Root,
    dvModpath _Modpath,
    utSym sym)
{
    if(dvModpathGetSym(_Modpath) != utSymNull) {
        removeRootModpathFromHashTable(Root, _Modpath);
    }
    dvModpathSetSym(_Modpath, sym);
    if(sym != utSymNull) {
        addRootModpathToHashTable(Root, _Modpath);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Modpath to the head of the list on the Root.
----------------------------------------------------------------------------------------*/
void dvRootInsertModpath(
    dvRoot Root,
    dvModpath _Modpath)
{
#if defined(DD_DEBUG)
    if(Root == dvRootNull) {
        utExit("Non-existent Root");
    }
    if(_Modpath == dvModpathNull) {
        utExit("Non-existent Modpath");
    }
    if(dvModpathGetRoot(_Modpath) != dvRootNull) {
        utExit("Attempting to add dv to Root twice");
    }
#endif
    dvModpathSetNextRootModpath(_Modpath, dvRootGetFirstModpath(Root));
    if(dvRootGetFirstModpath(Root) != dvModpathNull) {
        dvModpathSetPrevRootModpath(dvRootGetFirstModpath(Root), _Modpath);
    }
    dvRootSetFirstModpath(Root, _Modpath);
    dvModpathSetPrevRootModpath(_Modpath, dvModpathNull);
    if(dvRootGetLastModpath(Root) == dvModpathNull) {
        dvRootSetLastModpath(Root, _Modpath);
    }
    dvModpathSetRoot(_Modpath, Root);
    if(dvModpathGetSym(_Modpath) != utSymNull) {
        addRootModpathToHashTable(Root, _Modpath);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Modpath to the end of the list on the Root.
----------------------------------------------------------------------------------------*/
void dvRootAppendModpath(
    dvRoot Root,
    dvModpath _Modpath)
{
#if defined(DD_DEBUG)
    if(Root == dvRootNull) {
        utExit("Non-existent Root");
    }
    if(_Modpath == dvModpathNull) {
        utExit("Non-existent Modpath");
    }
    if(dvModpathGetRoot(_Modpath) != dvRootNull) {
        utExit("Attempting to add dv to Root twice");
    }
#endif
    dvModpathSetPrevRootModpath(_Modpath, dvRootGetLastModpath(Root));
    if(dvRootGetLastModpath(Root) != dvModpathNull) {
        dvModpathSetNextRootModpath(dvRootGetLastModpath(Root), _Modpath);
    }
    dvRootSetLastModpath(Root, _Modpath);
    dvModpathSetNextRootModpath(_Modpath, dvModpathNull);
    if(dvRootGetFirstModpath(Root) == dvModpathNull) {
        dvRootSetFirstModpath(Root, _Modpath);
    }
    dvModpathSetRoot(_Modpath, Root);
    if(dvModpathGetSym(_Modpath) != utSymNull) {
        addRootModpathToHashTable(Root, _Modpath);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Modpath to the Root after the previous Modpath.
----------------------------------------------------------------------------------------*/
void dvRootInsertAfterModpath(
    dvRoot Root,
    dvModpath prevModpath,
    dvModpath _Modpath)
{
    dvModpath nextModpath = dvModpathGetNextRootModpath(prevModpath);

#if defined(DD_DEBUG)
    if(Root == dvRootNull) {
        utExit("Non-existent Root");
    }
    if(_Modpath == dvModpathNull) {
        utExit("Non-existent Modpath");
    }
    if(dvModpathGetRoot(_Modpath) != dvRootNull) {
        utExit("Attempting to add dv to Root twice");
    }
#endif
    dvModpathSetNextRootModpath(_Modpath, nextModpath);
    dvModpathSetNextRootModpath(prevModpath, _Modpath);
    dvModpathSetPrevRootModpath(_Modpath, prevModpath);
    if(nextModpath != dvModpathNull) {
        dvModpathSetPrevRootModpath(nextModpath, _Modpath);
    }
    if(dvRootGetLastModpath(Root) == prevModpath) {
        dvRootSetLastModpath(Root, _Modpath);
    }
    dvModpathSetRoot(_Modpath, Root);
    if(dvModpathGetSym(_Modpath) != utSymNull) {
        addRootModpathToHashTable(Root, _Modpath);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Modpath from the Root.
----------------------------------------------------------------------------------------*/
void dvRootRemoveModpath(
    dvRoot Root,
    dvModpath _Modpath)
{
    dvModpath pModpath, nModpath;

#if defined(DD_DEBUG)
    if(_Modpath == dvModpathNull) {
        utExit("Non-existent Modpath");
    }
    if(dvModpathGetRoot(_Modpath) != dvRootNull && dvModpathGetRoot(_Modpath) != Root) {
        utExit("Delete Modpath from non-owning Root");
    }
#endif
    nModpath = dvModpathGetNextRootModpath(_Modpath);
    pModpath = dvModpathGetPrevRootModpath(_Modpath);
    if(pModpath != dvModpathNull) {
        dvModpathSetNextRootModpath(pModpath, nModpath);
    } else if(dvRootGetFirstModpath(Root) == _Modpath) {
        dvRootSetFirstModpath(Root, nModpath);
    }
    if(nModpath != dvModpathNull) {
        dvModpathSetPrevRootModpath(nModpath, pModpath);
    } else if(dvRootGetLastModpath(Root) == _Modpath) {
        dvRootSetLastModpath(Root, pModpath);
    }
    dvModpathSetNextRootModpath(_Modpath, dvModpathNull);
    dvModpathSetPrevRootModpath(_Modpath, dvModpathNull);
    dvModpathSetRoot(_Modpath, dvRootNull);
    if(dvModpathGetSym(_Modpath) != utSymNull) {
        removeRootModpathFromHashTable(Root, _Modpath);
    }
}

static void addRootModuleToHashTable(dvRoot Root, dvModule _Module);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeRootModuleHashTable(
    dvRoot Root)
{
    dvModule _Module;
    dvModule *Modules;
    uint32 numModules = dvRootGetNumModuleTable(Root) << 1;

    if(numModules == 0) {
        numModules = 2;
        dvRootAllocModuleTables(Root, 2);
    } else {
        dvRootResizeModuleTables(Root, numModules);
    }
    Modules = dvRootGetModuleTables(Root);
    /* Zero out the table */
    while(numModules-- != 0) {
        *Modules++ = dvModuleNull;
    }
    dvRootSetNumModule(Root, 0);
    dvForeachRootModule(Root, _Module) {
        if(dvModuleGetSym(_Module) != utSymNull) {
            addRootModuleToHashTable(Root, _Module);
        }
    } dvEndForeachRootModule;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Root.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addRootModuleToHashTable(
    dvRoot Root,
    dvModule _Module)
{
    dvModule nextModule;
    uint32 index;

    if(dvRootGetNumModule(Root) >= dvRootGetNumModuleTable(Root)) {
        resizeRootModuleHashTable(Root);
        return;
    }
    index = (dvRootGetNumModuleTable(Root) - 1) & utSymGetHashValue(dvModuleGetSym(_Module));
    nextModule = dvRootGetiModuleTable(Root, index);
    dvModuleSetNextTableRootModule(_Module, nextModule);
    dvRootSetiModuleTable(Root, index, _Module);
    dvRootSetNumModule(Root, dvRootGetNumModule(Root) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Module from the hash table.
----------------------------------------------------------------------------------------*/
static void removeRootModuleFromHashTable(
   dvRoot Root,
   dvModule _Module)
{
    uint32 index = (dvRootGetNumModuleTable(Root) - 1) & utSymGetHashValue(dvModuleGetSym(_Module));
    dvModule prevModule, nextModule;
    
    nextModule = dvRootGetiModuleTable(Root, index);
    if(nextModule == _Module) {
        dvRootSetiModuleTable(Root, index, dvModuleGetNextTableRootModule(nextModule));
    } else {
        do {
            prevModule = nextModule;
            nextModule = dvModuleGetNextTableRootModule(nextModule);
        } while(nextModule != _Module);
        dvModuleSetNextTableRootModule(prevModule, dvModuleGetNextTableRootModule(_Module));
    }
    dvRootSetNumModule(Root, dvRootGetNumModule(Root) - 1);
    dvModuleSetNextTableRootModule(_Module, dvModuleNull);
}

/*----------------------------------------------------------------------------------------
  Find the Module from the Root and its name.
----------------------------------------------------------------------------------------*/
dvModule dvRootFindModule(
    dvRoot Root,
    utSym sym)
{
    uint32 mask = dvRootGetNumModuleTable(Root) - 1;
    dvModule _Module;

    if(mask + 1 != 0) {
        _Module = dvRootGetiModuleTable(Root, utSymGetHashValue(sym) & mask);
        while(_Module != dvModuleNull) {
            if(dvModuleGetSym(_Module) == sym) {
                return _Module;
            }
            _Module = dvModuleGetNextTableRootModule(_Module);
        }
    }
    return dvModuleNull;
}

/*----------------------------------------------------------------------------------------
  Find the Module from the Root and its name.
----------------------------------------------------------------------------------------*/
void dvRootRenameModule(
    dvRoot Root,
    dvModule _Module,
    utSym sym)
{
    if(dvModuleGetSym(_Module) != utSymNull) {
        removeRootModuleFromHashTable(Root, _Module);
    }
    dvModuleSetSym(_Module, sym);
    if(sym != utSymNull) {
        addRootModuleToHashTable(Root, _Module);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Module to the head of the list on the Root.
----------------------------------------------------------------------------------------*/
void dvRootInsertModule(
    dvRoot Root,
    dvModule _Module)
{
#if defined(DD_DEBUG)
    if(Root == dvRootNull) {
        utExit("Non-existent Root");
    }
    if(_Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
#endif
    dvModuleSetNextRootModule(_Module, dvRootGetFirstModule(Root));
    if(dvRootGetFirstModule(Root) != dvModuleNull) {
        dvModuleSetPrevRootModule(dvRootGetFirstModule(Root), _Module);
    }
    dvRootSetFirstModule(Root, _Module);
    dvModuleSetPrevRootModule(_Module, dvModuleNull);
    if(dvRootGetLastModule(Root) == dvModuleNull) {
        dvRootSetLastModule(Root, _Module);
    }
    if(dvModuleGetSym(_Module) != utSymNull) {
        addRootModuleToHashTable(Root, _Module);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Module to the end of the list on the Root.
----------------------------------------------------------------------------------------*/
void dvRootAppendModule(
    dvRoot Root,
    dvModule _Module)
{
#if defined(DD_DEBUG)
    if(Root == dvRootNull) {
        utExit("Non-existent Root");
    }
    if(_Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
#endif
    dvModuleSetPrevRootModule(_Module, dvRootGetLastModule(Root));
    if(dvRootGetLastModule(Root) != dvModuleNull) {
        dvModuleSetNextRootModule(dvRootGetLastModule(Root), _Module);
    }
    dvRootSetLastModule(Root, _Module);
    dvModuleSetNextRootModule(_Module, dvModuleNull);
    if(dvRootGetFirstModule(Root) == dvModuleNull) {
        dvRootSetFirstModule(Root, _Module);
    }
    if(dvModuleGetSym(_Module) != utSymNull) {
        addRootModuleToHashTable(Root, _Module);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Module to the Root after the previous Module.
----------------------------------------------------------------------------------------*/
void dvRootInsertAfterModule(
    dvRoot Root,
    dvModule prevModule,
    dvModule _Module)
{
    dvModule nextModule = dvModuleGetNextRootModule(prevModule);

#if defined(DD_DEBUG)
    if(Root == dvRootNull) {
        utExit("Non-existent Root");
    }
    if(_Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
#endif
    dvModuleSetNextRootModule(_Module, nextModule);
    dvModuleSetNextRootModule(prevModule, _Module);
    dvModuleSetPrevRootModule(_Module, prevModule);
    if(nextModule != dvModuleNull) {
        dvModuleSetPrevRootModule(nextModule, _Module);
    }
    if(dvRootGetLastModule(Root) == prevModule) {
        dvRootSetLastModule(Root, _Module);
    }
    if(dvModuleGetSym(_Module) != utSymNull) {
        addRootModuleToHashTable(Root, _Module);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Module from the Root.
----------------------------------------------------------------------------------------*/
void dvRootRemoveModule(
    dvRoot Root,
    dvModule _Module)
{
    dvModule pModule, nModule;

#if defined(DD_DEBUG)
    if(_Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
#endif
    nModule = dvModuleGetNextRootModule(_Module);
    pModule = dvModuleGetPrevRootModule(_Module);
    if(pModule != dvModuleNull) {
        dvModuleSetNextRootModule(pModule, nModule);
    } else if(dvRootGetFirstModule(Root) == _Module) {
        dvRootSetFirstModule(Root, nModule);
    }
    if(nModule != dvModuleNull) {
        dvModuleSetPrevRootModule(nModule, pModule);
    } else if(dvRootGetLastModule(Root) == _Module) {
        dvRootSetLastModule(Root, pModule);
    }
    dvModuleSetNextRootModule(_Module, dvModuleNull);
    dvModuleSetPrevRootModule(_Module, dvModuleNull);
    if(dvModuleGetSym(_Module) != utSymNull) {
        removeRootModuleFromHashTable(Root, _Module);
    }
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowRoot(
    dvRoot Root)
{
    utDatabaseShowObject("dv", "Root", dvRoot2Index(Root));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocModpath(void)
{
    dvModpath Modpath = dvModpathAlloc();

    return dvModpath2Index(Modpath);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Modpath.
----------------------------------------------------------------------------------------*/
static void allocModpaths(void)
{
    dvSetAllocatedModpath(2);
    dvSetUsedModpath(0);
    dvModpaths.Root = utNewA(dvRoot, (dvAllocatedModpath()));
    dvModpaths.NextRootModpath = utNewA(dvModpath, (dvAllocatedModpath()));
    dvModpaths.PrevRootModpath = utNewA(dvModpath, (dvAllocatedModpath()));
    dvModpaths.Sym = utNewA(utSym, (dvAllocatedModpath()));
    dvModpaths.NextTableRootModpath = utNewA(dvModpath, (dvAllocatedModpath()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Modpath.
----------------------------------------------------------------------------------------*/
static void reallocModpaths(
    uint32 newSize)
{
    utResizeArray(dvModpaths.Root, (newSize));
    utResizeArray(dvModpaths.NextRootModpath, (newSize));
    utResizeArray(dvModpaths.PrevRootModpath, (newSize));
    utResizeArray(dvModpaths.Sym, (newSize));
    utResizeArray(dvModpaths.NextTableRootModpath, (newSize));
    dvSetAllocatedModpath(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Modpaths.
----------------------------------------------------------------------------------------*/
void dvModpathAllocMore(void)
{
    reallocModpaths(dvAllocatedModpath() + (dvAllocatedModpath() >> 1));
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowModpath(
    dvModpath Modpath)
{
    utDatabaseShowObject("dv", "Modpath", dvModpath2Index(Modpath));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocModule(void)
{
    dvModule Module = dvModuleAlloc();

    return dvModule2Index(Module);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Module.
----------------------------------------------------------------------------------------*/
static void allocModules(void)
{
    dvSetAllocatedModule(2);
    dvSetUsedModule(0);
    dvModules.PrefixSym = utNewA(utSym, (dvAllocatedModule()));
    dvModules.Persistent = utNewA(uint8, (dvAllocatedModule() + 7) >> 3);
    dvModules.UndoRedo = utNewA(uint8, (dvAllocatedModule() + 7) >> 3);
    dvModules.NumFields = utNewA(uint16, (dvAllocatedModule()));
    dvModules.NumClasses = utNewA(uint32, (dvAllocatedModule()));
    dvModules.NumEnums = utNewA(uint32, (dvAllocatedModule()));
    dvModules.NextRootModule = utNewA(dvModule, (dvAllocatedModule()));
    dvModules.PrevRootModule = utNewA(dvModule, (dvAllocatedModule()));
    dvModules.Sym = utNewA(utSym, (dvAllocatedModule()));
    dvModules.NextTableRootModule = utNewA(dvModule, (dvAllocatedModule()));
    dvModules.FirstClass = utNewA(dvClass, (dvAllocatedModule()));
    dvModules.LastClass = utNewA(dvClass, (dvAllocatedModule()));
    dvModules.ClassTableIndex = utNewA(uint32, (dvAllocatedModule()));
    dvModules.NumClassTable = utNewA(uint32, (dvAllocatedModule()));
    dvSetUsedModuleClassTable(0);
    dvSetAllocatedModuleClassTable(2);
    dvSetFreeModuleClassTable(0);
    dvModules.ClassTable = utNewA(dvClass, dvAllocatedModuleClassTable());
    dvModules.NumClass = utNewA(uint32, (dvAllocatedModule()));
    dvModules.FirstEnum = utNewA(dvEnum, (dvAllocatedModule()));
    dvModules.LastEnum = utNewA(dvEnum, (dvAllocatedModule()));
    dvModules.EnumTableIndex = utNewA(uint32, (dvAllocatedModule()));
    dvModules.NumEnumTable = utNewA(uint32, (dvAllocatedModule()));
    dvSetUsedModuleEnumTable(0);
    dvSetAllocatedModuleEnumTable(2);
    dvSetFreeModuleEnumTable(0);
    dvModules.EnumTable = utNewA(dvEnum, dvAllocatedModuleEnumTable());
    dvModules.NumEnum = utNewA(uint32, (dvAllocatedModule()));
    dvModules.FirstTypedef = utNewA(dvTypedef, (dvAllocatedModule()));
    dvModules.LastTypedef = utNewA(dvTypedef, (dvAllocatedModule()));
    dvModules.TypedefTableIndex = utNewA(uint32, (dvAllocatedModule()));
    dvModules.NumTypedefTable = utNewA(uint32, (dvAllocatedModule()));
    dvSetUsedModuleTypedefTable(0);
    dvSetAllocatedModuleTypedefTable(2);
    dvSetFreeModuleTypedefTable(0);
    dvModules.TypedefTable = utNewA(dvTypedef, dvAllocatedModuleTypedefTable());
    dvModules.NumTypedef = utNewA(uint32, (dvAllocatedModule()));
    dvModules.FirstSchema = utNewA(dvSchema, (dvAllocatedModule()));
    dvModules.LastSchema = utNewA(dvSchema, (dvAllocatedModule()));
    dvModules.SchemaTableIndex = utNewA(uint32, (dvAllocatedModule()));
    dvModules.NumSchemaTable = utNewA(uint32, (dvAllocatedModule()));
    dvSetUsedModuleSchemaTable(0);
    dvSetAllocatedModuleSchemaTable(2);
    dvSetFreeModuleSchemaTable(0);
    dvModules.SchemaTable = utNewA(dvSchema, dvAllocatedModuleSchemaTable());
    dvModules.NumSchema = utNewA(uint32, (dvAllocatedModule()));
    dvModules.FirstImportLink = utNewA(dvLink, (dvAllocatedModule()));
    dvModules.LastImportLink = utNewA(dvLink, (dvAllocatedModule()));
    dvModules.FirstExportLink = utNewA(dvLink, (dvAllocatedModule()));
    dvModules.LastExportLink = utNewA(dvLink, (dvAllocatedModule()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Module.
----------------------------------------------------------------------------------------*/
static void reallocModules(
    uint32 newSize)
{
    utResizeArray(dvModules.PrefixSym, (newSize));
    utResizeArray(dvModules.Persistent, (newSize + 7) >> 3);
    utResizeArray(dvModules.UndoRedo, (newSize + 7) >> 3);
    utResizeArray(dvModules.NumFields, (newSize));
    utResizeArray(dvModules.NumClasses, (newSize));
    utResizeArray(dvModules.NumEnums, (newSize));
    utResizeArray(dvModules.NextRootModule, (newSize));
    utResizeArray(dvModules.PrevRootModule, (newSize));
    utResizeArray(dvModules.Sym, (newSize));
    utResizeArray(dvModules.NextTableRootModule, (newSize));
    utResizeArray(dvModules.FirstClass, (newSize));
    utResizeArray(dvModules.LastClass, (newSize));
    utResizeArray(dvModules.ClassTableIndex, (newSize));
    utResizeArray(dvModules.NumClassTable, (newSize));
    utResizeArray(dvModules.NumClass, (newSize));
    utResizeArray(dvModules.FirstEnum, (newSize));
    utResizeArray(dvModules.LastEnum, (newSize));
    utResizeArray(dvModules.EnumTableIndex, (newSize));
    utResizeArray(dvModules.NumEnumTable, (newSize));
    utResizeArray(dvModules.NumEnum, (newSize));
    utResizeArray(dvModules.FirstTypedef, (newSize));
    utResizeArray(dvModules.LastTypedef, (newSize));
    utResizeArray(dvModules.TypedefTableIndex, (newSize));
    utResizeArray(dvModules.NumTypedefTable, (newSize));
    utResizeArray(dvModules.NumTypedef, (newSize));
    utResizeArray(dvModules.FirstSchema, (newSize));
    utResizeArray(dvModules.LastSchema, (newSize));
    utResizeArray(dvModules.SchemaTableIndex, (newSize));
    utResizeArray(dvModules.NumSchemaTable, (newSize));
    utResizeArray(dvModules.NumSchema, (newSize));
    utResizeArray(dvModules.FirstImportLink, (newSize));
    utResizeArray(dvModules.LastImportLink, (newSize));
    utResizeArray(dvModules.FirstExportLink, (newSize));
    utResizeArray(dvModules.LastExportLink, (newSize));
    dvSetAllocatedModule(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Modules.
----------------------------------------------------------------------------------------*/
void dvModuleAllocMore(void)
{
    reallocModules(dvAllocatedModule() + (dvAllocatedModule() >> 1));
}

/*----------------------------------------------------------------------------------------
  Compact the Module.ClassTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactModuleClassTables(void)
{
    uint32 elementSize = sizeof(dvClass);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvClass *toPtr = dvModules.ClassTable;
    dvClass *fromPtr = toPtr;
    dvModule Module;
    uint32 size;

    while(fromPtr < dvModules.ClassTable + dvUsedModuleClassTable()) {
        Module = *(dvModule *)(void *)fromPtr;
        if(Module != dvModuleNull) {
            /* Need to move it to toPtr */
            size = utMax(dvModuleGetNumClassTable(Module) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvModuleSetClassTableIndex(Module, toPtr - dvModules.ClassTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvModule *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedModuleClassTable(toPtr - dvModules.ClassTable);
    dvSetFreeModuleClassTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Module.ClassTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreModuleClassTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedModuleClassTable() - dvUsedModuleClassTable();

    if((dvFreeModuleClassTable() << 2) > dvUsedModuleClassTable()) {
        dvCompactModuleClassTables();
        freeSpace = dvAllocatedModuleClassTable() - dvUsedModuleClassTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedModuleClassTable(dvAllocatedModuleClassTable() + spaceNeeded - freeSpace +
            (dvAllocatedModuleClassTable() >> 2));
        utResizeArray(dvModules.ClassTable, dvAllocatedModuleClassTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Module.ClassTable array.
----------------------------------------------------------------------------------------*/
void dvModuleAllocClassTables(
    dvModule Module,
    uint32 numClassTables)
{
    uint32 freeSpace = dvAllocatedModuleClassTable() - dvUsedModuleClassTable();
    uint32 elementSize = sizeof(dvClass);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numClassTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvModuleGetNumClassTable(Module) == 0);
#endif
    if(numClassTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreModuleClassTables(spaceNeeded);
    }
    dvModuleSetClassTableIndex(Module, dvUsedModuleClassTable() + usedHeaderSize);
    dvModuleSetNumClassTable(Module, numClassTables);
    *(dvModule *)(void *)(dvModules.ClassTable + dvUsedModuleClassTable()) = Module;
    memset(dvModuleGetClassTables(Module), 0xff, numClassTables*elementSize);
    dvSetUsedModuleClassTable(dvUsedModuleClassTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleGetClassTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getModuleClassTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    *numValues = dvModuleGetNumClassTable(Module);
    return dvModuleGetClassTables(Module);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleAllocClassTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocModuleClassTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    dvModuleSetClassTableIndex(Module, 0);
    dvModuleSetNumClassTable(Module, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvModuleAllocClassTables(Module, numValues);
    return dvModuleGetClassTables(Module);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Module.ClassTable array.
----------------------------------------------------------------------------------------*/
void dvModuleFreeClassTables(
    dvModule Module)
{
    uint32 elementSize = sizeof(dvClass);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvModuleGetNumClassTable(Module) + usedHeaderSize, freeHeaderSize);
    dvClass *dataPtr = dvModuleGetClassTables(Module) - usedHeaderSize;

    if(dvModuleGetNumClassTable(Module) == 0) {
        return;
    }
    *(dvModule *)(void *)(dataPtr) = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = size;
    dvModuleSetNumClassTable(Module, 0);
    dvSetFreeModuleClassTable(dvFreeModuleClassTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Module.ClassTable array.
----------------------------------------------------------------------------------------*/
void dvModuleResizeClassTables(
    dvModule Module,
    uint32 numClassTables)
{
    uint32 freeSpace = dvAllocatedModuleClassTable() - dvUsedModuleClassTable();
    uint32 elementSize = sizeof(dvClass);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numClassTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvModuleGetNumClassTable(Module) + usedHeaderSize, freeHeaderSize);
    dvClass *dataPtr;

    if(numClassTables == 0) {
        if(dvModuleGetNumClassTable(Module) != 0) {
            dvModuleFreeClassTables(Module);
        };
        return;
    }
    if(dvModuleGetNumClassTable(Module) == 0) {
        dvModuleAllocClassTables(Module, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreModuleClassTables(newSize);
    }
    dataPtr = dvModuleGetClassTables(Module) - usedHeaderSize;
    memcpy((void *)(dvModules.ClassTable + dvUsedModuleClassTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvModules.ClassTable + dvUsedModuleClassTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvModule *)(void *)dataPtr = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeModuleClassTable(dvFreeModuleClassTable() + oldSize);
    dvModuleSetClassTableIndex(Module, dvUsedModuleClassTable() + usedHeaderSize);
    dvModuleSetNumClassTable(Module, numClassTables);
    dvSetUsedModuleClassTable(dvUsedModuleClassTable() + newSize);
}

/*----------------------------------------------------------------------------------------
  Compact the Module.EnumTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactModuleEnumTables(void)
{
    uint32 elementSize = sizeof(dvEnum);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvEnum *toPtr = dvModules.EnumTable;
    dvEnum *fromPtr = toPtr;
    dvModule Module;
    uint32 size;

    while(fromPtr < dvModules.EnumTable + dvUsedModuleEnumTable()) {
        Module = *(dvModule *)(void *)fromPtr;
        if(Module != dvModuleNull) {
            /* Need to move it to toPtr */
            size = utMax(dvModuleGetNumEnumTable(Module) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvModuleSetEnumTableIndex(Module, toPtr - dvModules.EnumTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvModule *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedModuleEnumTable(toPtr - dvModules.EnumTable);
    dvSetFreeModuleEnumTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Module.EnumTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreModuleEnumTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedModuleEnumTable() - dvUsedModuleEnumTable();

    if((dvFreeModuleEnumTable() << 2) > dvUsedModuleEnumTable()) {
        dvCompactModuleEnumTables();
        freeSpace = dvAllocatedModuleEnumTable() - dvUsedModuleEnumTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedModuleEnumTable(dvAllocatedModuleEnumTable() + spaceNeeded - freeSpace +
            (dvAllocatedModuleEnumTable() >> 2));
        utResizeArray(dvModules.EnumTable, dvAllocatedModuleEnumTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Module.EnumTable array.
----------------------------------------------------------------------------------------*/
void dvModuleAllocEnumTables(
    dvModule Module,
    uint32 numEnumTables)
{
    uint32 freeSpace = dvAllocatedModuleEnumTable() - dvUsedModuleEnumTable();
    uint32 elementSize = sizeof(dvEnum);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numEnumTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvModuleGetNumEnumTable(Module) == 0);
#endif
    if(numEnumTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreModuleEnumTables(spaceNeeded);
    }
    dvModuleSetEnumTableIndex(Module, dvUsedModuleEnumTable() + usedHeaderSize);
    dvModuleSetNumEnumTable(Module, numEnumTables);
    *(dvModule *)(void *)(dvModules.EnumTable + dvUsedModuleEnumTable()) = Module;
    memset(dvModuleGetEnumTables(Module), 0xff, numEnumTables*elementSize);
    dvSetUsedModuleEnumTable(dvUsedModuleEnumTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleGetEnumTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getModuleEnumTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    *numValues = dvModuleGetNumEnumTable(Module);
    return dvModuleGetEnumTables(Module);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleAllocEnumTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocModuleEnumTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    dvModuleSetEnumTableIndex(Module, 0);
    dvModuleSetNumEnumTable(Module, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvModuleAllocEnumTables(Module, numValues);
    return dvModuleGetEnumTables(Module);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Module.EnumTable array.
----------------------------------------------------------------------------------------*/
void dvModuleFreeEnumTables(
    dvModule Module)
{
    uint32 elementSize = sizeof(dvEnum);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvModuleGetNumEnumTable(Module) + usedHeaderSize, freeHeaderSize);
    dvEnum *dataPtr = dvModuleGetEnumTables(Module) - usedHeaderSize;

    if(dvModuleGetNumEnumTable(Module) == 0) {
        return;
    }
    *(dvModule *)(void *)(dataPtr) = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = size;
    dvModuleSetNumEnumTable(Module, 0);
    dvSetFreeModuleEnumTable(dvFreeModuleEnumTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Module.EnumTable array.
----------------------------------------------------------------------------------------*/
void dvModuleResizeEnumTables(
    dvModule Module,
    uint32 numEnumTables)
{
    uint32 freeSpace = dvAllocatedModuleEnumTable() - dvUsedModuleEnumTable();
    uint32 elementSize = sizeof(dvEnum);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numEnumTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvModuleGetNumEnumTable(Module) + usedHeaderSize, freeHeaderSize);
    dvEnum *dataPtr;

    if(numEnumTables == 0) {
        if(dvModuleGetNumEnumTable(Module) != 0) {
            dvModuleFreeEnumTables(Module);
        };
        return;
    }
    if(dvModuleGetNumEnumTable(Module) == 0) {
        dvModuleAllocEnumTables(Module, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreModuleEnumTables(newSize);
    }
    dataPtr = dvModuleGetEnumTables(Module) - usedHeaderSize;
    memcpy((void *)(dvModules.EnumTable + dvUsedModuleEnumTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvModules.EnumTable + dvUsedModuleEnumTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvModule *)(void *)dataPtr = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeModuleEnumTable(dvFreeModuleEnumTable() + oldSize);
    dvModuleSetEnumTableIndex(Module, dvUsedModuleEnumTable() + usedHeaderSize);
    dvModuleSetNumEnumTable(Module, numEnumTables);
    dvSetUsedModuleEnumTable(dvUsedModuleEnumTable() + newSize);
}

/*----------------------------------------------------------------------------------------
  Compact the Module.TypedefTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactModuleTypedefTables(void)
{
    uint32 elementSize = sizeof(dvTypedef);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvTypedef *toPtr = dvModules.TypedefTable;
    dvTypedef *fromPtr = toPtr;
    dvModule Module;
    uint32 size;

    while(fromPtr < dvModules.TypedefTable + dvUsedModuleTypedefTable()) {
        Module = *(dvModule *)(void *)fromPtr;
        if(Module != dvModuleNull) {
            /* Need to move it to toPtr */
            size = utMax(dvModuleGetNumTypedefTable(Module) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvModuleSetTypedefTableIndex(Module, toPtr - dvModules.TypedefTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvModule *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedModuleTypedefTable(toPtr - dvModules.TypedefTable);
    dvSetFreeModuleTypedefTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Module.TypedefTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreModuleTypedefTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedModuleTypedefTable() - dvUsedModuleTypedefTable();

    if((dvFreeModuleTypedefTable() << 2) > dvUsedModuleTypedefTable()) {
        dvCompactModuleTypedefTables();
        freeSpace = dvAllocatedModuleTypedefTable() - dvUsedModuleTypedefTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedModuleTypedefTable(dvAllocatedModuleTypedefTable() + spaceNeeded - freeSpace +
            (dvAllocatedModuleTypedefTable() >> 2));
        utResizeArray(dvModules.TypedefTable, dvAllocatedModuleTypedefTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Module.TypedefTable array.
----------------------------------------------------------------------------------------*/
void dvModuleAllocTypedefTables(
    dvModule Module,
    uint32 numTypedefTables)
{
    uint32 freeSpace = dvAllocatedModuleTypedefTable() - dvUsedModuleTypedefTable();
    uint32 elementSize = sizeof(dvTypedef);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numTypedefTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvModuleGetNumTypedefTable(Module) == 0);
#endif
    if(numTypedefTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreModuleTypedefTables(spaceNeeded);
    }
    dvModuleSetTypedefTableIndex(Module, dvUsedModuleTypedefTable() + usedHeaderSize);
    dvModuleSetNumTypedefTable(Module, numTypedefTables);
    *(dvModule *)(void *)(dvModules.TypedefTable + dvUsedModuleTypedefTable()) = Module;
    memset(dvModuleGetTypedefTables(Module), 0xff, numTypedefTables*elementSize);
    dvSetUsedModuleTypedefTable(dvUsedModuleTypedefTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleGetTypedefTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getModuleTypedefTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    *numValues = dvModuleGetNumTypedefTable(Module);
    return dvModuleGetTypedefTables(Module);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleAllocTypedefTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocModuleTypedefTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    dvModuleSetTypedefTableIndex(Module, 0);
    dvModuleSetNumTypedefTable(Module, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvModuleAllocTypedefTables(Module, numValues);
    return dvModuleGetTypedefTables(Module);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Module.TypedefTable array.
----------------------------------------------------------------------------------------*/
void dvModuleFreeTypedefTables(
    dvModule Module)
{
    uint32 elementSize = sizeof(dvTypedef);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvModuleGetNumTypedefTable(Module) + usedHeaderSize, freeHeaderSize);
    dvTypedef *dataPtr = dvModuleGetTypedefTables(Module) - usedHeaderSize;

    if(dvModuleGetNumTypedefTable(Module) == 0) {
        return;
    }
    *(dvModule *)(void *)(dataPtr) = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = size;
    dvModuleSetNumTypedefTable(Module, 0);
    dvSetFreeModuleTypedefTable(dvFreeModuleTypedefTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Module.TypedefTable array.
----------------------------------------------------------------------------------------*/
void dvModuleResizeTypedefTables(
    dvModule Module,
    uint32 numTypedefTables)
{
    uint32 freeSpace = dvAllocatedModuleTypedefTable() - dvUsedModuleTypedefTable();
    uint32 elementSize = sizeof(dvTypedef);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numTypedefTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvModuleGetNumTypedefTable(Module) + usedHeaderSize, freeHeaderSize);
    dvTypedef *dataPtr;

    if(numTypedefTables == 0) {
        if(dvModuleGetNumTypedefTable(Module) != 0) {
            dvModuleFreeTypedefTables(Module);
        };
        return;
    }
    if(dvModuleGetNumTypedefTable(Module) == 0) {
        dvModuleAllocTypedefTables(Module, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreModuleTypedefTables(newSize);
    }
    dataPtr = dvModuleGetTypedefTables(Module) - usedHeaderSize;
    memcpy((void *)(dvModules.TypedefTable + dvUsedModuleTypedefTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvModules.TypedefTable + dvUsedModuleTypedefTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvModule *)(void *)dataPtr = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeModuleTypedefTable(dvFreeModuleTypedefTable() + oldSize);
    dvModuleSetTypedefTableIndex(Module, dvUsedModuleTypedefTable() + usedHeaderSize);
    dvModuleSetNumTypedefTable(Module, numTypedefTables);
    dvSetUsedModuleTypedefTable(dvUsedModuleTypedefTable() + newSize);
}

/*----------------------------------------------------------------------------------------
  Compact the Module.SchemaTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactModuleSchemaTables(void)
{
    uint32 elementSize = sizeof(dvSchema);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvSchema *toPtr = dvModules.SchemaTable;
    dvSchema *fromPtr = toPtr;
    dvModule Module;
    uint32 size;

    while(fromPtr < dvModules.SchemaTable + dvUsedModuleSchemaTable()) {
        Module = *(dvModule *)(void *)fromPtr;
        if(Module != dvModuleNull) {
            /* Need to move it to toPtr */
            size = utMax(dvModuleGetNumSchemaTable(Module) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvModuleSetSchemaTableIndex(Module, toPtr - dvModules.SchemaTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvModule *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedModuleSchemaTable(toPtr - dvModules.SchemaTable);
    dvSetFreeModuleSchemaTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Module.SchemaTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreModuleSchemaTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedModuleSchemaTable() - dvUsedModuleSchemaTable();

    if((dvFreeModuleSchemaTable() << 2) > dvUsedModuleSchemaTable()) {
        dvCompactModuleSchemaTables();
        freeSpace = dvAllocatedModuleSchemaTable() - dvUsedModuleSchemaTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedModuleSchemaTable(dvAllocatedModuleSchemaTable() + spaceNeeded - freeSpace +
            (dvAllocatedModuleSchemaTable() >> 2));
        utResizeArray(dvModules.SchemaTable, dvAllocatedModuleSchemaTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Module.SchemaTable array.
----------------------------------------------------------------------------------------*/
void dvModuleAllocSchemaTables(
    dvModule Module,
    uint32 numSchemaTables)
{
    uint32 freeSpace = dvAllocatedModuleSchemaTable() - dvUsedModuleSchemaTable();
    uint32 elementSize = sizeof(dvSchema);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numSchemaTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvModuleGetNumSchemaTable(Module) == 0);
#endif
    if(numSchemaTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreModuleSchemaTables(spaceNeeded);
    }
    dvModuleSetSchemaTableIndex(Module, dvUsedModuleSchemaTable() + usedHeaderSize);
    dvModuleSetNumSchemaTable(Module, numSchemaTables);
    *(dvModule *)(void *)(dvModules.SchemaTable + dvUsedModuleSchemaTable()) = Module;
    memset(dvModuleGetSchemaTables(Module), 0xff, numSchemaTables*elementSize);
    dvSetUsedModuleSchemaTable(dvUsedModuleSchemaTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleGetSchemaTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getModuleSchemaTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    *numValues = dvModuleGetNumSchemaTable(Module);
    return dvModuleGetSchemaTables(Module);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvModuleAllocSchemaTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocModuleSchemaTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvModule Module = dvIndex2Module(objectNumber);

    dvModuleSetSchemaTableIndex(Module, 0);
    dvModuleSetNumSchemaTable(Module, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvModuleAllocSchemaTables(Module, numValues);
    return dvModuleGetSchemaTables(Module);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Module.SchemaTable array.
----------------------------------------------------------------------------------------*/
void dvModuleFreeSchemaTables(
    dvModule Module)
{
    uint32 elementSize = sizeof(dvSchema);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvModuleGetNumSchemaTable(Module) + usedHeaderSize, freeHeaderSize);
    dvSchema *dataPtr = dvModuleGetSchemaTables(Module) - usedHeaderSize;

    if(dvModuleGetNumSchemaTable(Module) == 0) {
        return;
    }
    *(dvModule *)(void *)(dataPtr) = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = size;
    dvModuleSetNumSchemaTable(Module, 0);
    dvSetFreeModuleSchemaTable(dvFreeModuleSchemaTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Module.SchemaTable array.
----------------------------------------------------------------------------------------*/
void dvModuleResizeSchemaTables(
    dvModule Module,
    uint32 numSchemaTables)
{
    uint32 freeSpace = dvAllocatedModuleSchemaTable() - dvUsedModuleSchemaTable();
    uint32 elementSize = sizeof(dvSchema);
    uint32 usedHeaderSize = (sizeof(dvModule) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvModule) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numSchemaTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvModuleGetNumSchemaTable(Module) + usedHeaderSize, freeHeaderSize);
    dvSchema *dataPtr;

    if(numSchemaTables == 0) {
        if(dvModuleGetNumSchemaTable(Module) != 0) {
            dvModuleFreeSchemaTables(Module);
        };
        return;
    }
    if(dvModuleGetNumSchemaTable(Module) == 0) {
        dvModuleAllocSchemaTables(Module, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreModuleSchemaTables(newSize);
    }
    dataPtr = dvModuleGetSchemaTables(Module) - usedHeaderSize;
    memcpy((void *)(dvModules.SchemaTable + dvUsedModuleSchemaTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvModules.SchemaTable + dvUsedModuleSchemaTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvModule *)(void *)dataPtr = dvModuleNull;
    *(uint32 *)(void *)(((dvModule *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeModuleSchemaTable(dvFreeModuleSchemaTable() + oldSize);
    dvModuleSetSchemaTableIndex(Module, dvUsedModuleSchemaTable() + usedHeaderSize);
    dvModuleSetNumSchemaTable(Module, numSchemaTables);
    dvSetUsedModuleSchemaTable(dvUsedModuleSchemaTable() + newSize);
}

static void addModuleClassToHashTable(dvModule Module, dvClass _Class);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeModuleClassHashTable(
    dvModule Module)
{
    dvClass _Class;
    dvClass *Classs;
    uint32 numClasss = dvModuleGetNumClassTable(Module) << 1;

    if(numClasss == 0) {
        numClasss = 2;
        dvModuleAllocClassTables(Module, 2);
    } else {
        dvModuleResizeClassTables(Module, numClasss);
    }
    Classs = dvModuleGetClassTables(Module);
    /* Zero out the table */
    while(numClasss-- != 0) {
        *Classs++ = dvClassNull;
    }
    dvModuleSetNumClass(Module, 0);
    dvForeachModuleClass(Module, _Class) {
        if(dvClassGetSym(_Class) != utSymNull) {
            addModuleClassToHashTable(Module, _Class);
        }
    } dvEndForeachModuleClass;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Module.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addModuleClassToHashTable(
    dvModule Module,
    dvClass _Class)
{
    dvClass nextClass;
    uint32 index;

    if(dvModuleGetNumClass(Module) >= dvModuleGetNumClassTable(Module)) {
        resizeModuleClassHashTable(Module);
        return;
    }
    index = (dvModuleGetNumClassTable(Module) - 1) & utSymGetHashValue(dvClassGetSym(_Class));
    nextClass = dvModuleGetiClassTable(Module, index);
    dvClassSetNextTableModuleClass(_Class, nextClass);
    dvModuleSetiClassTable(Module, index, _Class);
    dvModuleSetNumClass(Module, dvModuleGetNumClass(Module) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Class from the hash table.
----------------------------------------------------------------------------------------*/
static void removeModuleClassFromHashTable(
   dvModule Module,
   dvClass _Class)
{
    uint32 index = (dvModuleGetNumClassTable(Module) - 1) & utSymGetHashValue(dvClassGetSym(_Class));
    dvClass prevClass, nextClass;
    
    nextClass = dvModuleGetiClassTable(Module, index);
    if(nextClass == _Class) {
        dvModuleSetiClassTable(Module, index, dvClassGetNextTableModuleClass(nextClass));
    } else {
        do {
            prevClass = nextClass;
            nextClass = dvClassGetNextTableModuleClass(nextClass);
        } while(nextClass != _Class);
        dvClassSetNextTableModuleClass(prevClass, dvClassGetNextTableModuleClass(_Class));
    }
    dvModuleSetNumClass(Module, dvModuleGetNumClass(Module) - 1);
    dvClassSetNextTableModuleClass(_Class, dvClassNull);
}

/*----------------------------------------------------------------------------------------
  Find the Class from the Module and its name.
----------------------------------------------------------------------------------------*/
dvClass dvModuleFindClass(
    dvModule Module,
    utSym sym)
{
    uint32 mask = dvModuleGetNumClassTable(Module) - 1;
    dvClass _Class;

    if(mask + 1 != 0) {
        _Class = dvModuleGetiClassTable(Module, utSymGetHashValue(sym) & mask);
        while(_Class != dvClassNull) {
            if(dvClassGetSym(_Class) == sym) {
                return _Class;
            }
            _Class = dvClassGetNextTableModuleClass(_Class);
        }
    }
    return dvClassNull;
}

/*----------------------------------------------------------------------------------------
  Find the Class from the Module and its name.
----------------------------------------------------------------------------------------*/
void dvModuleRenameClass(
    dvModule Module,
    dvClass _Class,
    utSym sym)
{
    if(dvClassGetSym(_Class) != utSymNull) {
        removeModuleClassFromHashTable(Module, _Class);
    }
    dvClassSetSym(_Class, sym);
    if(sym != utSymNull) {
        addModuleClassToHashTable(Module, _Class);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Class to the head of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleInsertClass(
    dvModule Module,
    dvClass _Class)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetModule(_Class) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvClassSetNextModuleClass(_Class, dvModuleGetFirstClass(Module));
    if(dvModuleGetFirstClass(Module) != dvClassNull) {
        dvClassSetPrevModuleClass(dvModuleGetFirstClass(Module), _Class);
    }
    dvModuleSetFirstClass(Module, _Class);
    dvClassSetPrevModuleClass(_Class, dvClassNull);
    if(dvModuleGetLastClass(Module) == dvClassNull) {
        dvModuleSetLastClass(Module, _Class);
    }
    dvClassSetModule(_Class, Module);
    if(dvClassGetSym(_Class) != utSymNull) {
        addModuleClassToHashTable(Module, _Class);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Class to the end of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleAppendClass(
    dvModule Module,
    dvClass _Class)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetModule(_Class) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvClassSetPrevModuleClass(_Class, dvModuleGetLastClass(Module));
    if(dvModuleGetLastClass(Module) != dvClassNull) {
        dvClassSetNextModuleClass(dvModuleGetLastClass(Module), _Class);
    }
    dvModuleSetLastClass(Module, _Class);
    dvClassSetNextModuleClass(_Class, dvClassNull);
    if(dvModuleGetFirstClass(Module) == dvClassNull) {
        dvModuleSetFirstClass(Module, _Class);
    }
    dvClassSetModule(_Class, Module);
    if(dvClassGetSym(_Class) != utSymNull) {
        addModuleClassToHashTable(Module, _Class);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Class to the Module after the previous Class.
----------------------------------------------------------------------------------------*/
void dvModuleInsertAfterClass(
    dvModule Module,
    dvClass prevClass,
    dvClass _Class)
{
    dvClass nextClass = dvClassGetNextModuleClass(prevClass);

#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetModule(_Class) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvClassSetNextModuleClass(_Class, nextClass);
    dvClassSetNextModuleClass(prevClass, _Class);
    dvClassSetPrevModuleClass(_Class, prevClass);
    if(nextClass != dvClassNull) {
        dvClassSetPrevModuleClass(nextClass, _Class);
    }
    if(dvModuleGetLastClass(Module) == prevClass) {
        dvModuleSetLastClass(Module, _Class);
    }
    dvClassSetModule(_Class, Module);
    if(dvClassGetSym(_Class) != utSymNull) {
        addModuleClassToHashTable(Module, _Class);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Class from the Module.
----------------------------------------------------------------------------------------*/
void dvModuleRemoveClass(
    dvModule Module,
    dvClass _Class)
{
    dvClass pClass, nClass;

#if defined(DD_DEBUG)
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetModule(_Class) != dvModuleNull && dvClassGetModule(_Class) != Module) {
        utExit("Delete Class from non-owning Module");
    }
#endif
    nClass = dvClassGetNextModuleClass(_Class);
    pClass = dvClassGetPrevModuleClass(_Class);
    if(pClass != dvClassNull) {
        dvClassSetNextModuleClass(pClass, nClass);
    } else if(dvModuleGetFirstClass(Module) == _Class) {
        dvModuleSetFirstClass(Module, nClass);
    }
    if(nClass != dvClassNull) {
        dvClassSetPrevModuleClass(nClass, pClass);
    } else if(dvModuleGetLastClass(Module) == _Class) {
        dvModuleSetLastClass(Module, pClass);
    }
    dvClassSetNextModuleClass(_Class, dvClassNull);
    dvClassSetPrevModuleClass(_Class, dvClassNull);
    dvClassSetModule(_Class, dvModuleNull);
    if(dvClassGetSym(_Class) != utSymNull) {
        removeModuleClassFromHashTable(Module, _Class);
    }
}

static void addModuleEnumToHashTable(dvModule Module, dvEnum _Enum);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeModuleEnumHashTable(
    dvModule Module)
{
    dvEnum _Enum;
    dvEnum *Enums;
    uint32 numEnums = dvModuleGetNumEnumTable(Module) << 1;

    if(numEnums == 0) {
        numEnums = 2;
        dvModuleAllocEnumTables(Module, 2);
    } else {
        dvModuleResizeEnumTables(Module, numEnums);
    }
    Enums = dvModuleGetEnumTables(Module);
    /* Zero out the table */
    while(numEnums-- != 0) {
        *Enums++ = dvEnumNull;
    }
    dvModuleSetNumEnum(Module, 0);
    dvForeachModuleEnum(Module, _Enum) {
        if(dvEnumGetSym(_Enum) != utSymNull) {
            addModuleEnumToHashTable(Module, _Enum);
        }
    } dvEndForeachModuleEnum;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Module.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addModuleEnumToHashTable(
    dvModule Module,
    dvEnum _Enum)
{
    dvEnum nextEnum;
    uint32 index;

    if(dvModuleGetNumEnum(Module) >= dvModuleGetNumEnumTable(Module)) {
        resizeModuleEnumHashTable(Module);
        return;
    }
    index = (dvModuleGetNumEnumTable(Module) - 1) & utSymGetHashValue(dvEnumGetSym(_Enum));
    nextEnum = dvModuleGetiEnumTable(Module, index);
    dvEnumSetNextTableModuleEnum(_Enum, nextEnum);
    dvModuleSetiEnumTable(Module, index, _Enum);
    dvModuleSetNumEnum(Module, dvModuleGetNumEnum(Module) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Enum from the hash table.
----------------------------------------------------------------------------------------*/
static void removeModuleEnumFromHashTable(
   dvModule Module,
   dvEnum _Enum)
{
    uint32 index = (dvModuleGetNumEnumTable(Module) - 1) & utSymGetHashValue(dvEnumGetSym(_Enum));
    dvEnum prevEnum, nextEnum;
    
    nextEnum = dvModuleGetiEnumTable(Module, index);
    if(nextEnum == _Enum) {
        dvModuleSetiEnumTable(Module, index, dvEnumGetNextTableModuleEnum(nextEnum));
    } else {
        do {
            prevEnum = nextEnum;
            nextEnum = dvEnumGetNextTableModuleEnum(nextEnum);
        } while(nextEnum != _Enum);
        dvEnumSetNextTableModuleEnum(prevEnum, dvEnumGetNextTableModuleEnum(_Enum));
    }
    dvModuleSetNumEnum(Module, dvModuleGetNumEnum(Module) - 1);
    dvEnumSetNextTableModuleEnum(_Enum, dvEnumNull);
}

/*----------------------------------------------------------------------------------------
  Find the Enum from the Module and its name.
----------------------------------------------------------------------------------------*/
dvEnum dvModuleFindEnum(
    dvModule Module,
    utSym sym)
{
    uint32 mask = dvModuleGetNumEnumTable(Module) - 1;
    dvEnum _Enum;

    if(mask + 1 != 0) {
        _Enum = dvModuleGetiEnumTable(Module, utSymGetHashValue(sym) & mask);
        while(_Enum != dvEnumNull) {
            if(dvEnumGetSym(_Enum) == sym) {
                return _Enum;
            }
            _Enum = dvEnumGetNextTableModuleEnum(_Enum);
        }
    }
    return dvEnumNull;
}

/*----------------------------------------------------------------------------------------
  Find the Enum from the Module and its name.
----------------------------------------------------------------------------------------*/
void dvModuleRenameEnum(
    dvModule Module,
    dvEnum _Enum,
    utSym sym)
{
    if(dvEnumGetSym(_Enum) != utSymNull) {
        removeModuleEnumFromHashTable(Module, _Enum);
    }
    dvEnumSetSym(_Enum, sym);
    if(sym != utSymNull) {
        addModuleEnumToHashTable(Module, _Enum);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Enum to the head of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleInsertEnum(
    dvModule Module,
    dvEnum _Enum)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(dvEnumGetModule(_Enum) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvEnumSetNextModuleEnum(_Enum, dvModuleGetFirstEnum(Module));
    if(dvModuleGetFirstEnum(Module) != dvEnumNull) {
        dvEnumSetPrevModuleEnum(dvModuleGetFirstEnum(Module), _Enum);
    }
    dvModuleSetFirstEnum(Module, _Enum);
    dvEnumSetPrevModuleEnum(_Enum, dvEnumNull);
    if(dvModuleGetLastEnum(Module) == dvEnumNull) {
        dvModuleSetLastEnum(Module, _Enum);
    }
    dvEnumSetModule(_Enum, Module);
    if(dvEnumGetSym(_Enum) != utSymNull) {
        addModuleEnumToHashTable(Module, _Enum);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Enum to the end of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleAppendEnum(
    dvModule Module,
    dvEnum _Enum)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(dvEnumGetModule(_Enum) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvEnumSetPrevModuleEnum(_Enum, dvModuleGetLastEnum(Module));
    if(dvModuleGetLastEnum(Module) != dvEnumNull) {
        dvEnumSetNextModuleEnum(dvModuleGetLastEnum(Module), _Enum);
    }
    dvModuleSetLastEnum(Module, _Enum);
    dvEnumSetNextModuleEnum(_Enum, dvEnumNull);
    if(dvModuleGetFirstEnum(Module) == dvEnumNull) {
        dvModuleSetFirstEnum(Module, _Enum);
    }
    dvEnumSetModule(_Enum, Module);
    if(dvEnumGetSym(_Enum) != utSymNull) {
        addModuleEnumToHashTable(Module, _Enum);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Enum to the Module after the previous Enum.
----------------------------------------------------------------------------------------*/
void dvModuleInsertAfterEnum(
    dvModule Module,
    dvEnum prevEnum,
    dvEnum _Enum)
{
    dvEnum nextEnum = dvEnumGetNextModuleEnum(prevEnum);

#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(dvEnumGetModule(_Enum) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvEnumSetNextModuleEnum(_Enum, nextEnum);
    dvEnumSetNextModuleEnum(prevEnum, _Enum);
    dvEnumSetPrevModuleEnum(_Enum, prevEnum);
    if(nextEnum != dvEnumNull) {
        dvEnumSetPrevModuleEnum(nextEnum, _Enum);
    }
    if(dvModuleGetLastEnum(Module) == prevEnum) {
        dvModuleSetLastEnum(Module, _Enum);
    }
    dvEnumSetModule(_Enum, Module);
    if(dvEnumGetSym(_Enum) != utSymNull) {
        addModuleEnumToHashTable(Module, _Enum);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Enum from the Module.
----------------------------------------------------------------------------------------*/
void dvModuleRemoveEnum(
    dvModule Module,
    dvEnum _Enum)
{
    dvEnum pEnum, nEnum;

#if defined(DD_DEBUG)
    if(_Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(dvEnumGetModule(_Enum) != dvModuleNull && dvEnumGetModule(_Enum) != Module) {
        utExit("Delete Enum from non-owning Module");
    }
#endif
    nEnum = dvEnumGetNextModuleEnum(_Enum);
    pEnum = dvEnumGetPrevModuleEnum(_Enum);
    if(pEnum != dvEnumNull) {
        dvEnumSetNextModuleEnum(pEnum, nEnum);
    } else if(dvModuleGetFirstEnum(Module) == _Enum) {
        dvModuleSetFirstEnum(Module, nEnum);
    }
    if(nEnum != dvEnumNull) {
        dvEnumSetPrevModuleEnum(nEnum, pEnum);
    } else if(dvModuleGetLastEnum(Module) == _Enum) {
        dvModuleSetLastEnum(Module, pEnum);
    }
    dvEnumSetNextModuleEnum(_Enum, dvEnumNull);
    dvEnumSetPrevModuleEnum(_Enum, dvEnumNull);
    dvEnumSetModule(_Enum, dvModuleNull);
    if(dvEnumGetSym(_Enum) != utSymNull) {
        removeModuleEnumFromHashTable(Module, _Enum);
    }
}

static void addModuleTypedefToHashTable(dvModule Module, dvTypedef _Typedef);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeModuleTypedefHashTable(
    dvModule Module)
{
    dvTypedef _Typedef;
    dvTypedef *Typedefs;
    uint32 numTypedefs = dvModuleGetNumTypedefTable(Module) << 1;

    if(numTypedefs == 0) {
        numTypedefs = 2;
        dvModuleAllocTypedefTables(Module, 2);
    } else {
        dvModuleResizeTypedefTables(Module, numTypedefs);
    }
    Typedefs = dvModuleGetTypedefTables(Module);
    /* Zero out the table */
    while(numTypedefs-- != 0) {
        *Typedefs++ = dvTypedefNull;
    }
    dvModuleSetNumTypedef(Module, 0);
    dvForeachModuleTypedef(Module, _Typedef) {
        if(dvTypedefGetSym(_Typedef) != utSymNull) {
            addModuleTypedefToHashTable(Module, _Typedef);
        }
    } dvEndForeachModuleTypedef;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Module.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addModuleTypedefToHashTable(
    dvModule Module,
    dvTypedef _Typedef)
{
    dvTypedef nextTypedef;
    uint32 index;

    if(dvModuleGetNumTypedef(Module) >= dvModuleGetNumTypedefTable(Module)) {
        resizeModuleTypedefHashTable(Module);
        return;
    }
    index = (dvModuleGetNumTypedefTable(Module) - 1) & utSymGetHashValue(dvTypedefGetSym(_Typedef));
    nextTypedef = dvModuleGetiTypedefTable(Module, index);
    dvTypedefSetNextTableModuleTypedef(_Typedef, nextTypedef);
    dvModuleSetiTypedefTable(Module, index, _Typedef);
    dvModuleSetNumTypedef(Module, dvModuleGetNumTypedef(Module) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Typedef from the hash table.
----------------------------------------------------------------------------------------*/
static void removeModuleTypedefFromHashTable(
   dvModule Module,
   dvTypedef _Typedef)
{
    uint32 index = (dvModuleGetNumTypedefTable(Module) - 1) & utSymGetHashValue(dvTypedefGetSym(_Typedef));
    dvTypedef prevTypedef, nextTypedef;
    
    nextTypedef = dvModuleGetiTypedefTable(Module, index);
    if(nextTypedef == _Typedef) {
        dvModuleSetiTypedefTable(Module, index, dvTypedefGetNextTableModuleTypedef(nextTypedef));
    } else {
        do {
            prevTypedef = nextTypedef;
            nextTypedef = dvTypedefGetNextTableModuleTypedef(nextTypedef);
        } while(nextTypedef != _Typedef);
        dvTypedefSetNextTableModuleTypedef(prevTypedef, dvTypedefGetNextTableModuleTypedef(_Typedef));
    }
    dvModuleSetNumTypedef(Module, dvModuleGetNumTypedef(Module) - 1);
    dvTypedefSetNextTableModuleTypedef(_Typedef, dvTypedefNull);
}

/*----------------------------------------------------------------------------------------
  Find the Typedef from the Module and its name.
----------------------------------------------------------------------------------------*/
dvTypedef dvModuleFindTypedef(
    dvModule Module,
    utSym sym)
{
    uint32 mask = dvModuleGetNumTypedefTable(Module) - 1;
    dvTypedef _Typedef;

    if(mask + 1 != 0) {
        _Typedef = dvModuleGetiTypedefTable(Module, utSymGetHashValue(sym) & mask);
        while(_Typedef != dvTypedefNull) {
            if(dvTypedefGetSym(_Typedef) == sym) {
                return _Typedef;
            }
            _Typedef = dvTypedefGetNextTableModuleTypedef(_Typedef);
        }
    }
    return dvTypedefNull;
}

/*----------------------------------------------------------------------------------------
  Find the Typedef from the Module and its name.
----------------------------------------------------------------------------------------*/
void dvModuleRenameTypedef(
    dvModule Module,
    dvTypedef _Typedef,
    utSym sym)
{
    if(dvTypedefGetSym(_Typedef) != utSymNull) {
        removeModuleTypedefFromHashTable(Module, _Typedef);
    }
    dvTypedefSetSym(_Typedef, sym);
    if(sym != utSymNull) {
        addModuleTypedefToHashTable(Module, _Typedef);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Typedef to the head of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleInsertTypedef(
    dvModule Module,
    dvTypedef _Typedef)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Typedef == dvTypedefNull) {
        utExit("Non-existent Typedef");
    }
    if(dvTypedefGetModule(_Typedef) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvTypedefSetNextModuleTypedef(_Typedef, dvModuleGetFirstTypedef(Module));
    if(dvModuleGetFirstTypedef(Module) != dvTypedefNull) {
        dvTypedefSetPrevModuleTypedef(dvModuleGetFirstTypedef(Module), _Typedef);
    }
    dvModuleSetFirstTypedef(Module, _Typedef);
    dvTypedefSetPrevModuleTypedef(_Typedef, dvTypedefNull);
    if(dvModuleGetLastTypedef(Module) == dvTypedefNull) {
        dvModuleSetLastTypedef(Module, _Typedef);
    }
    dvTypedefSetModule(_Typedef, Module);
    if(dvTypedefGetSym(_Typedef) != utSymNull) {
        addModuleTypedefToHashTable(Module, _Typedef);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Typedef to the end of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleAppendTypedef(
    dvModule Module,
    dvTypedef _Typedef)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Typedef == dvTypedefNull) {
        utExit("Non-existent Typedef");
    }
    if(dvTypedefGetModule(_Typedef) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvTypedefSetPrevModuleTypedef(_Typedef, dvModuleGetLastTypedef(Module));
    if(dvModuleGetLastTypedef(Module) != dvTypedefNull) {
        dvTypedefSetNextModuleTypedef(dvModuleGetLastTypedef(Module), _Typedef);
    }
    dvModuleSetLastTypedef(Module, _Typedef);
    dvTypedefSetNextModuleTypedef(_Typedef, dvTypedefNull);
    if(dvModuleGetFirstTypedef(Module) == dvTypedefNull) {
        dvModuleSetFirstTypedef(Module, _Typedef);
    }
    dvTypedefSetModule(_Typedef, Module);
    if(dvTypedefGetSym(_Typedef) != utSymNull) {
        addModuleTypedefToHashTable(Module, _Typedef);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Typedef to the Module after the previous Typedef.
----------------------------------------------------------------------------------------*/
void dvModuleInsertAfterTypedef(
    dvModule Module,
    dvTypedef prevTypedef,
    dvTypedef _Typedef)
{
    dvTypedef nextTypedef = dvTypedefGetNextModuleTypedef(prevTypedef);

#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Typedef == dvTypedefNull) {
        utExit("Non-existent Typedef");
    }
    if(dvTypedefGetModule(_Typedef) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvTypedefSetNextModuleTypedef(_Typedef, nextTypedef);
    dvTypedefSetNextModuleTypedef(prevTypedef, _Typedef);
    dvTypedefSetPrevModuleTypedef(_Typedef, prevTypedef);
    if(nextTypedef != dvTypedefNull) {
        dvTypedefSetPrevModuleTypedef(nextTypedef, _Typedef);
    }
    if(dvModuleGetLastTypedef(Module) == prevTypedef) {
        dvModuleSetLastTypedef(Module, _Typedef);
    }
    dvTypedefSetModule(_Typedef, Module);
    if(dvTypedefGetSym(_Typedef) != utSymNull) {
        addModuleTypedefToHashTable(Module, _Typedef);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Typedef from the Module.
----------------------------------------------------------------------------------------*/
void dvModuleRemoveTypedef(
    dvModule Module,
    dvTypedef _Typedef)
{
    dvTypedef pTypedef, nTypedef;

#if defined(DD_DEBUG)
    if(_Typedef == dvTypedefNull) {
        utExit("Non-existent Typedef");
    }
    if(dvTypedefGetModule(_Typedef) != dvModuleNull && dvTypedefGetModule(_Typedef) != Module) {
        utExit("Delete Typedef from non-owning Module");
    }
#endif
    nTypedef = dvTypedefGetNextModuleTypedef(_Typedef);
    pTypedef = dvTypedefGetPrevModuleTypedef(_Typedef);
    if(pTypedef != dvTypedefNull) {
        dvTypedefSetNextModuleTypedef(pTypedef, nTypedef);
    } else if(dvModuleGetFirstTypedef(Module) == _Typedef) {
        dvModuleSetFirstTypedef(Module, nTypedef);
    }
    if(nTypedef != dvTypedefNull) {
        dvTypedefSetPrevModuleTypedef(nTypedef, pTypedef);
    } else if(dvModuleGetLastTypedef(Module) == _Typedef) {
        dvModuleSetLastTypedef(Module, pTypedef);
    }
    dvTypedefSetNextModuleTypedef(_Typedef, dvTypedefNull);
    dvTypedefSetPrevModuleTypedef(_Typedef, dvTypedefNull);
    dvTypedefSetModule(_Typedef, dvModuleNull);
    if(dvTypedefGetSym(_Typedef) != utSymNull) {
        removeModuleTypedefFromHashTable(Module, _Typedef);
    }
}

static void addModuleSchemaToHashTable(dvModule Module, dvSchema _Schema);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeModuleSchemaHashTable(
    dvModule Module)
{
    dvSchema _Schema;
    dvSchema *Schemas;
    uint32 numSchemas = dvModuleGetNumSchemaTable(Module) << 1;

    if(numSchemas == 0) {
        numSchemas = 2;
        dvModuleAllocSchemaTables(Module, 2);
    } else {
        dvModuleResizeSchemaTables(Module, numSchemas);
    }
    Schemas = dvModuleGetSchemaTables(Module);
    /* Zero out the table */
    while(numSchemas-- != 0) {
        *Schemas++ = dvSchemaNull;
    }
    dvModuleSetNumSchema(Module, 0);
    dvForeachModuleSchema(Module, _Schema) {
        if(dvSchemaGetSym(_Schema) != utSymNull) {
            addModuleSchemaToHashTable(Module, _Schema);
        }
    } dvEndForeachModuleSchema;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Module.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addModuleSchemaToHashTable(
    dvModule Module,
    dvSchema _Schema)
{
    dvSchema nextSchema;
    uint32 index;

    if(dvModuleGetNumSchema(Module) >= dvModuleGetNumSchemaTable(Module)) {
        resizeModuleSchemaHashTable(Module);
        return;
    }
    index = (dvModuleGetNumSchemaTable(Module) - 1) & utSymGetHashValue(dvSchemaGetSym(_Schema));
    nextSchema = dvModuleGetiSchemaTable(Module, index);
    dvSchemaSetNextTableModuleSchema(_Schema, nextSchema);
    dvModuleSetiSchemaTable(Module, index, _Schema);
    dvModuleSetNumSchema(Module, dvModuleGetNumSchema(Module) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Schema from the hash table.
----------------------------------------------------------------------------------------*/
static void removeModuleSchemaFromHashTable(
   dvModule Module,
   dvSchema _Schema)
{
    uint32 index = (dvModuleGetNumSchemaTable(Module) - 1) & utSymGetHashValue(dvSchemaGetSym(_Schema));
    dvSchema prevSchema, nextSchema;
    
    nextSchema = dvModuleGetiSchemaTable(Module, index);
    if(nextSchema == _Schema) {
        dvModuleSetiSchemaTable(Module, index, dvSchemaGetNextTableModuleSchema(nextSchema));
    } else {
        do {
            prevSchema = nextSchema;
            nextSchema = dvSchemaGetNextTableModuleSchema(nextSchema);
        } while(nextSchema != _Schema);
        dvSchemaSetNextTableModuleSchema(prevSchema, dvSchemaGetNextTableModuleSchema(_Schema));
    }
    dvModuleSetNumSchema(Module, dvModuleGetNumSchema(Module) - 1);
    dvSchemaSetNextTableModuleSchema(_Schema, dvSchemaNull);
}

/*----------------------------------------------------------------------------------------
  Find the Schema from the Module and its name.
----------------------------------------------------------------------------------------*/
dvSchema dvModuleFindSchema(
    dvModule Module,
    utSym sym)
{
    uint32 mask = dvModuleGetNumSchemaTable(Module) - 1;
    dvSchema _Schema;

    if(mask + 1 != 0) {
        _Schema = dvModuleGetiSchemaTable(Module, utSymGetHashValue(sym) & mask);
        while(_Schema != dvSchemaNull) {
            if(dvSchemaGetSym(_Schema) == sym) {
                return _Schema;
            }
            _Schema = dvSchemaGetNextTableModuleSchema(_Schema);
        }
    }
    return dvSchemaNull;
}

/*----------------------------------------------------------------------------------------
  Find the Schema from the Module and its name.
----------------------------------------------------------------------------------------*/
void dvModuleRenameSchema(
    dvModule Module,
    dvSchema _Schema,
    utSym sym)
{
    if(dvSchemaGetSym(_Schema) != utSymNull) {
        removeModuleSchemaFromHashTable(Module, _Schema);
    }
    dvSchemaSetSym(_Schema, sym);
    if(sym != utSymNull) {
        addModuleSchemaToHashTable(Module, _Schema);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Schema to the head of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleInsertSchema(
    dvModule Module,
    dvSchema _Schema)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(dvSchemaGetModule(_Schema) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvSchemaSetNextModuleSchema(_Schema, dvModuleGetFirstSchema(Module));
    if(dvModuleGetFirstSchema(Module) != dvSchemaNull) {
        dvSchemaSetPrevModuleSchema(dvModuleGetFirstSchema(Module), _Schema);
    }
    dvModuleSetFirstSchema(Module, _Schema);
    dvSchemaSetPrevModuleSchema(_Schema, dvSchemaNull);
    if(dvModuleGetLastSchema(Module) == dvSchemaNull) {
        dvModuleSetLastSchema(Module, _Schema);
    }
    dvSchemaSetModule(_Schema, Module);
    if(dvSchemaGetSym(_Schema) != utSymNull) {
        addModuleSchemaToHashTable(Module, _Schema);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Schema to the end of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleAppendSchema(
    dvModule Module,
    dvSchema _Schema)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(dvSchemaGetModule(_Schema) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvSchemaSetPrevModuleSchema(_Schema, dvModuleGetLastSchema(Module));
    if(dvModuleGetLastSchema(Module) != dvSchemaNull) {
        dvSchemaSetNextModuleSchema(dvModuleGetLastSchema(Module), _Schema);
    }
    dvModuleSetLastSchema(Module, _Schema);
    dvSchemaSetNextModuleSchema(_Schema, dvSchemaNull);
    if(dvModuleGetFirstSchema(Module) == dvSchemaNull) {
        dvModuleSetFirstSchema(Module, _Schema);
    }
    dvSchemaSetModule(_Schema, Module);
    if(dvSchemaGetSym(_Schema) != utSymNull) {
        addModuleSchemaToHashTable(Module, _Schema);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Schema to the Module after the previous Schema.
----------------------------------------------------------------------------------------*/
void dvModuleInsertAfterSchema(
    dvModule Module,
    dvSchema prevSchema,
    dvSchema _Schema)
{
    dvSchema nextSchema = dvSchemaGetNextModuleSchema(prevSchema);

#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(dvSchemaGetModule(_Schema) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvSchemaSetNextModuleSchema(_Schema, nextSchema);
    dvSchemaSetNextModuleSchema(prevSchema, _Schema);
    dvSchemaSetPrevModuleSchema(_Schema, prevSchema);
    if(nextSchema != dvSchemaNull) {
        dvSchemaSetPrevModuleSchema(nextSchema, _Schema);
    }
    if(dvModuleGetLastSchema(Module) == prevSchema) {
        dvModuleSetLastSchema(Module, _Schema);
    }
    dvSchemaSetModule(_Schema, Module);
    if(dvSchemaGetSym(_Schema) != utSymNull) {
        addModuleSchemaToHashTable(Module, _Schema);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Schema from the Module.
----------------------------------------------------------------------------------------*/
void dvModuleRemoveSchema(
    dvModule Module,
    dvSchema _Schema)
{
    dvSchema pSchema, nSchema;

#if defined(DD_DEBUG)
    if(_Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(dvSchemaGetModule(_Schema) != dvModuleNull && dvSchemaGetModule(_Schema) != Module) {
        utExit("Delete Schema from non-owning Module");
    }
#endif
    nSchema = dvSchemaGetNextModuleSchema(_Schema);
    pSchema = dvSchemaGetPrevModuleSchema(_Schema);
    if(pSchema != dvSchemaNull) {
        dvSchemaSetNextModuleSchema(pSchema, nSchema);
    } else if(dvModuleGetFirstSchema(Module) == _Schema) {
        dvModuleSetFirstSchema(Module, nSchema);
    }
    if(nSchema != dvSchemaNull) {
        dvSchemaSetPrevModuleSchema(nSchema, pSchema);
    } else if(dvModuleGetLastSchema(Module) == _Schema) {
        dvModuleSetLastSchema(Module, pSchema);
    }
    dvSchemaSetNextModuleSchema(_Schema, dvSchemaNull);
    dvSchemaSetPrevModuleSchema(_Schema, dvSchemaNull);
    dvSchemaSetModule(_Schema, dvModuleNull);
    if(dvSchemaGetSym(_Schema) != utSymNull) {
        removeModuleSchemaFromHashTable(Module, _Schema);
    }
}

/*----------------------------------------------------------------------------------------
  Add the ImportLink to the head of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleInsertImportLink(
    dvModule Module,
    dvLink _Link)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetImportModule(_Link) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvLinkSetNextModuleImportLink(_Link, dvModuleGetFirstImportLink(Module));
    dvModuleSetFirstImportLink(Module, _Link);
    if(dvModuleGetLastImportLink(Module) == dvLinkNull) {
        dvModuleSetLastImportLink(Module, _Link);
    }
    dvLinkSetImportModule(_Link, Module);
}

/*----------------------------------------------------------------------------------------
  Add the ImportLink to the end of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleAppendImportLink(
    dvModule Module,
    dvLink _Link)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetImportModule(_Link) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    if(dvModuleGetLastImportLink(Module) != dvLinkNull) {
        dvLinkSetNextModuleImportLink(dvModuleGetLastImportLink(Module), _Link);
    } else {
        dvModuleSetFirstImportLink(Module, _Link);
    }
    dvModuleSetLastImportLink(Module, _Link);
    dvLinkSetNextModuleImportLink(_Link, dvLinkNull);
    dvLinkSetImportModule(_Link, Module);
}

/*----------------------------------------------------------------------------------------
  Insert the ImportLink to the Module after the previous ImportLink.
----------------------------------------------------------------------------------------*/
void dvModuleInsertAfterImportLink(
    dvModule Module,
    dvLink prevLink,
    dvLink _Link)
{
    dvLink nextLink = dvLinkGetNextModuleImportLink(prevLink);

#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetImportModule(_Link) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvLinkSetNextModuleImportLink(_Link, nextLink);
    dvLinkSetNextModuleImportLink(prevLink, _Link);
    if(dvModuleGetLastImportLink(Module) == prevLink) {
        dvModuleSetLastImportLink(Module, _Link);
    }
    dvLinkSetImportModule(_Link, Module);
}

/*----------------------------------------------------------------------------------------
 Remove the ImportLink from the Module.
----------------------------------------------------------------------------------------*/
void dvModuleRemoveImportLink(
    dvModule Module,
    dvLink _Link)
{
    dvLink pLink, nLink;

#if defined(DD_DEBUG)
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetImportModule(_Link) != dvModuleNull && dvLinkGetImportModule(_Link) != Module) {
        utExit("Delete Link from non-owning Module");
    }
#endif
    pLink = dvLinkNull;
    for(nLink = dvModuleGetFirstImportLink(Module); nLink != dvLinkNull && nLink != _Link;
            nLink = dvLinkGetNextModuleImportLink(nLink)) {
        pLink = nLink;
    }
    if(pLink != dvLinkNull) {
        dvLinkSetNextModuleImportLink(pLink, dvLinkGetNextModuleImportLink(_Link));
    } else {
        dvModuleSetFirstImportLink(Module, dvLinkGetNextModuleImportLink(_Link));
    }
    dvLinkSetNextModuleImportLink(_Link, dvLinkNull);
    if(dvModuleGetLastImportLink(Module) == _Link) {
        dvModuleSetLastImportLink(Module, pLink);
    }
    dvLinkSetImportModule(_Link, dvModuleNull);
}

/*----------------------------------------------------------------------------------------
  Add the ExportLink to the head of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleInsertExportLink(
    dvModule Module,
    dvLink _Link)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetExportModule(_Link) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvLinkSetNextModuleExportLink(_Link, dvModuleGetFirstExportLink(Module));
    dvModuleSetFirstExportLink(Module, _Link);
    if(dvModuleGetLastExportLink(Module) == dvLinkNull) {
        dvModuleSetLastExportLink(Module, _Link);
    }
    dvLinkSetExportModule(_Link, Module);
}

/*----------------------------------------------------------------------------------------
  Add the ExportLink to the end of the list on the Module.
----------------------------------------------------------------------------------------*/
void dvModuleAppendExportLink(
    dvModule Module,
    dvLink _Link)
{
#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetExportModule(_Link) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    if(dvModuleGetLastExportLink(Module) != dvLinkNull) {
        dvLinkSetNextModuleExportLink(dvModuleGetLastExportLink(Module), _Link);
    } else {
        dvModuleSetFirstExportLink(Module, _Link);
    }
    dvModuleSetLastExportLink(Module, _Link);
    dvLinkSetNextModuleExportLink(_Link, dvLinkNull);
    dvLinkSetExportModule(_Link, Module);
}

/*----------------------------------------------------------------------------------------
  Insert the ExportLink to the Module after the previous ExportLink.
----------------------------------------------------------------------------------------*/
void dvModuleInsertAfterExportLink(
    dvModule Module,
    dvLink prevLink,
    dvLink _Link)
{
    dvLink nextLink = dvLinkGetNextModuleExportLink(prevLink);

#if defined(DD_DEBUG)
    if(Module == dvModuleNull) {
        utExit("Non-existent Module");
    }
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetExportModule(_Link) != dvModuleNull) {
        utExit("Attempting to add dv to Module twice");
    }
#endif
    dvLinkSetNextModuleExportLink(_Link, nextLink);
    dvLinkSetNextModuleExportLink(prevLink, _Link);
    if(dvModuleGetLastExportLink(Module) == prevLink) {
        dvModuleSetLastExportLink(Module, _Link);
    }
    dvLinkSetExportModule(_Link, Module);
}

/*----------------------------------------------------------------------------------------
 Remove the ExportLink from the Module.
----------------------------------------------------------------------------------------*/
void dvModuleRemoveExportLink(
    dvModule Module,
    dvLink _Link)
{
    dvLink pLink, nLink;

#if defined(DD_DEBUG)
    if(_Link == dvLinkNull) {
        utExit("Non-existent Link");
    }
    if(dvLinkGetExportModule(_Link) != dvModuleNull && dvLinkGetExportModule(_Link) != Module) {
        utExit("Delete Link from non-owning Module");
    }
#endif
    pLink = dvLinkNull;
    for(nLink = dvModuleGetFirstExportLink(Module); nLink != dvLinkNull && nLink != _Link;
            nLink = dvLinkGetNextModuleExportLink(nLink)) {
        pLink = nLink;
    }
    if(pLink != dvLinkNull) {
        dvLinkSetNextModuleExportLink(pLink, dvLinkGetNextModuleExportLink(_Link));
    } else {
        dvModuleSetFirstExportLink(Module, dvLinkGetNextModuleExportLink(_Link));
    }
    dvLinkSetNextModuleExportLink(_Link, dvLinkNull);
    if(dvModuleGetLastExportLink(Module) == _Link) {
        dvModuleSetLastExportLink(Module, pLink);
    }
    dvLinkSetExportModule(_Link, dvModuleNull);
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowModule(
    dvModule Module)
{
    utDatabaseShowObject("dv", "Module", dvModule2Index(Module));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocLink(void)
{
    dvLink Link = dvLinkAlloc();

    return dvLink2Index(Link);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Link.
----------------------------------------------------------------------------------------*/
static void allocLinks(void)
{
    dvSetAllocatedLink(2);
    dvSetUsedLink(0);
    dvLinks.ImportModule = utNewA(dvModule, (dvAllocatedLink()));
    dvLinks.NextModuleImportLink = utNewA(dvLink, (dvAllocatedLink()));
    dvLinks.ExportModule = utNewA(dvModule, (dvAllocatedLink()));
    dvLinks.NextModuleExportLink = utNewA(dvLink, (dvAllocatedLink()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Link.
----------------------------------------------------------------------------------------*/
static void reallocLinks(
    uint32 newSize)
{
    utResizeArray(dvLinks.ImportModule, (newSize));
    utResizeArray(dvLinks.NextModuleImportLink, (newSize));
    utResizeArray(dvLinks.ExportModule, (newSize));
    utResizeArray(dvLinks.NextModuleExportLink, (newSize));
    dvSetAllocatedLink(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Links.
----------------------------------------------------------------------------------------*/
void dvLinkAllocMore(void)
{
    reallocLinks(dvAllocatedLink() + (dvAllocatedLink() >> 1));
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowLink(
    dvLink Link)
{
    utDatabaseShowObject("dv", "Link", dvLink2Index(Link));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocSchema(void)
{
    dvSchema Schema = dvSchemaAlloc();

    return dvSchema2Index(Schema);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Schema.
----------------------------------------------------------------------------------------*/
static void allocSchemas(void)
{
    dvSetAllocatedSchema(2);
    dvSetUsedSchema(0);
    dvSchemas.Module = utNewA(dvModule, (dvAllocatedSchema()));
    dvSchemas.NextModuleSchema = utNewA(dvSchema, (dvAllocatedSchema()));
    dvSchemas.PrevModuleSchema = utNewA(dvSchema, (dvAllocatedSchema()));
    dvSchemas.Sym = utNewA(utSym, (dvAllocatedSchema()));
    dvSchemas.NextTableModuleSchema = utNewA(dvSchema, (dvAllocatedSchema()));
    dvSchemas.FirstClass = utNewA(dvClass, (dvAllocatedSchema()));
    dvSchemas.LastClass = utNewA(dvClass, (dvAllocatedSchema()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Schema.
----------------------------------------------------------------------------------------*/
static void reallocSchemas(
    uint32 newSize)
{
    utResizeArray(dvSchemas.Module, (newSize));
    utResizeArray(dvSchemas.NextModuleSchema, (newSize));
    utResizeArray(dvSchemas.PrevModuleSchema, (newSize));
    utResizeArray(dvSchemas.Sym, (newSize));
    utResizeArray(dvSchemas.NextTableModuleSchema, (newSize));
    utResizeArray(dvSchemas.FirstClass, (newSize));
    utResizeArray(dvSchemas.LastClass, (newSize));
    dvSetAllocatedSchema(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Schemas.
----------------------------------------------------------------------------------------*/
void dvSchemaAllocMore(void)
{
    reallocSchemas(dvAllocatedSchema() + (dvAllocatedSchema() >> 1));
}

/*----------------------------------------------------------------------------------------
  Add the Class to the head of the list on the Schema.
----------------------------------------------------------------------------------------*/
void dvSchemaInsertClass(
    dvSchema Schema,
    dvClass _Class)
{
#if defined(DD_DEBUG)
    if(Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetSchema(_Class) != dvSchemaNull) {
        utExit("Attempting to add dv to Schema twice");
    }
#endif
    dvClassSetNextSchemaClass(_Class, dvSchemaGetFirstClass(Schema));
    dvSchemaSetFirstClass(Schema, _Class);
    if(dvSchemaGetLastClass(Schema) == dvClassNull) {
        dvSchemaSetLastClass(Schema, _Class);
    }
    dvClassSetSchema(_Class, Schema);
}

/*----------------------------------------------------------------------------------------
  Add the Class to the end of the list on the Schema.
----------------------------------------------------------------------------------------*/
void dvSchemaAppendClass(
    dvSchema Schema,
    dvClass _Class)
{
#if defined(DD_DEBUG)
    if(Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetSchema(_Class) != dvSchemaNull) {
        utExit("Attempting to add dv to Schema twice");
    }
#endif
    if(dvSchemaGetLastClass(Schema) != dvClassNull) {
        dvClassSetNextSchemaClass(dvSchemaGetLastClass(Schema), _Class);
    } else {
        dvSchemaSetFirstClass(Schema, _Class);
    }
    dvSchemaSetLastClass(Schema, _Class);
    dvClassSetNextSchemaClass(_Class, dvClassNull);
    dvClassSetSchema(_Class, Schema);
}

/*----------------------------------------------------------------------------------------
  Insert the Class to the Schema after the previous Class.
----------------------------------------------------------------------------------------*/
void dvSchemaInsertAfterClass(
    dvSchema Schema,
    dvClass prevClass,
    dvClass _Class)
{
    dvClass nextClass = dvClassGetNextSchemaClass(prevClass);

#if defined(DD_DEBUG)
    if(Schema == dvSchemaNull) {
        utExit("Non-existent Schema");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetSchema(_Class) != dvSchemaNull) {
        utExit("Attempting to add dv to Schema twice");
    }
#endif
    dvClassSetNextSchemaClass(_Class, nextClass);
    dvClassSetNextSchemaClass(prevClass, _Class);
    if(dvSchemaGetLastClass(Schema) == prevClass) {
        dvSchemaSetLastClass(Schema, _Class);
    }
    dvClassSetSchema(_Class, Schema);
}

/*----------------------------------------------------------------------------------------
 Remove the Class from the Schema.
----------------------------------------------------------------------------------------*/
void dvSchemaRemoveClass(
    dvSchema Schema,
    dvClass _Class)
{
    dvClass pClass, nClass;

#if defined(DD_DEBUG)
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetSchema(_Class) != dvSchemaNull && dvClassGetSchema(_Class) != Schema) {
        utExit("Delete Class from non-owning Schema");
    }
#endif
    pClass = dvClassNull;
    for(nClass = dvSchemaGetFirstClass(Schema); nClass != dvClassNull && nClass != _Class;
            nClass = dvClassGetNextSchemaClass(nClass)) {
        pClass = nClass;
    }
    if(pClass != dvClassNull) {
        dvClassSetNextSchemaClass(pClass, dvClassGetNextSchemaClass(_Class));
    } else {
        dvSchemaSetFirstClass(Schema, dvClassGetNextSchemaClass(_Class));
    }
    dvClassSetNextSchemaClass(_Class, dvClassNull);
    if(dvSchemaGetLastClass(Schema) == _Class) {
        dvSchemaSetLastClass(Schema, pClass);
    }
    dvClassSetSchema(_Class, dvSchemaNull);
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowSchema(
    dvSchema Schema)
{
    utDatabaseShowObject("dv", "Schema", dvSchema2Index(Schema));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocEnum(void)
{
    dvEnum Enum = dvEnumAlloc();

    return dvEnum2Index(Enum);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Enum.
----------------------------------------------------------------------------------------*/
static void allocEnums(void)
{
    dvSetAllocatedEnum(2);
    dvSetUsedEnum(0);
    dvEnums.PrefixSym = utNewA(utSym, (dvAllocatedEnum()));
    dvEnums.NumEntries = utNewA(uint16, (dvAllocatedEnum()));
    dvEnums.Module = utNewA(dvModule, (dvAllocatedEnum()));
    dvEnums.NextModuleEnum = utNewA(dvEnum, (dvAllocatedEnum()));
    dvEnums.PrevModuleEnum = utNewA(dvEnum, (dvAllocatedEnum()));
    dvEnums.Sym = utNewA(utSym, (dvAllocatedEnum()));
    dvEnums.NextTableModuleEnum = utNewA(dvEnum, (dvAllocatedEnum()));
    dvEnums.FirstEntry = utNewA(dvEntry, (dvAllocatedEnum()));
    dvEnums.LastEntry = utNewA(dvEntry, (dvAllocatedEnum()));
    dvEnums.EntryTableIndex = utNewA(uint32, (dvAllocatedEnum()));
    dvEnums.NumEntryTable = utNewA(uint32, (dvAllocatedEnum()));
    dvSetUsedEnumEntryTable(0);
    dvSetAllocatedEnumEntryTable(2);
    dvSetFreeEnumEntryTable(0);
    dvEnums.EntryTable = utNewA(dvEntry, dvAllocatedEnumEntryTable());
    dvEnums.NumEntry = utNewA(uint32, (dvAllocatedEnum()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Enum.
----------------------------------------------------------------------------------------*/
static void reallocEnums(
    uint32 newSize)
{
    utResizeArray(dvEnums.PrefixSym, (newSize));
    utResizeArray(dvEnums.NumEntries, (newSize));
    utResizeArray(dvEnums.Module, (newSize));
    utResizeArray(dvEnums.NextModuleEnum, (newSize));
    utResizeArray(dvEnums.PrevModuleEnum, (newSize));
    utResizeArray(dvEnums.Sym, (newSize));
    utResizeArray(dvEnums.NextTableModuleEnum, (newSize));
    utResizeArray(dvEnums.FirstEntry, (newSize));
    utResizeArray(dvEnums.LastEntry, (newSize));
    utResizeArray(dvEnums.EntryTableIndex, (newSize));
    utResizeArray(dvEnums.NumEntryTable, (newSize));
    utResizeArray(dvEnums.NumEntry, (newSize));
    dvSetAllocatedEnum(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Enums.
----------------------------------------------------------------------------------------*/
void dvEnumAllocMore(void)
{
    reallocEnums(dvAllocatedEnum() + (dvAllocatedEnum() >> 1));
}

/*----------------------------------------------------------------------------------------
  Compact the Enum.EntryTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactEnumEntryTables(void)
{
    uint32 elementSize = sizeof(dvEntry);
    uint32 usedHeaderSize = (sizeof(dvEnum) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvEnum) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvEntry *toPtr = dvEnums.EntryTable;
    dvEntry *fromPtr = toPtr;
    dvEnum Enum;
    uint32 size;

    while(fromPtr < dvEnums.EntryTable + dvUsedEnumEntryTable()) {
        Enum = *(dvEnum *)(void *)fromPtr;
        if(Enum != dvEnumNull) {
            /* Need to move it to toPtr */
            size = utMax(dvEnumGetNumEntryTable(Enum) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvEnumSetEntryTableIndex(Enum, toPtr - dvEnums.EntryTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvEnum *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedEnumEntryTable(toPtr - dvEnums.EntryTable);
    dvSetFreeEnumEntryTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Enum.EntryTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreEnumEntryTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedEnumEntryTable() - dvUsedEnumEntryTable();

    if((dvFreeEnumEntryTable() << 2) > dvUsedEnumEntryTable()) {
        dvCompactEnumEntryTables();
        freeSpace = dvAllocatedEnumEntryTable() - dvUsedEnumEntryTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedEnumEntryTable(dvAllocatedEnumEntryTable() + spaceNeeded - freeSpace +
            (dvAllocatedEnumEntryTable() >> 2));
        utResizeArray(dvEnums.EntryTable, dvAllocatedEnumEntryTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Enum.EntryTable array.
----------------------------------------------------------------------------------------*/
void dvEnumAllocEntryTables(
    dvEnum Enum,
    uint32 numEntryTables)
{
    uint32 freeSpace = dvAllocatedEnumEntryTable() - dvUsedEnumEntryTable();
    uint32 elementSize = sizeof(dvEntry);
    uint32 usedHeaderSize = (sizeof(dvEnum) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvEnum) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numEntryTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvEnumGetNumEntryTable(Enum) == 0);
#endif
    if(numEntryTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreEnumEntryTables(spaceNeeded);
    }
    dvEnumSetEntryTableIndex(Enum, dvUsedEnumEntryTable() + usedHeaderSize);
    dvEnumSetNumEntryTable(Enum, numEntryTables);
    *(dvEnum *)(void *)(dvEnums.EntryTable + dvUsedEnumEntryTable()) = Enum;
    memset(dvEnumGetEntryTables(Enum), 0xff, numEntryTables*elementSize);
    dvSetUsedEnumEntryTable(dvUsedEnumEntryTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvEnumGetEntryTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getEnumEntryTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvEnum Enum = dvIndex2Enum(objectNumber);

    *numValues = dvEnumGetNumEntryTable(Enum);
    return dvEnumGetEntryTables(Enum);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvEnumAllocEntryTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocEnumEntryTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvEnum Enum = dvIndex2Enum(objectNumber);

    dvEnumSetEntryTableIndex(Enum, 0);
    dvEnumSetNumEntryTable(Enum, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvEnumAllocEntryTables(Enum, numValues);
    return dvEnumGetEntryTables(Enum);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Enum.EntryTable array.
----------------------------------------------------------------------------------------*/
void dvEnumFreeEntryTables(
    dvEnum Enum)
{
    uint32 elementSize = sizeof(dvEntry);
    uint32 usedHeaderSize = (sizeof(dvEnum) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvEnum) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvEnumGetNumEntryTable(Enum) + usedHeaderSize, freeHeaderSize);
    dvEntry *dataPtr = dvEnumGetEntryTables(Enum) - usedHeaderSize;

    if(dvEnumGetNumEntryTable(Enum) == 0) {
        return;
    }
    *(dvEnum *)(void *)(dataPtr) = dvEnumNull;
    *(uint32 *)(void *)(((dvEnum *)(void *)dataPtr) + 1) = size;
    dvEnumSetNumEntryTable(Enum, 0);
    dvSetFreeEnumEntryTable(dvFreeEnumEntryTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Enum.EntryTable array.
----------------------------------------------------------------------------------------*/
void dvEnumResizeEntryTables(
    dvEnum Enum,
    uint32 numEntryTables)
{
    uint32 freeSpace = dvAllocatedEnumEntryTable() - dvUsedEnumEntryTable();
    uint32 elementSize = sizeof(dvEntry);
    uint32 usedHeaderSize = (sizeof(dvEnum) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvEnum) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numEntryTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvEnumGetNumEntryTable(Enum) + usedHeaderSize, freeHeaderSize);
    dvEntry *dataPtr;

    if(numEntryTables == 0) {
        if(dvEnumGetNumEntryTable(Enum) != 0) {
            dvEnumFreeEntryTables(Enum);
        };
        return;
    }
    if(dvEnumGetNumEntryTable(Enum) == 0) {
        dvEnumAllocEntryTables(Enum, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreEnumEntryTables(newSize);
    }
    dataPtr = dvEnumGetEntryTables(Enum) - usedHeaderSize;
    memcpy((void *)(dvEnums.EntryTable + dvUsedEnumEntryTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvEnums.EntryTable + dvUsedEnumEntryTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvEnum *)(void *)dataPtr = dvEnumNull;
    *(uint32 *)(void *)(((dvEnum *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeEnumEntryTable(dvFreeEnumEntryTable() + oldSize);
    dvEnumSetEntryTableIndex(Enum, dvUsedEnumEntryTable() + usedHeaderSize);
    dvEnumSetNumEntryTable(Enum, numEntryTables);
    dvSetUsedEnumEntryTable(dvUsedEnumEntryTable() + newSize);
}

static void addEnumEntryToHashTable(dvEnum Enum, dvEntry _Entry);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeEnumEntryHashTable(
    dvEnum Enum)
{
    dvEntry _Entry;
    dvEntry *Entrys;
    uint32 numEntrys = dvEnumGetNumEntryTable(Enum) << 1;

    if(numEntrys == 0) {
        numEntrys = 2;
        dvEnumAllocEntryTables(Enum, 2);
    } else {
        dvEnumResizeEntryTables(Enum, numEntrys);
    }
    Entrys = dvEnumGetEntryTables(Enum);
    /* Zero out the table */
    while(numEntrys-- != 0) {
        *Entrys++ = dvEntryNull;
    }
    dvEnumSetNumEntry(Enum, 0);
    dvForeachEnumEntry(Enum, _Entry) {
        if(dvEntryGetSym(_Entry) != utSymNull) {
            addEnumEntryToHashTable(Enum, _Entry);
        }
    } dvEndForeachEnumEntry;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Enum.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addEnumEntryToHashTable(
    dvEnum Enum,
    dvEntry _Entry)
{
    dvEntry nextEntry;
    uint32 index;

    if(dvEnumGetNumEntry(Enum) >= dvEnumGetNumEntryTable(Enum)) {
        resizeEnumEntryHashTable(Enum);
        return;
    }
    index = (dvEnumGetNumEntryTable(Enum) - 1) & utSymGetHashValue(dvEntryGetSym(_Entry));
    nextEntry = dvEnumGetiEntryTable(Enum, index);
    dvEntrySetNextTableEnumEntry(_Entry, nextEntry);
    dvEnumSetiEntryTable(Enum, index, _Entry);
    dvEnumSetNumEntry(Enum, dvEnumGetNumEntry(Enum) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Entry from the hash table.
----------------------------------------------------------------------------------------*/
static void removeEnumEntryFromHashTable(
   dvEnum Enum,
   dvEntry _Entry)
{
    uint32 index = (dvEnumGetNumEntryTable(Enum) - 1) & utSymGetHashValue(dvEntryGetSym(_Entry));
    dvEntry prevEntry, nextEntry;
    
    nextEntry = dvEnumGetiEntryTable(Enum, index);
    if(nextEntry == _Entry) {
        dvEnumSetiEntryTable(Enum, index, dvEntryGetNextTableEnumEntry(nextEntry));
    } else {
        do {
            prevEntry = nextEntry;
            nextEntry = dvEntryGetNextTableEnumEntry(nextEntry);
        } while(nextEntry != _Entry);
        dvEntrySetNextTableEnumEntry(prevEntry, dvEntryGetNextTableEnumEntry(_Entry));
    }
    dvEnumSetNumEntry(Enum, dvEnumGetNumEntry(Enum) - 1);
    dvEntrySetNextTableEnumEntry(_Entry, dvEntryNull);
}

/*----------------------------------------------------------------------------------------
  Find the Entry from the Enum and its name.
----------------------------------------------------------------------------------------*/
dvEntry dvEnumFindEntry(
    dvEnum Enum,
    utSym sym)
{
    uint32 mask = dvEnumGetNumEntryTable(Enum) - 1;
    dvEntry _Entry;

    if(mask + 1 != 0) {
        _Entry = dvEnumGetiEntryTable(Enum, utSymGetHashValue(sym) & mask);
        while(_Entry != dvEntryNull) {
            if(dvEntryGetSym(_Entry) == sym) {
                return _Entry;
            }
            _Entry = dvEntryGetNextTableEnumEntry(_Entry);
        }
    }
    return dvEntryNull;
}

/*----------------------------------------------------------------------------------------
  Find the Entry from the Enum and its name.
----------------------------------------------------------------------------------------*/
void dvEnumRenameEntry(
    dvEnum Enum,
    dvEntry _Entry,
    utSym sym)
{
    if(dvEntryGetSym(_Entry) != utSymNull) {
        removeEnumEntryFromHashTable(Enum, _Entry);
    }
    dvEntrySetSym(_Entry, sym);
    if(sym != utSymNull) {
        addEnumEntryToHashTable(Enum, _Entry);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Entry to the head of the list on the Enum.
----------------------------------------------------------------------------------------*/
void dvEnumInsertEntry(
    dvEnum Enum,
    dvEntry _Entry)
{
#if defined(DD_DEBUG)
    if(Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(_Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(dvEntryGetEnum(_Entry) != dvEnumNull) {
        utExit("Attempting to add dv to Enum twice");
    }
#endif
    dvEntrySetNextEnumEntry(_Entry, dvEnumGetFirstEntry(Enum));
    if(dvEnumGetFirstEntry(Enum) != dvEntryNull) {
        dvEntrySetPrevEnumEntry(dvEnumGetFirstEntry(Enum), _Entry);
    }
    dvEnumSetFirstEntry(Enum, _Entry);
    dvEntrySetPrevEnumEntry(_Entry, dvEntryNull);
    if(dvEnumGetLastEntry(Enum) == dvEntryNull) {
        dvEnumSetLastEntry(Enum, _Entry);
    }
    dvEntrySetEnum(_Entry, Enum);
    if(dvEntryGetSym(_Entry) != utSymNull) {
        addEnumEntryToHashTable(Enum, _Entry);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Entry to the end of the list on the Enum.
----------------------------------------------------------------------------------------*/
void dvEnumAppendEntry(
    dvEnum Enum,
    dvEntry _Entry)
{
#if defined(DD_DEBUG)
    if(Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(_Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(dvEntryGetEnum(_Entry) != dvEnumNull) {
        utExit("Attempting to add dv to Enum twice");
    }
#endif
    dvEntrySetPrevEnumEntry(_Entry, dvEnumGetLastEntry(Enum));
    if(dvEnumGetLastEntry(Enum) != dvEntryNull) {
        dvEntrySetNextEnumEntry(dvEnumGetLastEntry(Enum), _Entry);
    }
    dvEnumSetLastEntry(Enum, _Entry);
    dvEntrySetNextEnumEntry(_Entry, dvEntryNull);
    if(dvEnumGetFirstEntry(Enum) == dvEntryNull) {
        dvEnumSetFirstEntry(Enum, _Entry);
    }
    dvEntrySetEnum(_Entry, Enum);
    if(dvEntryGetSym(_Entry) != utSymNull) {
        addEnumEntryToHashTable(Enum, _Entry);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Entry to the Enum after the previous Entry.
----------------------------------------------------------------------------------------*/
void dvEnumInsertAfterEntry(
    dvEnum Enum,
    dvEntry prevEntry,
    dvEntry _Entry)
{
    dvEntry nextEntry = dvEntryGetNextEnumEntry(prevEntry);

#if defined(DD_DEBUG)
    if(Enum == dvEnumNull) {
        utExit("Non-existent Enum");
    }
    if(_Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(dvEntryGetEnum(_Entry) != dvEnumNull) {
        utExit("Attempting to add dv to Enum twice");
    }
#endif
    dvEntrySetNextEnumEntry(_Entry, nextEntry);
    dvEntrySetNextEnumEntry(prevEntry, _Entry);
    dvEntrySetPrevEnumEntry(_Entry, prevEntry);
    if(nextEntry != dvEntryNull) {
        dvEntrySetPrevEnumEntry(nextEntry, _Entry);
    }
    if(dvEnumGetLastEntry(Enum) == prevEntry) {
        dvEnumSetLastEntry(Enum, _Entry);
    }
    dvEntrySetEnum(_Entry, Enum);
    if(dvEntryGetSym(_Entry) != utSymNull) {
        addEnumEntryToHashTable(Enum, _Entry);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Entry from the Enum.
----------------------------------------------------------------------------------------*/
void dvEnumRemoveEntry(
    dvEnum Enum,
    dvEntry _Entry)
{
    dvEntry pEntry, nEntry;

#if defined(DD_DEBUG)
    if(_Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(dvEntryGetEnum(_Entry) != dvEnumNull && dvEntryGetEnum(_Entry) != Enum) {
        utExit("Delete Entry from non-owning Enum");
    }
#endif
    nEntry = dvEntryGetNextEnumEntry(_Entry);
    pEntry = dvEntryGetPrevEnumEntry(_Entry);
    if(pEntry != dvEntryNull) {
        dvEntrySetNextEnumEntry(pEntry, nEntry);
    } else if(dvEnumGetFirstEntry(Enum) == _Entry) {
        dvEnumSetFirstEntry(Enum, nEntry);
    }
    if(nEntry != dvEntryNull) {
        dvEntrySetPrevEnumEntry(nEntry, pEntry);
    } else if(dvEnumGetLastEntry(Enum) == _Entry) {
        dvEnumSetLastEntry(Enum, pEntry);
    }
    dvEntrySetNextEnumEntry(_Entry, dvEntryNull);
    dvEntrySetPrevEnumEntry(_Entry, dvEntryNull);
    dvEntrySetEnum(_Entry, dvEnumNull);
    if(dvEntryGetSym(_Entry) != utSymNull) {
        removeEnumEntryFromHashTable(Enum, _Entry);
    }
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowEnum(
    dvEnum Enum)
{
    utDatabaseShowObject("dv", "Enum", dvEnum2Index(Enum));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocEntry(void)
{
    dvEntry Entry = dvEntryAlloc();

    return dvEntry2Index(Entry);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Entry.
----------------------------------------------------------------------------------------*/
static void allocEntrys(void)
{
    dvSetAllocatedEntry(2);
    dvSetUsedEntry(0);
    dvEntrys.Value = utNewA(uint32, (dvAllocatedEntry()));
    dvEntrys.Enum = utNewA(dvEnum, (dvAllocatedEntry()));
    dvEntrys.NextEnumEntry = utNewA(dvEntry, (dvAllocatedEntry()));
    dvEntrys.PrevEnumEntry = utNewA(dvEntry, (dvAllocatedEntry()));
    dvEntrys.Sym = utNewA(utSym, (dvAllocatedEntry()));
    dvEntrys.NextTableEnumEntry = utNewA(dvEntry, (dvAllocatedEntry()));
    dvEntrys.FirstCase = utNewA(dvCase, (dvAllocatedEntry()));
    dvEntrys.LastCase = utNewA(dvCase, (dvAllocatedEntry()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Entry.
----------------------------------------------------------------------------------------*/
static void reallocEntrys(
    uint32 newSize)
{
    utResizeArray(dvEntrys.Value, (newSize));
    utResizeArray(dvEntrys.Enum, (newSize));
    utResizeArray(dvEntrys.NextEnumEntry, (newSize));
    utResizeArray(dvEntrys.PrevEnumEntry, (newSize));
    utResizeArray(dvEntrys.Sym, (newSize));
    utResizeArray(dvEntrys.NextTableEnumEntry, (newSize));
    utResizeArray(dvEntrys.FirstCase, (newSize));
    utResizeArray(dvEntrys.LastCase, (newSize));
    dvSetAllocatedEntry(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Entrys.
----------------------------------------------------------------------------------------*/
void dvEntryAllocMore(void)
{
    reallocEntrys(dvAllocatedEntry() + (dvAllocatedEntry() >> 1));
}

/*----------------------------------------------------------------------------------------
  Add the Case to the head of the list on the Entry.
----------------------------------------------------------------------------------------*/
void dvEntryInsertCase(
    dvEntry Entry,
    dvCase _Case)
{
#if defined(DD_DEBUG)
    if(Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetEntry(_Case) != dvEntryNull) {
        utExit("Attempting to add dv to Entry twice");
    }
#endif
    dvCaseSetNextEntryCase(_Case, dvEntryGetFirstCase(Entry));
    dvEntrySetFirstCase(Entry, _Case);
    if(dvEntryGetLastCase(Entry) == dvCaseNull) {
        dvEntrySetLastCase(Entry, _Case);
    }
    dvCaseSetEntry(_Case, Entry);
}

/*----------------------------------------------------------------------------------------
  Add the Case to the end of the list on the Entry.
----------------------------------------------------------------------------------------*/
void dvEntryAppendCase(
    dvEntry Entry,
    dvCase _Case)
{
#if defined(DD_DEBUG)
    if(Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetEntry(_Case) != dvEntryNull) {
        utExit("Attempting to add dv to Entry twice");
    }
#endif
    if(dvEntryGetLastCase(Entry) != dvCaseNull) {
        dvCaseSetNextEntryCase(dvEntryGetLastCase(Entry), _Case);
    } else {
        dvEntrySetFirstCase(Entry, _Case);
    }
    dvEntrySetLastCase(Entry, _Case);
    dvCaseSetNextEntryCase(_Case, dvCaseNull);
    dvCaseSetEntry(_Case, Entry);
}

/*----------------------------------------------------------------------------------------
  Insert the Case to the Entry after the previous Case.
----------------------------------------------------------------------------------------*/
void dvEntryInsertAfterCase(
    dvEntry Entry,
    dvCase prevCase,
    dvCase _Case)
{
    dvCase nextCase = dvCaseGetNextEntryCase(prevCase);

#if defined(DD_DEBUG)
    if(Entry == dvEntryNull) {
        utExit("Non-existent Entry");
    }
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetEntry(_Case) != dvEntryNull) {
        utExit("Attempting to add dv to Entry twice");
    }
#endif
    dvCaseSetNextEntryCase(_Case, nextCase);
    dvCaseSetNextEntryCase(prevCase, _Case);
    if(dvEntryGetLastCase(Entry) == prevCase) {
        dvEntrySetLastCase(Entry, _Case);
    }
    dvCaseSetEntry(_Case, Entry);
}

/*----------------------------------------------------------------------------------------
 Remove the Case from the Entry.
----------------------------------------------------------------------------------------*/
void dvEntryRemoveCase(
    dvEntry Entry,
    dvCase _Case)
{
    dvCase pCase, nCase;

#if defined(DD_DEBUG)
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetEntry(_Case) != dvEntryNull && dvCaseGetEntry(_Case) != Entry) {
        utExit("Delete Case from non-owning Entry");
    }
#endif
    pCase = dvCaseNull;
    for(nCase = dvEntryGetFirstCase(Entry); nCase != dvCaseNull && nCase != _Case;
            nCase = dvCaseGetNextEntryCase(nCase)) {
        pCase = nCase;
    }
    if(pCase != dvCaseNull) {
        dvCaseSetNextEntryCase(pCase, dvCaseGetNextEntryCase(_Case));
    } else {
        dvEntrySetFirstCase(Entry, dvCaseGetNextEntryCase(_Case));
    }
    dvCaseSetNextEntryCase(_Case, dvCaseNull);
    if(dvEntryGetLastCase(Entry) == _Case) {
        dvEntrySetLastCase(Entry, pCase);
    }
    dvCaseSetEntry(_Case, dvEntryNull);
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowEntry(
    dvEntry Entry)
{
    utDatabaseShowObject("dv", "Entry", dvEntry2Index(Entry));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocTypedef(void)
{
    dvTypedef Typedef = dvTypedefAlloc();

    return dvTypedef2Index(Typedef);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Typedef.
----------------------------------------------------------------------------------------*/
static void allocTypedefs(void)
{
    dvSetAllocatedTypedef(2);
    dvSetUsedTypedef(0);
    dvTypedefs.Module = utNewA(dvModule, (dvAllocatedTypedef()));
    dvTypedefs.NextModuleTypedef = utNewA(dvTypedef, (dvAllocatedTypedef()));
    dvTypedefs.PrevModuleTypedef = utNewA(dvTypedef, (dvAllocatedTypedef()));
    dvTypedefs.Sym = utNewA(utSym, (dvAllocatedTypedef()));
    dvTypedefs.NextTableModuleTypedef = utNewA(dvTypedef, (dvAllocatedTypedef()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Typedef.
----------------------------------------------------------------------------------------*/
static void reallocTypedefs(
    uint32 newSize)
{
    utResizeArray(dvTypedefs.Module, (newSize));
    utResizeArray(dvTypedefs.NextModuleTypedef, (newSize));
    utResizeArray(dvTypedefs.PrevModuleTypedef, (newSize));
    utResizeArray(dvTypedefs.Sym, (newSize));
    utResizeArray(dvTypedefs.NextTableModuleTypedef, (newSize));
    dvSetAllocatedTypedef(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Typedefs.
----------------------------------------------------------------------------------------*/
void dvTypedefAllocMore(void)
{
    reallocTypedefs(dvAllocatedTypedef() + (dvAllocatedTypedef() >> 1));
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowTypedef(
    dvTypedef Typedef)
{
    utDatabaseShowObject("dv", "Typedef", dvTypedef2Index(Typedef));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocClass(void)
{
    dvClass Class = dvClassAlloc();

    return dvClass2Index(Class);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Class.
----------------------------------------------------------------------------------------*/
static void allocClasss(void)
{
    dvSetAllocatedClass(2);
    dvSetUsedClass(0);
    dvClasss.MemoryStyle = utNewA(dvMemoryStyle, (dvAllocatedClass()));
    dvClasss.ReferenceSize = utNewA(uint8, (dvAllocatedClass()));
    dvClasss.GenerateArrayClass = utNewA(uint8, (dvAllocatedClass() + 7) >> 3);
    dvClasss.NumFields = utNewA(uint16, (dvAllocatedClass()));
    dvClasss.Module = utNewA(dvModule, (dvAllocatedClass()));
    dvClasss.NextModuleClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.PrevModuleClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.Sym = utNewA(utSym, (dvAllocatedClass()));
    dvClasss.NextTableModuleClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.Schema = utNewA(dvSchema, (dvAllocatedClass()));
    dvClasss.NextSchemaClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.FirstProperty = utNewA(dvProperty, (dvAllocatedClass()));
    dvClasss.LastProperty = utNewA(dvProperty, (dvAllocatedClass()));
    dvClasss.PropertyTableIndex = utNewA(uint32, (dvAllocatedClass()));
    dvClasss.NumPropertyTable = utNewA(uint32, (dvAllocatedClass()));
    dvSetUsedClassPropertyTable(0);
    dvSetAllocatedClassPropertyTable(2);
    dvSetFreeClassPropertyTable(0);
    dvClasss.PropertyTable = utNewA(dvProperty, dvAllocatedClassPropertyTable());
    dvClasss.NumProperty = utNewA(uint32, (dvAllocatedClass()));
    dvClasss.FreeListProperty = utNewA(dvProperty, (dvAllocatedClass()));
    dvClasss.BaseClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.FirstDerivedClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.NextClassDerivedClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.LastDerivedClass = utNewA(dvClass, (dvAllocatedClass()));
    dvClasss.FirstChildRelationship = utNewA(dvRelationship, (dvAllocatedClass()));
    dvClasss.LastChildRelationship = utNewA(dvRelationship, (dvAllocatedClass()));
    dvClasss.FirstParentRelationship = utNewA(dvRelationship, (dvAllocatedClass()));
    dvClasss.LastParentRelationship = utNewA(dvRelationship, (dvAllocatedClass()));
    dvClasss.FirstUnion = utNewA(dvUnion, (dvAllocatedClass()));
    dvClasss.LastUnion = utNewA(dvUnion, (dvAllocatedClass()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Class.
----------------------------------------------------------------------------------------*/
static void reallocClasss(
    uint32 newSize)
{
    utResizeArray(dvClasss.MemoryStyle, (newSize));
    utResizeArray(dvClasss.ReferenceSize, (newSize));
    utResizeArray(dvClasss.GenerateArrayClass, (newSize + 7) >> 3);
    utResizeArray(dvClasss.NumFields, (newSize));
    utResizeArray(dvClasss.Module, (newSize));
    utResizeArray(dvClasss.NextModuleClass, (newSize));
    utResizeArray(dvClasss.PrevModuleClass, (newSize));
    utResizeArray(dvClasss.Sym, (newSize));
    utResizeArray(dvClasss.NextTableModuleClass, (newSize));
    utResizeArray(dvClasss.Schema, (newSize));
    utResizeArray(dvClasss.NextSchemaClass, (newSize));
    utResizeArray(dvClasss.FirstProperty, (newSize));
    utResizeArray(dvClasss.LastProperty, (newSize));
    utResizeArray(dvClasss.PropertyTableIndex, (newSize));
    utResizeArray(dvClasss.NumPropertyTable, (newSize));
    utResizeArray(dvClasss.NumProperty, (newSize));
    utResizeArray(dvClasss.FreeListProperty, (newSize));
    utResizeArray(dvClasss.BaseClass, (newSize));
    utResizeArray(dvClasss.FirstDerivedClass, (newSize));
    utResizeArray(dvClasss.NextClassDerivedClass, (newSize));
    utResizeArray(dvClasss.LastDerivedClass, (newSize));
    utResizeArray(dvClasss.FirstChildRelationship, (newSize));
    utResizeArray(dvClasss.LastChildRelationship, (newSize));
    utResizeArray(dvClasss.FirstParentRelationship, (newSize));
    utResizeArray(dvClasss.LastParentRelationship, (newSize));
    utResizeArray(dvClasss.FirstUnion, (newSize));
    utResizeArray(dvClasss.LastUnion, (newSize));
    dvSetAllocatedClass(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Classs.
----------------------------------------------------------------------------------------*/
void dvClassAllocMore(void)
{
    reallocClasss(dvAllocatedClass() + (dvAllocatedClass() >> 1));
}

/*----------------------------------------------------------------------------------------
  Compact the Class.PropertyTable heap to free memory.
----------------------------------------------------------------------------------------*/
void dvCompactClassPropertyTables(void)
{
    uint32 elementSize = sizeof(dvProperty);
    uint32 usedHeaderSize = (sizeof(dvClass) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvClass) + sizeof(uint32) + elementSize - 1)/elementSize;
    dvProperty *toPtr = dvClasss.PropertyTable;
    dvProperty *fromPtr = toPtr;
    dvClass Class;
    uint32 size;

    while(fromPtr < dvClasss.PropertyTable + dvUsedClassPropertyTable()) {
        Class = *(dvClass *)(void *)fromPtr;
        if(Class != dvClassNull) {
            /* Need to move it to toPtr */
            size = utMax(dvClassGetNumPropertyTable(Class) + usedHeaderSize, freeHeaderSize);
            memmove((void *)toPtr, (void *)fromPtr, size*elementSize);
            dvClassSetPropertyTableIndex(Class, toPtr - dvClasss.PropertyTable + usedHeaderSize);
            toPtr += size;
        } else {
            /* Just skip it */
            size = *(uint32 *)(void *)(((dvClass *)(void *)fromPtr) + 1);
        }
        fromPtr += size;
    }
    dvSetUsedClassPropertyTable(toPtr - dvClasss.PropertyTable);
    dvSetFreeClassPropertyTable(0);
}

/*----------------------------------------------------------------------------------------
  Allocate more memory for the Class.PropertyTable heap.
----------------------------------------------------------------------------------------*/
static void allocMoreClassPropertyTables(
    uint32 spaceNeeded)
{
    uint32 freeSpace = dvAllocatedClassPropertyTable() - dvUsedClassPropertyTable();

    if((dvFreeClassPropertyTable() << 2) > dvUsedClassPropertyTable()) {
        dvCompactClassPropertyTables();
        freeSpace = dvAllocatedClassPropertyTable() - dvUsedClassPropertyTable();
    }
    if(freeSpace < spaceNeeded) {
        dvSetAllocatedClassPropertyTable(dvAllocatedClassPropertyTable() + spaceNeeded - freeSpace +
            (dvAllocatedClassPropertyTable() >> 2));
        utResizeArray(dvClasss.PropertyTable, dvAllocatedClassPropertyTable());
    }
}

/*----------------------------------------------------------------------------------------
  Allocate memory for a new Class.PropertyTable array.
----------------------------------------------------------------------------------------*/
void dvClassAllocPropertyTables(
    dvClass Class,
    uint32 numPropertyTables)
{
    uint32 freeSpace = dvAllocatedClassPropertyTable() - dvUsedClassPropertyTable();
    uint32 elementSize = sizeof(dvProperty);
    uint32 usedHeaderSize = (sizeof(dvClass) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvClass) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 spaceNeeded = utMax(numPropertyTables + usedHeaderSize, freeHeaderSize);

#if defined(DD_DEBUG)
    utAssert(dvClassGetNumPropertyTable(Class) == 0);
#endif
    if(numPropertyTables == 0) {
        return;
    }
    if(freeSpace < spaceNeeded) {
        allocMoreClassPropertyTables(spaceNeeded);
    }
    dvClassSetPropertyTableIndex(Class, dvUsedClassPropertyTable() + usedHeaderSize);
    dvClassSetNumPropertyTable(Class, numPropertyTables);
    *(dvClass *)(void *)(dvClasss.PropertyTable + dvUsedClassPropertyTable()) = Class;
    memset(dvClassGetPropertyTables(Class), 0xff, numPropertyTables*elementSize);
    dvSetUsedClassPropertyTable(dvUsedClassPropertyTable() + spaceNeeded);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvClassGetPropertyTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *getClassPropertyTables(
    uint64 objectNumber,
    uint32 *numValues)
{
    dvClass Class = dvIndex2Class(objectNumber);

    *numValues = dvClassGetNumPropertyTable(Class);
    return dvClassGetPropertyTables(Class);
}

/*----------------------------------------------------------------------------------------
  Wrapper around dvClassAllocPropertyTables for the database manager.
----------------------------------------------------------------------------------------*/
static void *allocClassPropertyTables(
    uint64 objectNumber,
    uint32 numValues)
{
    dvClass Class = dvIndex2Class(objectNumber);

    dvClassSetPropertyTableIndex(Class, 0);
    dvClassSetNumPropertyTable(Class, 0);
    if(numValues == 0) {
        return NULL;
    }
    dvClassAllocPropertyTables(Class, numValues);
    return dvClassGetPropertyTables(Class);
}

/*----------------------------------------------------------------------------------------
  Free memory used by the Class.PropertyTable array.
----------------------------------------------------------------------------------------*/
void dvClassFreePropertyTables(
    dvClass Class)
{
    uint32 elementSize = sizeof(dvProperty);
    uint32 usedHeaderSize = (sizeof(dvClass) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvClass) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 size = utMax(dvClassGetNumPropertyTable(Class) + usedHeaderSize, freeHeaderSize);
    dvProperty *dataPtr = dvClassGetPropertyTables(Class) - usedHeaderSize;

    if(dvClassGetNumPropertyTable(Class) == 0) {
        return;
    }
    *(dvClass *)(void *)(dataPtr) = dvClassNull;
    *(uint32 *)(void *)(((dvClass *)(void *)dataPtr) + 1) = size;
    dvClassSetNumPropertyTable(Class, 0);
    dvSetFreeClassPropertyTable(dvFreeClassPropertyTable() + size);
}

/*----------------------------------------------------------------------------------------
  Resize the Class.PropertyTable array.
----------------------------------------------------------------------------------------*/
void dvClassResizePropertyTables(
    dvClass Class,
    uint32 numPropertyTables)
{
    uint32 freeSpace = dvAllocatedClassPropertyTable() - dvUsedClassPropertyTable();
    uint32 elementSize = sizeof(dvProperty);
    uint32 usedHeaderSize = (sizeof(dvClass) + elementSize - 1)/elementSize;
    uint32 freeHeaderSize = (sizeof(dvClass) + sizeof(uint32) + elementSize - 1)/elementSize;
    uint32 newSize = utMax(numPropertyTables + usedHeaderSize, freeHeaderSize);
    uint32 oldSize = utMax(dvClassGetNumPropertyTable(Class) + usedHeaderSize, freeHeaderSize);
    dvProperty *dataPtr;

    if(numPropertyTables == 0) {
        if(dvClassGetNumPropertyTable(Class) != 0) {
            dvClassFreePropertyTables(Class);
        };
        return;
    }
    if(dvClassGetNumPropertyTable(Class) == 0) {
        dvClassAllocPropertyTables(Class, newSize);
        return;
    };
    if(freeSpace < newSize) {
        allocMoreClassPropertyTables(newSize);
    }
    dataPtr = dvClassGetPropertyTables(Class) - usedHeaderSize;
    memcpy((void *)(dvClasss.PropertyTable + dvUsedClassPropertyTable()), dataPtr,
        elementSize*utMin(oldSize, newSize));
    if(newSize > oldSize) {
        memset(dvClasss.PropertyTable + dvUsedClassPropertyTable() + oldSize, 0xff,
            elementSize*(newSize - oldSize));
    }
    *(dvClass *)(void *)dataPtr = dvClassNull;
    *(uint32 *)(void *)(((dvClass *)(void *)dataPtr) + 1) = oldSize;
    dvSetFreeClassPropertyTable(dvFreeClassPropertyTable() + oldSize);
    dvClassSetPropertyTableIndex(Class, dvUsedClassPropertyTable() + usedHeaderSize);
    dvClassSetNumPropertyTable(Class, numPropertyTables);
    dvSetUsedClassPropertyTable(dvUsedClassPropertyTable() + newSize);
}

static void addClassPropertyToHashTable(dvClass Class, dvProperty _Property);
/*----------------------------------------------------------------------------------------
  Increase the size of the hash table.
----------------------------------------------------------------------------------------*/
static void resizeClassPropertyHashTable(
    dvClass Class)
{
    dvProperty _Property;
    dvProperty *Propertys;
    uint32 numPropertys = dvClassGetNumPropertyTable(Class) << 1;

    if(numPropertys == 0) {
        numPropertys = 2;
        dvClassAllocPropertyTables(Class, 2);
    } else {
        dvClassResizePropertyTables(Class, numPropertys);
    }
    Propertys = dvClassGetPropertyTables(Class);
    /* Zero out the table */
    while(numPropertys-- != 0) {
        *Propertys++ = dvPropertyNull;
    }
    dvClassSetNumProperty(Class, 0);
    dvForeachClassProperty(Class, _Property) {
        if(dvPropertyGetSym(_Property) != utSymNull) {
            addClassPropertyToHashTable(Class, _Property);
        }
    } dvEndForeachClassProperty;
}

/*----------------------------------------------------------------------------------------
  Add an  to the Class.  If the table is near full, build a new one twice
  as big, delete the old one, and return the new one.
----------------------------------------------------------------------------------------*/
static void addClassPropertyToHashTable(
    dvClass Class,
    dvProperty _Property)
{
    dvProperty nextProperty;
    uint32 index;

    if(dvClassGetNumProperty(Class) >= dvClassGetNumPropertyTable(Class)) {
        resizeClassPropertyHashTable(Class);
        return;
    }
    index = (dvClassGetNumPropertyTable(Class) - 1) & utSymGetHashValue(dvPropertyGetSym(_Property));
    nextProperty = dvClassGetiPropertyTable(Class, index);
    dvPropertySetNextTableClassProperty(_Property, nextProperty);
    dvClassSetiPropertyTable(Class, index, _Property);
    dvClassSetNumProperty(Class, dvClassGetNumProperty(Class) + 1);
}

/*----------------------------------------------------------------------------------------
  Remove the Property from the hash table.
----------------------------------------------------------------------------------------*/
static void removeClassPropertyFromHashTable(
   dvClass Class,
   dvProperty _Property)
{
    uint32 index = (dvClassGetNumPropertyTable(Class) - 1) & utSymGetHashValue(dvPropertyGetSym(_Property));
    dvProperty prevProperty, nextProperty;
    
    nextProperty = dvClassGetiPropertyTable(Class, index);
    if(nextProperty == _Property) {
        dvClassSetiPropertyTable(Class, index, dvPropertyGetNextTableClassProperty(nextProperty));
    } else {
        do {
            prevProperty = nextProperty;
            nextProperty = dvPropertyGetNextTableClassProperty(nextProperty);
        } while(nextProperty != _Property);
        dvPropertySetNextTableClassProperty(prevProperty, dvPropertyGetNextTableClassProperty(_Property));
    }
    dvClassSetNumProperty(Class, dvClassGetNumProperty(Class) - 1);
    dvPropertySetNextTableClassProperty(_Property, dvPropertyNull);
}

/*----------------------------------------------------------------------------------------
  Find the Property from the Class and its name.
----------------------------------------------------------------------------------------*/
dvProperty dvClassFindProperty(
    dvClass Class,
    utSym sym)
{
    uint32 mask = dvClassGetNumPropertyTable(Class) - 1;
    dvProperty _Property;

    if(mask + 1 != 0) {
        _Property = dvClassGetiPropertyTable(Class, utSymGetHashValue(sym) & mask);
        while(_Property != dvPropertyNull) {
            if(dvPropertyGetSym(_Property) == sym) {
                return _Property;
            }
            _Property = dvPropertyGetNextTableClassProperty(_Property);
        }
    }
    return dvPropertyNull;
}

/*----------------------------------------------------------------------------------------
  Find the Property from the Class and its name.
----------------------------------------------------------------------------------------*/
void dvClassRenameProperty(
    dvClass Class,
    dvProperty _Property,
    utSym sym)
{
    if(dvPropertyGetSym(_Property) != utSymNull) {
        removeClassPropertyFromHashTable(Class, _Property);
    }
    dvPropertySetSym(_Property, sym);
    if(sym != utSymNull) {
        addClassPropertyToHashTable(Class, _Property);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Property to the head of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassInsertProperty(
    dvClass Class,
    dvProperty _Property)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetClass(_Property) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvPropertySetNextClassProperty(_Property, dvClassGetFirstProperty(Class));
    if(dvClassGetFirstProperty(Class) != dvPropertyNull) {
        dvPropertySetPrevClassProperty(dvClassGetFirstProperty(Class), _Property);
    }
    dvClassSetFirstProperty(Class, _Property);
    dvPropertySetPrevClassProperty(_Property, dvPropertyNull);
    if(dvClassGetLastProperty(Class) == dvPropertyNull) {
        dvClassSetLastProperty(Class, _Property);
    }
    dvPropertySetClass(_Property, Class);
    if(dvPropertyGetSym(_Property) != utSymNull) {
        addClassPropertyToHashTable(Class, _Property);
    }
}

/*----------------------------------------------------------------------------------------
  Add the Property to the end of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassAppendProperty(
    dvClass Class,
    dvProperty _Property)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetClass(_Property) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvPropertySetPrevClassProperty(_Property, dvClassGetLastProperty(Class));
    if(dvClassGetLastProperty(Class) != dvPropertyNull) {
        dvPropertySetNextClassProperty(dvClassGetLastProperty(Class), _Property);
    }
    dvClassSetLastProperty(Class, _Property);
    dvPropertySetNextClassProperty(_Property, dvPropertyNull);
    if(dvClassGetFirstProperty(Class) == dvPropertyNull) {
        dvClassSetFirstProperty(Class, _Property);
    }
    dvPropertySetClass(_Property, Class);
    if(dvPropertyGetSym(_Property) != utSymNull) {
        addClassPropertyToHashTable(Class, _Property);
    }
}

/*----------------------------------------------------------------------------------------
  Insert the Property to the Class after the previous Property.
----------------------------------------------------------------------------------------*/
void dvClassInsertAfterProperty(
    dvClass Class,
    dvProperty prevProperty,
    dvProperty _Property)
{
    dvProperty nextProperty = dvPropertyGetNextClassProperty(prevProperty);

#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetClass(_Property) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvPropertySetNextClassProperty(_Property, nextProperty);
    dvPropertySetNextClassProperty(prevProperty, _Property);
    dvPropertySetPrevClassProperty(_Property, prevProperty);
    if(nextProperty != dvPropertyNull) {
        dvPropertySetPrevClassProperty(nextProperty, _Property);
    }
    if(dvClassGetLastProperty(Class) == prevProperty) {
        dvClassSetLastProperty(Class, _Property);
    }
    dvPropertySetClass(_Property, Class);
    if(dvPropertyGetSym(_Property) != utSymNull) {
        addClassPropertyToHashTable(Class, _Property);
    }
}

/*----------------------------------------------------------------------------------------
 Remove the Property from the Class.
----------------------------------------------------------------------------------------*/
void dvClassRemoveProperty(
    dvClass Class,
    dvProperty _Property)
{
    dvProperty pProperty, nProperty;

#if defined(DD_DEBUG)
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetClass(_Property) != dvClassNull && dvPropertyGetClass(_Property) != Class) {
        utExit("Delete Property from non-owning Class");
    }
#endif
    nProperty = dvPropertyGetNextClassProperty(_Property);
    pProperty = dvPropertyGetPrevClassProperty(_Property);
    if(pProperty != dvPropertyNull) {
        dvPropertySetNextClassProperty(pProperty, nProperty);
    } else if(dvClassGetFirstProperty(Class) == _Property) {
        dvClassSetFirstProperty(Class, nProperty);
    }
    if(nProperty != dvPropertyNull) {
        dvPropertySetPrevClassProperty(nProperty, pProperty);
    } else if(dvClassGetLastProperty(Class) == _Property) {
        dvClassSetLastProperty(Class, pProperty);
    }
    dvPropertySetNextClassProperty(_Property, dvPropertyNull);
    dvPropertySetPrevClassProperty(_Property, dvPropertyNull);
    dvPropertySetClass(_Property, dvClassNull);
    if(dvPropertyGetSym(_Property) != utSymNull) {
        removeClassPropertyFromHashTable(Class, _Property);
    }
}

/*----------------------------------------------------------------------------------------
  Add the DerivedClass to the head of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassInsertDerivedClass(
    dvClass Class,
    dvClass _Class)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetBaseClass(_Class) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvClassSetNextClassDerivedClass(_Class, dvClassGetFirstDerivedClass(Class));
    dvClassSetFirstDerivedClass(Class, _Class);
    if(dvClassGetLastDerivedClass(Class) == dvClassNull) {
        dvClassSetLastDerivedClass(Class, _Class);
    }
    dvClassSetBaseClass(_Class, Class);
}

/*----------------------------------------------------------------------------------------
  Add the DerivedClass to the end of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassAppendDerivedClass(
    dvClass Class,
    dvClass _Class)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetBaseClass(_Class) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    if(dvClassGetLastDerivedClass(Class) != dvClassNull) {
        dvClassSetNextClassDerivedClass(dvClassGetLastDerivedClass(Class), _Class);
    } else {
        dvClassSetFirstDerivedClass(Class, _Class);
    }
    dvClassSetLastDerivedClass(Class, _Class);
    dvClassSetNextClassDerivedClass(_Class, dvClassNull);
    dvClassSetBaseClass(_Class, Class);
}

/*----------------------------------------------------------------------------------------
  Insert the DerivedClass to the Class after the previous DerivedClass.
----------------------------------------------------------------------------------------*/
void dvClassInsertAfterDerivedClass(
    dvClass Class,
    dvClass prevClass,
    dvClass _Class)
{
    dvClass nextClass = dvClassGetNextClassDerivedClass(prevClass);

#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetBaseClass(_Class) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvClassSetNextClassDerivedClass(_Class, nextClass);
    dvClassSetNextClassDerivedClass(prevClass, _Class);
    if(dvClassGetLastDerivedClass(Class) == prevClass) {
        dvClassSetLastDerivedClass(Class, _Class);
    }
    dvClassSetBaseClass(_Class, Class);
}

/*----------------------------------------------------------------------------------------
 Remove the DerivedClass from the Class.
----------------------------------------------------------------------------------------*/
void dvClassRemoveDerivedClass(
    dvClass Class,
    dvClass _Class)
{
    dvClass pClass, nClass;

#if defined(DD_DEBUG)
    if(_Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(dvClassGetBaseClass(_Class) != dvClassNull && dvClassGetBaseClass(_Class) != Class) {
        utExit("Delete Class from non-owning Class");
    }
#endif
    pClass = dvClassNull;
    for(nClass = dvClassGetFirstDerivedClass(Class); nClass != dvClassNull && nClass != _Class;
            nClass = dvClassGetNextClassDerivedClass(nClass)) {
        pClass = nClass;
    }
    if(pClass != dvClassNull) {
        dvClassSetNextClassDerivedClass(pClass, dvClassGetNextClassDerivedClass(_Class));
    } else {
        dvClassSetFirstDerivedClass(Class, dvClassGetNextClassDerivedClass(_Class));
    }
    dvClassSetNextClassDerivedClass(_Class, dvClassNull);
    if(dvClassGetLastDerivedClass(Class) == _Class) {
        dvClassSetLastDerivedClass(Class, pClass);
    }
    dvClassSetBaseClass(_Class, dvClassNull);
}

/*----------------------------------------------------------------------------------------
  Add the ChildRelationship to the head of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassInsertChildRelationship(
    dvClass Class,
    dvRelationship _Relationship)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetParentClass(_Relationship) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvRelationshipSetNextClassChildRelationship(_Relationship, dvClassGetFirstChildRelationship(Class));
    dvClassSetFirstChildRelationship(Class, _Relationship);
    if(dvClassGetLastChildRelationship(Class) == dvRelationshipNull) {
        dvClassSetLastChildRelationship(Class, _Relationship);
    }
    dvRelationshipSetParentClass(_Relationship, Class);
}

/*----------------------------------------------------------------------------------------
  Add the ChildRelationship to the end of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassAppendChildRelationship(
    dvClass Class,
    dvRelationship _Relationship)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetParentClass(_Relationship) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    if(dvClassGetLastChildRelationship(Class) != dvRelationshipNull) {
        dvRelationshipSetNextClassChildRelationship(dvClassGetLastChildRelationship(Class), _Relationship);
    } else {
        dvClassSetFirstChildRelationship(Class, _Relationship);
    }
    dvClassSetLastChildRelationship(Class, _Relationship);
    dvRelationshipSetNextClassChildRelationship(_Relationship, dvRelationshipNull);
    dvRelationshipSetParentClass(_Relationship, Class);
}

/*----------------------------------------------------------------------------------------
  Insert the ChildRelationship to the Class after the previous ChildRelationship.
----------------------------------------------------------------------------------------*/
void dvClassInsertAfterChildRelationship(
    dvClass Class,
    dvRelationship prevRelationship,
    dvRelationship _Relationship)
{
    dvRelationship nextRelationship = dvRelationshipGetNextClassChildRelationship(prevRelationship);

#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetParentClass(_Relationship) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvRelationshipSetNextClassChildRelationship(_Relationship, nextRelationship);
    dvRelationshipSetNextClassChildRelationship(prevRelationship, _Relationship);
    if(dvClassGetLastChildRelationship(Class) == prevRelationship) {
        dvClassSetLastChildRelationship(Class, _Relationship);
    }
    dvRelationshipSetParentClass(_Relationship, Class);
}

/*----------------------------------------------------------------------------------------
 Remove the ChildRelationship from the Class.
----------------------------------------------------------------------------------------*/
void dvClassRemoveChildRelationship(
    dvClass Class,
    dvRelationship _Relationship)
{
    dvRelationship pRelationship, nRelationship;

#if defined(DD_DEBUG)
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetParentClass(_Relationship) != dvClassNull && dvRelationshipGetParentClass(_Relationship) != Class) {
        utExit("Delete Relationship from non-owning Class");
    }
#endif
    pRelationship = dvRelationshipNull;
    for(nRelationship = dvClassGetFirstChildRelationship(Class); nRelationship != dvRelationshipNull && nRelationship != _Relationship;
            nRelationship = dvRelationshipGetNextClassChildRelationship(nRelationship)) {
        pRelationship = nRelationship;
    }
    if(pRelationship != dvRelationshipNull) {
        dvRelationshipSetNextClassChildRelationship(pRelationship, dvRelationshipGetNextClassChildRelationship(_Relationship));
    } else {
        dvClassSetFirstChildRelationship(Class, dvRelationshipGetNextClassChildRelationship(_Relationship));
    }
    dvRelationshipSetNextClassChildRelationship(_Relationship, dvRelationshipNull);
    if(dvClassGetLastChildRelationship(Class) == _Relationship) {
        dvClassSetLastChildRelationship(Class, pRelationship);
    }
    dvRelationshipSetParentClass(_Relationship, dvClassNull);
}

/*----------------------------------------------------------------------------------------
  Add the ParentRelationship to the head of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassInsertParentRelationship(
    dvClass Class,
    dvRelationship _Relationship)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetChildClass(_Relationship) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvRelationshipSetNextClassParentRelationship(_Relationship, dvClassGetFirstParentRelationship(Class));
    dvClassSetFirstParentRelationship(Class, _Relationship);
    if(dvClassGetLastParentRelationship(Class) == dvRelationshipNull) {
        dvClassSetLastParentRelationship(Class, _Relationship);
    }
    dvRelationshipSetChildClass(_Relationship, Class);
}

/*----------------------------------------------------------------------------------------
  Add the ParentRelationship to the end of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassAppendParentRelationship(
    dvClass Class,
    dvRelationship _Relationship)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetChildClass(_Relationship) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    if(dvClassGetLastParentRelationship(Class) != dvRelationshipNull) {
        dvRelationshipSetNextClassParentRelationship(dvClassGetLastParentRelationship(Class), _Relationship);
    } else {
        dvClassSetFirstParentRelationship(Class, _Relationship);
    }
    dvClassSetLastParentRelationship(Class, _Relationship);
    dvRelationshipSetNextClassParentRelationship(_Relationship, dvRelationshipNull);
    dvRelationshipSetChildClass(_Relationship, Class);
}

/*----------------------------------------------------------------------------------------
  Insert the ParentRelationship to the Class after the previous ParentRelationship.
----------------------------------------------------------------------------------------*/
void dvClassInsertAfterParentRelationship(
    dvClass Class,
    dvRelationship prevRelationship,
    dvRelationship _Relationship)
{
    dvRelationship nextRelationship = dvRelationshipGetNextClassParentRelationship(prevRelationship);

#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetChildClass(_Relationship) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvRelationshipSetNextClassParentRelationship(_Relationship, nextRelationship);
    dvRelationshipSetNextClassParentRelationship(prevRelationship, _Relationship);
    if(dvClassGetLastParentRelationship(Class) == prevRelationship) {
        dvClassSetLastParentRelationship(Class, _Relationship);
    }
    dvRelationshipSetChildClass(_Relationship, Class);
}

/*----------------------------------------------------------------------------------------
 Remove the ParentRelationship from the Class.
----------------------------------------------------------------------------------------*/
void dvClassRemoveParentRelationship(
    dvClass Class,
    dvRelationship _Relationship)
{
    dvRelationship pRelationship, nRelationship;

#if defined(DD_DEBUG)
    if(_Relationship == dvRelationshipNull) {
        utExit("Non-existent Relationship");
    }
    if(dvRelationshipGetChildClass(_Relationship) != dvClassNull && dvRelationshipGetChildClass(_Relationship) != Class) {
        utExit("Delete Relationship from non-owning Class");
    }
#endif
    pRelationship = dvRelationshipNull;
    for(nRelationship = dvClassGetFirstParentRelationship(Class); nRelationship != dvRelationshipNull && nRelationship != _Relationship;
            nRelationship = dvRelationshipGetNextClassParentRelationship(nRelationship)) {
        pRelationship = nRelationship;
    }
    if(pRelationship != dvRelationshipNull) {
        dvRelationshipSetNextClassParentRelationship(pRelationship, dvRelationshipGetNextClassParentRelationship(_Relationship));
    } else {
        dvClassSetFirstParentRelationship(Class, dvRelationshipGetNextClassParentRelationship(_Relationship));
    }
    dvRelationshipSetNextClassParentRelationship(_Relationship, dvRelationshipNull);
    if(dvClassGetLastParentRelationship(Class) == _Relationship) {
        dvClassSetLastParentRelationship(Class, pRelationship);
    }
    dvRelationshipSetChildClass(_Relationship, dvClassNull);
}

/*----------------------------------------------------------------------------------------
  Add the Union to the head of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassInsertUnion(
    dvClass Class,
    dvUnion _Union)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(dvUnionGetClass(_Union) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvUnionSetNextClassUnion(_Union, dvClassGetFirstUnion(Class));
    dvClassSetFirstUnion(Class, _Union);
    if(dvClassGetLastUnion(Class) == dvUnionNull) {
        dvClassSetLastUnion(Class, _Union);
    }
    dvUnionSetClass(_Union, Class);
}

/*----------------------------------------------------------------------------------------
  Add the Union to the end of the list on the Class.
----------------------------------------------------------------------------------------*/
void dvClassAppendUnion(
    dvClass Class,
    dvUnion _Union)
{
#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(dvUnionGetClass(_Union) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    if(dvClassGetLastUnion(Class) != dvUnionNull) {
        dvUnionSetNextClassUnion(dvClassGetLastUnion(Class), _Union);
    } else {
        dvClassSetFirstUnion(Class, _Union);
    }
    dvClassSetLastUnion(Class, _Union);
    dvUnionSetNextClassUnion(_Union, dvUnionNull);
    dvUnionSetClass(_Union, Class);
}

/*----------------------------------------------------------------------------------------
  Insert the Union to the Class after the previous Union.
----------------------------------------------------------------------------------------*/
void dvClassInsertAfterUnion(
    dvClass Class,
    dvUnion prevUnion,
    dvUnion _Union)
{
    dvUnion nextUnion = dvUnionGetNextClassUnion(prevUnion);

#if defined(DD_DEBUG)
    if(Class == dvClassNull) {
        utExit("Non-existent Class");
    }
    if(_Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(dvUnionGetClass(_Union) != dvClassNull) {
        utExit("Attempting to add dv to Class twice");
    }
#endif
    dvUnionSetNextClassUnion(_Union, nextUnion);
    dvUnionSetNextClassUnion(prevUnion, _Union);
    if(dvClassGetLastUnion(Class) == prevUnion) {
        dvClassSetLastUnion(Class, _Union);
    }
    dvUnionSetClass(_Union, Class);
}

/*----------------------------------------------------------------------------------------
 Remove the Union from the Class.
----------------------------------------------------------------------------------------*/
void dvClassRemoveUnion(
    dvClass Class,
    dvUnion _Union)
{
    dvUnion pUnion, nUnion;

#if defined(DD_DEBUG)
    if(_Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(dvUnionGetClass(_Union) != dvClassNull && dvUnionGetClass(_Union) != Class) {
        utExit("Delete Union from non-owning Class");
    }
#endif
    pUnion = dvUnionNull;
    for(nUnion = dvClassGetFirstUnion(Class); nUnion != dvUnionNull && nUnion != _Union;
            nUnion = dvUnionGetNextClassUnion(nUnion)) {
        pUnion = nUnion;
    }
    if(pUnion != dvUnionNull) {
        dvUnionSetNextClassUnion(pUnion, dvUnionGetNextClassUnion(_Union));
    } else {
        dvClassSetFirstUnion(Class, dvUnionGetNextClassUnion(_Union));
    }
    dvUnionSetNextClassUnion(_Union, dvUnionNull);
    if(dvClassGetLastUnion(Class) == _Union) {
        dvClassSetLastUnion(Class, pUnion);
    }
    dvUnionSetClass(_Union, dvClassNull);
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowClass(
    dvClass Class)
{
    utDatabaseShowObject("dv", "Class", dvClass2Index(Class));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocProperty(void)
{
    dvProperty Property = dvPropertyAlloc();

    return dvProperty2Index(Property);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Property.
----------------------------------------------------------------------------------------*/
static void allocPropertys(void)
{
    dvSetAllocatedProperty(2);
    dvSetUsedProperty(0);
    dvPropertys.Type = utNewA(dvPropertyType, (dvAllocatedProperty()));
    dvPropertys.Array = utNewA(uint8, (dvAllocatedProperty() + 7) >> 3);
    dvPropertys.Cascade = utNewA(uint8, (dvAllocatedProperty() + 7) >> 3);
    dvPropertys.FieldNumber = utNewA(uint32, (dvAllocatedProperty()));
    dvPropertys.FirstElementProp = utNewA(dvProperty, (dvAllocatedProperty()));
    dvPropertys.NumElementsProp = utNewA(dvProperty, (dvAllocatedProperty()));
    dvPropertys.Hidden = utNewA(uint8, (dvAllocatedProperty() + 7) >> 3);
    dvPropertys.Line = utNewA(uint32, (dvAllocatedProperty()));
    dvPropertys.Class = utNewA(dvClass, (dvAllocatedProperty()));
    dvPropertys.NextClassProperty = utNewA(dvProperty, (dvAllocatedProperty()));
    dvPropertys.PrevClassProperty = utNewA(dvProperty, (dvAllocatedProperty()));
    dvPropertys.Sym = utNewA(utSym, (dvAllocatedProperty()));
    dvPropertys.NextTableClassProperty = utNewA(dvProperty, (dvAllocatedProperty()));
    dvPropertys.FirstCase = utNewA(dvCase, (dvAllocatedProperty()));
    dvPropertys.LastCase = utNewA(dvCase, (dvAllocatedProperty()));
    dvPropertys.Union = utNewA(dvUnion, (dvAllocatedProperty()));
    dvPropertys.NextUnionProperty = utNewA(dvProperty, (dvAllocatedProperty()));
    dvPropertys.union1 = utNewA(dvPropertyUnion1, dvAllocatedProperty());
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Property.
----------------------------------------------------------------------------------------*/
static void reallocPropertys(
    uint32 newSize)
{
    utResizeArray(dvPropertys.Type, (newSize));
    utResizeArray(dvPropertys.Array, (newSize + 7) >> 3);
    utResizeArray(dvPropertys.Cascade, (newSize + 7) >> 3);
    utResizeArray(dvPropertys.FieldNumber, (newSize));
    utResizeArray(dvPropertys.FirstElementProp, (newSize));
    utResizeArray(dvPropertys.NumElementsProp, (newSize));
    utResizeArray(dvPropertys.Hidden, (newSize + 7) >> 3);
    utResizeArray(dvPropertys.Line, (newSize));
    utResizeArray(dvPropertys.Class, (newSize));
    utResizeArray(dvPropertys.NextClassProperty, (newSize));
    utResizeArray(dvPropertys.PrevClassProperty, (newSize));
    utResizeArray(dvPropertys.Sym, (newSize));
    utResizeArray(dvPropertys.NextTableClassProperty, (newSize));
    utResizeArray(dvPropertys.FirstCase, (newSize));
    utResizeArray(dvPropertys.LastCase, (newSize));
    utResizeArray(dvPropertys.Union, (newSize));
    utResizeArray(dvPropertys.NextUnionProperty, (newSize));
    utResizeArray(dvPropertys.union1, newSize);
    dvSetAllocatedProperty(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Propertys.
----------------------------------------------------------------------------------------*/
void dvPropertyAllocMore(void)
{
    reallocPropertys(dvAllocatedProperty() + (dvAllocatedProperty() >> 1));
}

/*----------------------------------------------------------------------------------------
  Add the Case to the head of the list on the Property.
----------------------------------------------------------------------------------------*/
void dvPropertyInsertCase(
    dvProperty Property,
    dvCase _Case)
{
#if defined(DD_DEBUG)
    if(Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetProperty(_Case) != dvPropertyNull) {
        utExit("Attempting to add dv to Property twice");
    }
#endif
    dvCaseSetNextPropertyCase(_Case, dvPropertyGetFirstCase(Property));
    dvPropertySetFirstCase(Property, _Case);
    if(dvPropertyGetLastCase(Property) == dvCaseNull) {
        dvPropertySetLastCase(Property, _Case);
    }
    dvCaseSetProperty(_Case, Property);
}

/*----------------------------------------------------------------------------------------
  Add the Case to the end of the list on the Property.
----------------------------------------------------------------------------------------*/
void dvPropertyAppendCase(
    dvProperty Property,
    dvCase _Case)
{
#if defined(DD_DEBUG)
    if(Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetProperty(_Case) != dvPropertyNull) {
        utExit("Attempting to add dv to Property twice");
    }
#endif
    if(dvPropertyGetLastCase(Property) != dvCaseNull) {
        dvCaseSetNextPropertyCase(dvPropertyGetLastCase(Property), _Case);
    } else {
        dvPropertySetFirstCase(Property, _Case);
    }
    dvPropertySetLastCase(Property, _Case);
    dvCaseSetNextPropertyCase(_Case, dvCaseNull);
    dvCaseSetProperty(_Case, Property);
}

/*----------------------------------------------------------------------------------------
  Insert the Case to the Property after the previous Case.
----------------------------------------------------------------------------------------*/
void dvPropertyInsertAfterCase(
    dvProperty Property,
    dvCase prevCase,
    dvCase _Case)
{
    dvCase nextCase = dvCaseGetNextPropertyCase(prevCase);

#if defined(DD_DEBUG)
    if(Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetProperty(_Case) != dvPropertyNull) {
        utExit("Attempting to add dv to Property twice");
    }
#endif
    dvCaseSetNextPropertyCase(_Case, nextCase);
    dvCaseSetNextPropertyCase(prevCase, _Case);
    if(dvPropertyGetLastCase(Property) == prevCase) {
        dvPropertySetLastCase(Property, _Case);
    }
    dvCaseSetProperty(_Case, Property);
}

/*----------------------------------------------------------------------------------------
 Remove the Case from the Property.
----------------------------------------------------------------------------------------*/
void dvPropertyRemoveCase(
    dvProperty Property,
    dvCase _Case)
{
    dvCase pCase, nCase;

#if defined(DD_DEBUG)
    if(_Case == dvCaseNull) {
        utExit("Non-existent Case");
    }
    if(dvCaseGetProperty(_Case) != dvPropertyNull && dvCaseGetProperty(_Case) != Property) {
        utExit("Delete Case from non-owning Property");
    }
#endif
    pCase = dvCaseNull;
    for(nCase = dvPropertyGetFirstCase(Property); nCase != dvCaseNull && nCase != _Case;
            nCase = dvCaseGetNextPropertyCase(nCase)) {
        pCase = nCase;
    }
    if(pCase != dvCaseNull) {
        dvCaseSetNextPropertyCase(pCase, dvCaseGetNextPropertyCase(_Case));
    } else {
        dvPropertySetFirstCase(Property, dvCaseGetNextPropertyCase(_Case));
    }
    dvCaseSetNextPropertyCase(_Case, dvCaseNull);
    if(dvPropertyGetLastCase(Property) == _Case) {
        dvPropertySetLastCase(Property, pCase);
    }
    dvCaseSetProperty(_Case, dvPropertyNull);
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowProperty(
    dvProperty Property)
{
    utDatabaseShowObject("dv", "Property", dvProperty2Index(Property));
}
#endif

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocRelationship(void)
{
    dvRelationship Relationship = dvRelationshipAlloc();

    return dvRelationship2Index(Relationship);
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Relationship.
----------------------------------------------------------------------------------------*/
static void allocRelationships(void)
{
    dvSetAllocatedRelationship(2);
    dvSetUsedRelationship(0);
    dvRelationships.Type = utNewA(dvRelationshipType, (dvAllocatedRelationship()));
    dvRelationships.ParentLabelSym = utNewA(utSym, (dvAllocatedRelationship()));
    dvRelationships.ChildLabelSym = utNewA(utSym, (dvAllocatedRelationship()));
    dvRelationships.Mandatory = utNewA(uint8, (dvAllocatedRelationship() + 7) >> 3);
    dvRelationships.Cascade = utNewA(uint8, (dvAllocatedRelationship() + 7) >> 3);
    dvRelationships.AccessChild = utNewA(uint8, (dvAllocatedRelationship() + 7) >> 3);
    dvRelationships.AccessParent = utNewA(uint8, (dvAllocatedRelationship() + 7) >> 3);
    dvRelationships.SharedParent = utNewA(uint8, (dvAllocatedRelationship() + 7) >> 3);
    dvRelationships.ParentClass = utNewA(dvClass, (dvAllocatedRelationship()));
    dvRelationships.NextClassChildRelationship = utNewA(dvRelationship, (dvAllocatedRelationship()));
    dvRelationships.ChildClass = utNewA(dvClass, (dvAllocatedRelationship()));
    dvRelationships.NextClassParentRelationship = utNewA(dvRelationship, (dvAllocatedRelationship()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Relationship.
----------------------------------------------------------------------------------------*/
static void reallocRelationships(
    uint32 newSize)
{
    utResizeArray(dvRelationships.Type, (newSize));
    utResizeArray(dvRelationships.ParentLabelSym, (newSize));
    utResizeArray(dvRelationships.ChildLabelSym, (newSize));
    utResizeArray(dvRelationships.Mandatory, (newSize + 7) >> 3);
    utResizeArray(dvRelationships.Cascade, (newSize + 7) >> 3);
    utResizeArray(dvRelationships.AccessChild, (newSize + 7) >> 3);
    utResizeArray(dvRelationships.AccessParent, (newSize + 7) >> 3);
    utResizeArray(dvRelationships.SharedParent, (newSize + 7) >> 3);
    utResizeArray(dvRelationships.ParentClass, (newSize));
    utResizeArray(dvRelationships.NextClassChildRelationship, (newSize));
    utResizeArray(dvRelationships.ChildClass, (newSize));
    utResizeArray(dvRelationships.NextClassParentRelationship, (newSize));
    dvSetAllocatedRelationship(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Relationships.
----------------------------------------------------------------------------------------*/
void dvRelationshipAllocMore(void)
{
    reallocRelationships(dvAllocatedRelationship() + (dvAllocatedRelationship() >> 1));
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowRelationship(
    dvRelationship Relationship)
{
    utDatabaseShowObject("dv", "Relationship", dvRelationship2Index(Relationship));
}
#endif

/*----------------------------------------------------------------------------------------
  Destroy Union including everything in it. Remove from parents.
----------------------------------------------------------------------------------------*/
void dvUnionDestroy(
    dvUnion Union)
{
    dvClass owningClass = dvUnionGetClass(Union);

    if(dvUnionDestructorCallback != NULL) {
        dvUnionDestructorCallback(Union);
    }
    if(owningClass != dvClassNull) {
        dvClassRemoveUnion(owningClass, Union);
    }
    dvUnionFree(Union);
}

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocUnion(void)
{
    dvUnion Union = dvUnionAlloc();

    return dvUnion2Index(Union);
}

/*----------------------------------------------------------------------------------------
  Destructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static void destroyUnion(
    uint64 objectIndex)
{
    dvUnionDestroy(dvIndex2Union(objectIndex));
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Union.
----------------------------------------------------------------------------------------*/
static void allocUnions(void)
{
    dvSetAllocatedUnion(2);
    dvSetUsedUnion(0);
    dvSetFirstFreeUnion(dvUnionNull);
    dvUnions.PropertySym = utNewA(utSym, (dvAllocatedUnion()));
    dvUnions.TypeProperty = utNewA(dvProperty, (dvAllocatedUnion()));
    dvUnions.Line = utNewA(uint32, (dvAllocatedUnion()));
    dvUnions.Number = utNewA(uint16, (dvAllocatedUnion()));
    dvUnions.FieldNumber = utNewA(uint32, (dvAllocatedUnion()));
    dvUnions.NumCases = utNewA(uint16, (dvAllocatedUnion()));
    dvUnions.Class = utNewA(dvClass, (dvAllocatedUnion()));
    dvUnions.NextClassUnion = utNewA(dvUnion, (dvAllocatedUnion()));
    dvUnions.FirstProperty = utNewA(dvProperty, (dvAllocatedUnion()));
    dvUnions.LastProperty = utNewA(dvProperty, (dvAllocatedUnion()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Union.
----------------------------------------------------------------------------------------*/
static void reallocUnions(
    uint32 newSize)
{
    utResizeArray(dvUnions.PropertySym, (newSize));
    utResizeArray(dvUnions.TypeProperty, (newSize));
    utResizeArray(dvUnions.Line, (newSize));
    utResizeArray(dvUnions.Number, (newSize));
    utResizeArray(dvUnions.FieldNumber, (newSize));
    utResizeArray(dvUnions.NumCases, (newSize));
    utResizeArray(dvUnions.Class, (newSize));
    utResizeArray(dvUnions.NextClassUnion, (newSize));
    utResizeArray(dvUnions.FirstProperty, (newSize));
    utResizeArray(dvUnions.LastProperty, (newSize));
    dvSetAllocatedUnion(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Unions.
----------------------------------------------------------------------------------------*/
void dvUnionAllocMore(void)
{
    reallocUnions(dvAllocatedUnion() + (dvAllocatedUnion() >> 1));
}

/*----------------------------------------------------------------------------------------
  Add the Property to the head of the list on the Union.
----------------------------------------------------------------------------------------*/
void dvUnionInsertProperty(
    dvUnion Union,
    dvProperty _Property)
{
#if defined(DD_DEBUG)
    if(Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetUnion(_Property) != dvUnionNull) {
        utExit("Attempting to add dv to Union twice");
    }
#endif
    dvPropertySetNextUnionProperty(_Property, dvUnionGetFirstProperty(Union));
    dvUnionSetFirstProperty(Union, _Property);
    if(dvUnionGetLastProperty(Union) == dvPropertyNull) {
        dvUnionSetLastProperty(Union, _Property);
    }
    dvPropertySetUnion(_Property, Union);
}

/*----------------------------------------------------------------------------------------
  Add the Property to the end of the list on the Union.
----------------------------------------------------------------------------------------*/
void dvUnionAppendProperty(
    dvUnion Union,
    dvProperty _Property)
{
#if defined(DD_DEBUG)
    if(Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetUnion(_Property) != dvUnionNull) {
        utExit("Attempting to add dv to Union twice");
    }
#endif
    if(dvUnionGetLastProperty(Union) != dvPropertyNull) {
        dvPropertySetNextUnionProperty(dvUnionGetLastProperty(Union), _Property);
    } else {
        dvUnionSetFirstProperty(Union, _Property);
    }
    dvUnionSetLastProperty(Union, _Property);
    dvPropertySetNextUnionProperty(_Property, dvPropertyNull);
    dvPropertySetUnion(_Property, Union);
}

/*----------------------------------------------------------------------------------------
  Insert the Property to the Union after the previous Property.
----------------------------------------------------------------------------------------*/
void dvUnionInsertAfterProperty(
    dvUnion Union,
    dvProperty prevProperty,
    dvProperty _Property)
{
    dvProperty nextProperty = dvPropertyGetNextUnionProperty(prevProperty);

#if defined(DD_DEBUG)
    if(Union == dvUnionNull) {
        utExit("Non-existent Union");
    }
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetUnion(_Property) != dvUnionNull) {
        utExit("Attempting to add dv to Union twice");
    }
#endif
    dvPropertySetNextUnionProperty(_Property, nextProperty);
    dvPropertySetNextUnionProperty(prevProperty, _Property);
    if(dvUnionGetLastProperty(Union) == prevProperty) {
        dvUnionSetLastProperty(Union, _Property);
    }
    dvPropertySetUnion(_Property, Union);
}

/*----------------------------------------------------------------------------------------
 Remove the Property from the Union.
----------------------------------------------------------------------------------------*/
void dvUnionRemoveProperty(
    dvUnion Union,
    dvProperty _Property)
{
    dvProperty pProperty, nProperty;

#if defined(DD_DEBUG)
    if(_Property == dvPropertyNull) {
        utExit("Non-existent Property");
    }
    if(dvPropertyGetUnion(_Property) != dvUnionNull && dvPropertyGetUnion(_Property) != Union) {
        utExit("Delete Property from non-owning Union");
    }
#endif
    pProperty = dvPropertyNull;
    for(nProperty = dvUnionGetFirstProperty(Union); nProperty != dvPropertyNull && nProperty != _Property;
            nProperty = dvPropertyGetNextUnionProperty(nProperty)) {
        pProperty = nProperty;
    }
    if(pProperty != dvPropertyNull) {
        dvPropertySetNextUnionProperty(pProperty, dvPropertyGetNextUnionProperty(_Property));
    } else {
        dvUnionSetFirstProperty(Union, dvPropertyGetNextUnionProperty(_Property));
    }
    dvPropertySetNextUnionProperty(_Property, dvPropertyNull);
    if(dvUnionGetLastProperty(Union) == _Property) {
        dvUnionSetLastProperty(Union, pProperty);
    }
    dvPropertySetUnion(_Property, dvUnionNull);
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowUnion(
    dvUnion Union)
{
    utDatabaseShowObject("dv", "Union", dvUnion2Index(Union));
}
#endif

/*----------------------------------------------------------------------------------------
  Destroy Case including everything in it. Remove from parents.
----------------------------------------------------------------------------------------*/
void dvCaseDestroy(
    dvCase Case)
{
    dvEntry owningEntry = dvCaseGetEntry(Case);
    dvProperty owningProperty = dvCaseGetProperty(Case);

    if(dvCaseDestructorCallback != NULL) {
        dvCaseDestructorCallback(Case);
    }
    if(owningEntry != dvEntryNull) {
        dvEntryRemoveCase(owningEntry, Case);
#if defined(DD_DEBUG)
    } else {
        utExit("Case without owning Entry");
#endif
    }
    if(owningProperty != dvPropertyNull) {
        dvPropertyRemoveCase(owningProperty, Case);
#if defined(DD_DEBUG)
    } else {
        utExit("Case without owning Property");
#endif
    }
    dvCaseFree(Case);
}

/*----------------------------------------------------------------------------------------
  Default constructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static uint64 allocCase(void)
{
    dvCase Case = dvCaseAlloc();

    return dvCase2Index(Case);
}

/*----------------------------------------------------------------------------------------
  Destructor wrapper for the database manager.
----------------------------------------------------------------------------------------*/
static void destroyCase(
    uint64 objectIndex)
{
    dvCaseDestroy(dvIndex2Case(objectIndex));
}

/*----------------------------------------------------------------------------------------
  Allocate the field arrays of Case.
----------------------------------------------------------------------------------------*/
static void allocCases(void)
{
    dvSetAllocatedCase(2);
    dvSetUsedCase(0);
    dvSetFirstFreeCase(dvCaseNull);
    dvCases.EntrySym = utNewA(utSym, (dvAllocatedCase()));
    dvCases.Entry = utNewA(dvEntry, (dvAllocatedCase()));
    dvCases.NextEntryCase = utNewA(dvCase, (dvAllocatedCase()));
    dvCases.Property = utNewA(dvProperty, (dvAllocatedCase()));
    dvCases.NextPropertyCase = utNewA(dvCase, (dvAllocatedCase()));
}

/*----------------------------------------------------------------------------------------
  Realloc the arrays of properties for class Case.
----------------------------------------------------------------------------------------*/
static void reallocCases(
    uint32 newSize)
{
    utResizeArray(dvCases.EntrySym, (newSize));
    utResizeArray(dvCases.Entry, (newSize));
    utResizeArray(dvCases.NextEntryCase, (newSize));
    utResizeArray(dvCases.Property, (newSize));
    utResizeArray(dvCases.NextPropertyCase, (newSize));
    dvSetAllocatedCase(newSize);
}

/*----------------------------------------------------------------------------------------
  Allocate more Cases.
----------------------------------------------------------------------------------------*/
void dvCaseAllocMore(void)
{
    reallocCases(dvAllocatedCase() + (dvAllocatedCase() >> 1));
}

#if defined(DD_DEBUG)
/*----------------------------------------------------------------------------------------
  Write out all the fields of an object.
----------------------------------------------------------------------------------------*/
void dvShowCase(
    dvCase Case)
{
    utDatabaseShowObject("dv", "Case", dvCase2Index(Case));
}
#endif

/*----------------------------------------------------------------------------------------
  Free memory used by the dv database.
----------------------------------------------------------------------------------------*/
void dvDatabaseStop(void)
{
    utFree(dvRoots.FirstModpath);
    utFree(dvRoots.LastModpath);
    utFree(dvRoots.ModpathTableIndex);
    utFree(dvRoots.NumModpathTable);
    utFree(dvRoots.ModpathTable);
    utFree(dvRoots.NumModpath);
    utFree(dvRoots.FirstModule);
    utFree(dvRoots.LastModule);
    utFree(dvRoots.ModuleTableIndex);
    utFree(dvRoots.NumModuleTable);
    utFree(dvRoots.ModuleTable);
    utFree(dvRoots.NumModule);
    utFree(dvModpaths.Root);
    utFree(dvModpaths.NextRootModpath);
    utFree(dvModpaths.PrevRootModpath);
    utFree(dvModpaths.Sym);
    utFree(dvModpaths.NextTableRootModpath);
    utFree(dvModules.PrefixSym);
    utFree(dvModules.Persistent);
    utFree(dvModules.UndoRedo);
    utFree(dvModules.NumFields);
    utFree(dvModules.NumClasses);
    utFree(dvModules.NumEnums);
    utFree(dvModules.NextRootModule);
    utFree(dvModules.PrevRootModule);
    utFree(dvModules.Sym);
    utFree(dvModules.NextTableRootModule);
    utFree(dvModules.FirstClass);
    utFree(dvModules.LastClass);
    utFree(dvModules.ClassTableIndex);
    utFree(dvModules.NumClassTable);
    utFree(dvModules.ClassTable);
    utFree(dvModules.NumClass);
    utFree(dvModules.FirstEnum);
    utFree(dvModules.LastEnum);
    utFree(dvModules.EnumTableIndex);
    utFree(dvModules.NumEnumTable);
    utFree(dvModules.EnumTable);
    utFree(dvModules.NumEnum);
    utFree(dvModules.FirstTypedef);
    utFree(dvModules.LastTypedef);
    utFree(dvModules.TypedefTableIndex);
    utFree(dvModules.NumTypedefTable);
    utFree(dvModules.TypedefTable);
    utFree(dvModules.NumTypedef);
    utFree(dvModules.FirstSchema);
    utFree(dvModules.LastSchema);
    utFree(dvModules.SchemaTableIndex);
    utFree(dvModules.NumSchemaTable);
    utFree(dvModules.SchemaTable);
    utFree(dvModules.NumSchema);
    utFree(dvModules.FirstImportLink);
    utFree(dvModules.LastImportLink);
    utFree(dvModules.FirstExportLink);
    utFree(dvModules.LastExportLink);
    utFree(dvLinks.ImportModule);
    utFree(dvLinks.NextModuleImportLink);
    utFree(dvLinks.ExportModule);
    utFree(dvLinks.NextModuleExportLink);
    utFree(dvSchemas.Module);
    utFree(dvSchemas.NextModuleSchema);
    utFree(dvSchemas.PrevModuleSchema);
    utFree(dvSchemas.Sym);
    utFree(dvSchemas.NextTableModuleSchema);
    utFree(dvSchemas.FirstClass);
    utFree(dvSchemas.LastClass);
    utFree(dvEnums.PrefixSym);
    utFree(dvEnums.NumEntries);
    utFree(dvEnums.Module);
    utFree(dvEnums.NextModuleEnum);
    utFree(dvEnums.PrevModuleEnum);
    utFree(dvEnums.Sym);
    utFree(dvEnums.NextTableModuleEnum);
    utFree(dvEnums.FirstEntry);
    utFree(dvEnums.LastEntry);
    utFree(dvEnums.EntryTableIndex);
    utFree(dvEnums.NumEntryTable);
    utFree(dvEnums.EntryTable);
    utFree(dvEnums.NumEntry);
    utFree(dvEntrys.Value);
    utFree(dvEntrys.Enum);
    utFree(dvEntrys.NextEnumEntry);
    utFree(dvEntrys.PrevEnumEntry);
    utFree(dvEntrys.Sym);
    utFree(dvEntrys.NextTableEnumEntry);
    utFree(dvEntrys.FirstCase);
    utFree(dvEntrys.LastCase);
    utFree(dvTypedefs.Module);
    utFree(dvTypedefs.NextModuleTypedef);
    utFree(dvTypedefs.PrevModuleTypedef);
    utFree(dvTypedefs.Sym);
    utFree(dvTypedefs.NextTableModuleTypedef);
    utFree(dvClasss.MemoryStyle);
    utFree(dvClasss.ReferenceSize);
    utFree(dvClasss.GenerateArrayClass);
    utFree(dvClasss.NumFields);
    utFree(dvClasss.Module);
    utFree(dvClasss.NextModuleClass);
    utFree(dvClasss.PrevModuleClass);
    utFree(dvClasss.Sym);
    utFree(dvClasss.NextTableModuleClass);
    utFree(dvClasss.Schema);
    utFree(dvClasss.NextSchemaClass);
    utFree(dvClasss.FirstProperty);
    utFree(dvClasss.LastProperty);
    utFree(dvClasss.PropertyTableIndex);
    utFree(dvClasss.NumPropertyTable);
    utFree(dvClasss.PropertyTable);
    utFree(dvClasss.NumProperty);
    utFree(dvClasss.FreeListProperty);
    utFree(dvClasss.BaseClass);
    utFree(dvClasss.FirstDerivedClass);
    utFree(dvClasss.NextClassDerivedClass);
    utFree(dvClasss.LastDerivedClass);
    utFree(dvClasss.FirstChildRelationship);
    utFree(dvClasss.LastChildRelationship);
    utFree(dvClasss.FirstParentRelationship);
    utFree(dvClasss.LastParentRelationship);
    utFree(dvClasss.FirstUnion);
    utFree(dvClasss.LastUnion);
    utFree(dvPropertys.Type);
    utFree(dvPropertys.Array);
    utFree(dvPropertys.Cascade);
    utFree(dvPropertys.FieldNumber);
    utFree(dvPropertys.FirstElementProp);
    utFree(dvPropertys.NumElementsProp);
    utFree(dvPropertys.Hidden);
    utFree(dvPropertys.Line);
    utFree(dvPropertys.Class);
    utFree(dvPropertys.NextClassProperty);
    utFree(dvPropertys.PrevClassProperty);
    utFree(dvPropertys.Sym);
    utFree(dvPropertys.NextTableClassProperty);
    utFree(dvPropertys.FirstCase);
    utFree(dvPropertys.LastCase);
    utFree(dvPropertys.Union);
    utFree(dvPropertys.NextUnionProperty);
    utFree(dvPropertys.union1);
    utFree(dvRelationships.Type);
    utFree(dvRelationships.ParentLabelSym);
    utFree(dvRelationships.ChildLabelSym);
    utFree(dvRelationships.Mandatory);
    utFree(dvRelationships.Cascade);
    utFree(dvRelationships.AccessChild);
    utFree(dvRelationships.AccessParent);
    utFree(dvRelationships.SharedParent);
    utFree(dvRelationships.ParentClass);
    utFree(dvRelationships.NextClassChildRelationship);
    utFree(dvRelationships.ChildClass);
    utFree(dvRelationships.NextClassParentRelationship);
    utFree(dvUnions.PropertySym);
    utFree(dvUnions.TypeProperty);
    utFree(dvUnions.Line);
    utFree(dvUnions.Number);
    utFree(dvUnions.FieldNumber);
    utFree(dvUnions.NumCases);
    utFree(dvUnions.Class);
    utFree(dvUnions.NextClassUnion);
    utFree(dvUnions.FirstProperty);
    utFree(dvUnions.LastProperty);
    utFree(dvCases.EntrySym);
    utFree(dvCases.Entry);
    utFree(dvCases.NextEntryCase);
    utFree(dvCases.Property);
    utFree(dvCases.NextPropertyCase);
    utUnregisterModule(dvModuleID);
}

/*----------------------------------------------------------------------------------------
  Allocate memory used by the dv database.
----------------------------------------------------------------------------------------*/
void dvDatabaseStart(void)
{
    if(!utInitialized()) {
        utStart();
    }
    dvRootData.hash = 0x14a34f87;
    dvModuleID = utRegisterModule("dv", dvHash(), 13, 165, 3, sizeof(struct dvRootType_),
        &dvRootData, dvDatabaseStart, dvDatabaseStop);
    utRegisterEnum("RelationshipType", 6);
    utRegisterEntry("LINKED_LIST", 0);
    utRegisterEntry("DOUBLY_LINKED", 1);
    utRegisterEntry("TAIL_LINKED", 2);
    utRegisterEntry("POINTER", 3);
    utRegisterEntry("ARRAY", 4);
    utRegisterEntry("HASHED", 5);
    utRegisterEnum("PropertyType", 12);
    utRegisterEntry("INT", 0);
    utRegisterEntry("UINT", 1);
    utRegisterEntry("FLOAT", 2);
    utRegisterEntry("DOUBLE", 3);
    utRegisterEntry("BIT", 4);
    utRegisterEntry("BOOL", 5);
    utRegisterEntry("CHAR", 6);
    utRegisterEntry("ENUM", 7);
    utRegisterEntry("TYPEDEF", 8);
    utRegisterEntry("POINTER", 9);
    utRegisterEntry("SYM", 10);
    utRegisterEntry("UNBOUND", 11);
    utRegisterEnum("MemoryStyle", 2);
    utRegisterEntry("CREATE_ONLY", 0);
    utRegisterEntry("FREE_LIST", 1);
    utRegisterClass("Root", 12, &dvRootData.usedRoot, &dvRootData.allocatedRoot,
        NULL, 0, 4, allocRoot, NULL);
    utRegisterField("FirstModpath", &dvRoots.FirstModpath, sizeof(dvModpath), UT_POINTER, "Modpath");
    utRegisterField("LastModpath", &dvRoots.LastModpath, sizeof(dvModpath), UT_POINTER, "Modpath");
    utRegisterField("ModpathTableIndex", &dvRoots.ModpathTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumModpathTable", &dvRoots.NumModpathTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("ModpathTable", &dvRoots.ModpathTable, sizeof(dvModpath), UT_POINTER, "Modpath");
    utRegisterArray(&dvRootData.usedRootModpathTable, &dvRootData.allocatedRootModpathTable,
        getRootModpathTables, allocRootModpathTables);
    utRegisterField("NumModpath", &dvRoots.NumModpath, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FirstModule", &dvRoots.FirstModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("LastModule", &dvRoots.LastModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("ModuleTableIndex", &dvRoots.ModuleTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumModuleTable", &dvRoots.NumModuleTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("ModuleTable", &dvRoots.ModuleTable, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterArray(&dvRootData.usedRootModuleTable, &dvRootData.allocatedRootModuleTable,
        getRootModuleTables, allocRootModuleTables);
    utRegisterField("NumModule", &dvRoots.NumModule, sizeof(uint32), UT_UINT, NULL);
    utRegisterClass("Modpath", 5, &dvRootData.usedModpath, &dvRootData.allocatedModpath,
        NULL, 12, 4, allocModpath, NULL);
    utRegisterField("Root", &dvModpaths.Root, sizeof(dvRoot), UT_POINTER, "Root");
    utRegisterField("NextRootModpath", &dvModpaths.NextRootModpath, sizeof(dvModpath), UT_POINTER, "Modpath");
    utRegisterField("PrevRootModpath", &dvModpaths.PrevRootModpath, sizeof(dvModpath), UT_POINTER, "Modpath");
    utRegisterField("Sym", &dvModpaths.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableRootModpath", &dvModpaths.NextTableRootModpath, sizeof(dvModpath), UT_POINTER, "Modpath");
    utRegisterClass("Module", 38, &dvRootData.usedModule, &dvRootData.allocatedModule,
        NULL, 17, 4, allocModule, NULL);
    utRegisterField("PrefixSym", &dvModules.PrefixSym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("Persistent", &dvModules.Persistent, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("UndoRedo", &dvModules.UndoRedo, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("NumFields", &dvModules.NumFields, sizeof(uint16), UT_UINT, NULL);
    utRegisterField("NumClasses", &dvModules.NumClasses, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("NumEnums", &dvModules.NumEnums, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("NextRootModule", &dvModules.NextRootModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("PrevRootModule", &dvModules.PrevRootModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("Sym", &dvModules.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableRootModule", &dvModules.NextTableRootModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("FirstClass", &dvModules.FirstClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("LastClass", &dvModules.LastClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("ClassTableIndex", &dvModules.ClassTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumClassTable", &dvModules.NumClassTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("ClassTable", &dvModules.ClassTable, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterArray(&dvRootData.usedModuleClassTable, &dvRootData.allocatedModuleClassTable,
        getModuleClassTables, allocModuleClassTables);
    utRegisterField("NumClass", &dvModules.NumClass, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FirstEnum", &dvModules.FirstEnum, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterField("LastEnum", &dvModules.LastEnum, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterField("EnumTableIndex", &dvModules.EnumTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumEnumTable", &dvModules.NumEnumTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("EnumTable", &dvModules.EnumTable, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterArray(&dvRootData.usedModuleEnumTable, &dvRootData.allocatedModuleEnumTable,
        getModuleEnumTables, allocModuleEnumTables);
    utRegisterField("NumEnum", &dvModules.NumEnum, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FirstTypedef", &dvModules.FirstTypedef, sizeof(dvTypedef), UT_POINTER, "Typedef");
    utRegisterField("LastTypedef", &dvModules.LastTypedef, sizeof(dvTypedef), UT_POINTER, "Typedef");
    utRegisterField("TypedefTableIndex", &dvModules.TypedefTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumTypedefTable", &dvModules.NumTypedefTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("TypedefTable", &dvModules.TypedefTable, sizeof(dvTypedef), UT_POINTER, "Typedef");
    utRegisterArray(&dvRootData.usedModuleTypedefTable, &dvRootData.allocatedModuleTypedefTable,
        getModuleTypedefTables, allocModuleTypedefTables);
    utRegisterField("NumTypedef", &dvModules.NumTypedef, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FirstSchema", &dvModules.FirstSchema, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterField("LastSchema", &dvModules.LastSchema, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterField("SchemaTableIndex", &dvModules.SchemaTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumSchemaTable", &dvModules.NumSchemaTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("SchemaTable", &dvModules.SchemaTable, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterArray(&dvRootData.usedModuleSchemaTable, &dvRootData.allocatedModuleSchemaTable,
        getModuleSchemaTables, allocModuleSchemaTables);
    utRegisterField("NumSchema", &dvModules.NumSchema, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FirstImportLink", &dvModules.FirstImportLink, sizeof(dvLink), UT_POINTER, "Link");
    utRegisterField("LastImportLink", &dvModules.LastImportLink, sizeof(dvLink), UT_POINTER, "Link");
    utRegisterField("FirstExportLink", &dvModules.FirstExportLink, sizeof(dvLink), UT_POINTER, "Link");
    utRegisterField("LastExportLink", &dvModules.LastExportLink, sizeof(dvLink), UT_POINTER, "Link");
    utRegisterClass("Link", 4, &dvRootData.usedLink, &dvRootData.allocatedLink,
        NULL, 55, 4, allocLink, NULL);
    utRegisterField("ImportModule", &dvLinks.ImportModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("NextModuleImportLink", &dvLinks.NextModuleImportLink, sizeof(dvLink), UT_POINTER, "Link");
    utRegisterField("ExportModule", &dvLinks.ExportModule, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("NextModuleExportLink", &dvLinks.NextModuleExportLink, sizeof(dvLink), UT_POINTER, "Link");
    utRegisterClass("Schema", 7, &dvRootData.usedSchema, &dvRootData.allocatedSchema,
        NULL, 59, 4, allocSchema, NULL);
    utRegisterField("Module", &dvSchemas.Module, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("NextModuleSchema", &dvSchemas.NextModuleSchema, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterField("PrevModuleSchema", &dvSchemas.PrevModuleSchema, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterField("Sym", &dvSchemas.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableModuleSchema", &dvSchemas.NextTableModuleSchema, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterField("FirstClass", &dvSchemas.FirstClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("LastClass", &dvSchemas.LastClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterClass("Enum", 13, &dvRootData.usedEnum, &dvRootData.allocatedEnum,
        NULL, 66, 4, allocEnum, NULL);
    utRegisterField("PrefixSym", &dvEnums.PrefixSym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NumEntries", &dvEnums.NumEntries, sizeof(uint16), UT_UINT, NULL);
    utRegisterField("Module", &dvEnums.Module, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("NextModuleEnum", &dvEnums.NextModuleEnum, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterField("PrevModuleEnum", &dvEnums.PrevModuleEnum, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterField("Sym", &dvEnums.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableModuleEnum", &dvEnums.NextTableModuleEnum, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterField("FirstEntry", &dvEnums.FirstEntry, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterField("LastEntry", &dvEnums.LastEntry, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterField("EntryTableIndex", &dvEnums.EntryTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumEntryTable", &dvEnums.NumEntryTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("EntryTable", &dvEnums.EntryTable, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterArray(&dvRootData.usedEnumEntryTable, &dvRootData.allocatedEnumEntryTable,
        getEnumEntryTables, allocEnumEntryTables);
    utRegisterField("NumEntry", &dvEnums.NumEntry, sizeof(uint32), UT_UINT, NULL);
    utRegisterClass("Entry", 8, &dvRootData.usedEntry, &dvRootData.allocatedEntry,
        NULL, 79, 4, allocEntry, NULL);
    utRegisterField("Value", &dvEntrys.Value, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("Enum", &dvEntrys.Enum, sizeof(dvEnum), UT_POINTER, "Enum");
    utRegisterField("NextEnumEntry", &dvEntrys.NextEnumEntry, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterField("PrevEnumEntry", &dvEntrys.PrevEnumEntry, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterField("Sym", &dvEntrys.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableEnumEntry", &dvEntrys.NextTableEnumEntry, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterField("FirstCase", &dvEntrys.FirstCase, sizeof(dvCase), UT_POINTER, "Case");
    utRegisterField("LastCase", &dvEntrys.LastCase, sizeof(dvCase), UT_POINTER, "Case");
    utRegisterClass("Typedef", 5, &dvRootData.usedTypedef, &dvRootData.allocatedTypedef,
        NULL, 87, 4, allocTypedef, NULL);
    utRegisterField("Module", &dvTypedefs.Module, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("NextModuleTypedef", &dvTypedefs.NextModuleTypedef, sizeof(dvTypedef), UT_POINTER, "Typedef");
    utRegisterField("PrevModuleTypedef", &dvTypedefs.PrevModuleTypedef, sizeof(dvTypedef), UT_POINTER, "Typedef");
    utRegisterField("Sym", &dvTypedefs.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableModuleTypedef", &dvTypedefs.NextTableModuleTypedef, sizeof(dvTypedef), UT_POINTER, "Typedef");
    utRegisterClass("Class", 28, &dvRootData.usedClass, &dvRootData.allocatedClass,
        NULL, 92, 4, allocClass, NULL);
    utRegisterField("MemoryStyle", &dvClasss.MemoryStyle, sizeof(dvMemoryStyle), UT_ENUM, "MemoryStyle");
    utRegisterField("ReferenceSize", &dvClasss.ReferenceSize, sizeof(uint8), UT_UINT, NULL);
    utRegisterField("GenerateArrayClass", &dvClasss.GenerateArrayClass, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("NumFields", &dvClasss.NumFields, sizeof(uint16), UT_UINT, NULL);
    utRegisterField("Module", &dvClasss.Module, sizeof(dvModule), UT_POINTER, "Module");
    utRegisterField("NextModuleClass", &dvClasss.NextModuleClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("PrevModuleClass", &dvClasss.PrevModuleClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("Sym", &dvClasss.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableModuleClass", &dvClasss.NextTableModuleClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("Schema", &dvClasss.Schema, sizeof(dvSchema), UT_POINTER, "Schema");
    utRegisterField("NextSchemaClass", &dvClasss.NextSchemaClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("FirstProperty", &dvClasss.FirstProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("LastProperty", &dvClasss.LastProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("PropertyTableIndex", &dvClasss.PropertyTableIndex, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("NumPropertyTable", &dvClasss.NumPropertyTable, sizeof(uint32), UT_UINT, NULL);
    utSetFieldHidden();
    utRegisterField("PropertyTable", &dvClasss.PropertyTable, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterArray(&dvRootData.usedClassPropertyTable, &dvRootData.allocatedClassPropertyTable,
        getClassPropertyTables, allocClassPropertyTables);
    utRegisterField("NumProperty", &dvClasss.NumProperty, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FreeListProperty", &dvClasss.FreeListProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("BaseClass", &dvClasss.BaseClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("FirstDerivedClass", &dvClasss.FirstDerivedClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("NextClassDerivedClass", &dvClasss.NextClassDerivedClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("LastDerivedClass", &dvClasss.LastDerivedClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("FirstChildRelationship", &dvClasss.FirstChildRelationship, sizeof(dvRelationship), UT_POINTER, "Relationship");
    utRegisterField("LastChildRelationship", &dvClasss.LastChildRelationship, sizeof(dvRelationship), UT_POINTER, "Relationship");
    utRegisterField("FirstParentRelationship", &dvClasss.FirstParentRelationship, sizeof(dvRelationship), UT_POINTER, "Relationship");
    utRegisterField("LastParentRelationship", &dvClasss.LastParentRelationship, sizeof(dvRelationship), UT_POINTER, "Relationship");
    utRegisterField("FirstUnion", &dvClasss.FirstUnion, sizeof(dvUnion), UT_POINTER, "Union");
    utRegisterField("LastUnion", &dvClasss.LastUnion, sizeof(dvUnion), UT_POINTER, "Union");
    utRegisterClass("Property", 18, &dvRootData.usedProperty, &dvRootData.allocatedProperty,
        NULL, 120, 4, allocProperty, NULL);
    utRegisterField("Type", &dvPropertys.Type, sizeof(dvPropertyType), UT_ENUM, "PropertyType");
    utRegisterField("Array", &dvPropertys.Array, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("Cascade", &dvPropertys.Cascade, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("FieldNumber", &dvPropertys.FieldNumber, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("FirstElementProp", &dvPropertys.FirstElementProp, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("NumElementsProp", &dvPropertys.NumElementsProp, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("Hidden", &dvPropertys.Hidden, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("Line", &dvPropertys.Line, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("Class", &dvPropertys.Class, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("NextClassProperty", &dvPropertys.NextClassProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("PrevClassProperty", &dvPropertys.PrevClassProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("Sym", &dvPropertys.Sym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("NextTableClassProperty", &dvPropertys.NextTableClassProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("FirstCase", &dvPropertys.FirstCase, sizeof(dvCase), UT_POINTER, "Case");
    utRegisterField("LastCase", &dvPropertys.LastCase, sizeof(dvCase), UT_POINTER, "Case");
    utRegisterField("Union", &dvPropertys.Union, sizeof(dvUnion), UT_POINTER, "Union");
    utRegisterField("NextUnionProperty", &dvPropertys.NextUnionProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("union1", &dvPropertys.union1, sizeof(dvPropertyUnion1), UT_UNION, "Type");
    utRegisterUnion("Type", 6);
    utRegisterUnionCase(7, UT_POINTER, sizeof(dvEnum));
    utRegisterUnionCase(8, UT_POINTER, sizeof(dvTypedef));
    utRegisterUnionCase(9, UT_POINTER, sizeof(dvClass));
    utRegisterUnionCase(10, UT_SYM, sizeof(utSym));
    utRegisterUnionCase(0, UT_UINT, sizeof(uint8));
    utRegisterUnionCase(1, UT_UINT, sizeof(uint8));
    utRegisterClass("Relationship", 12, &dvRootData.usedRelationship, &dvRootData.allocatedRelationship,
        NULL, 138, 4, allocRelationship, NULL);
    utRegisterField("Type", &dvRelationships.Type, sizeof(dvRelationshipType), UT_ENUM, "RelationshipType");
    utRegisterField("ParentLabelSym", &dvRelationships.ParentLabelSym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("ChildLabelSym", &dvRelationships.ChildLabelSym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("Mandatory", &dvRelationships.Mandatory, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("Cascade", &dvRelationships.Cascade, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("AccessChild", &dvRelationships.AccessChild, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("AccessParent", &dvRelationships.AccessParent, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("SharedParent", &dvRelationships.SharedParent, sizeof(uint8), UT_BIT, NULL);
    utRegisterField("ParentClass", &dvRelationships.ParentClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("NextClassChildRelationship", &dvRelationships.NextClassChildRelationship, sizeof(dvRelationship), UT_POINTER, "Relationship");
    utRegisterField("ChildClass", &dvRelationships.ChildClass, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("NextClassParentRelationship", &dvRelationships.NextClassParentRelationship, sizeof(dvRelationship), UT_POINTER, "Relationship");
    utRegisterClass("Union", 10, &dvRootData.usedUnion, &dvRootData.allocatedUnion,
        &dvRootData.firstFreeUnion, 150, 4, allocUnion, destroyUnion);
    utRegisterField("PropertySym", &dvUnions.PropertySym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("TypeProperty", &dvUnions.TypeProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("Line", &dvUnions.Line, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("Number", &dvUnions.Number, sizeof(uint16), UT_UINT, NULL);
    utRegisterField("FieldNumber", &dvUnions.FieldNumber, sizeof(uint32), UT_UINT, NULL);
    utRegisterField("NumCases", &dvUnions.NumCases, sizeof(uint16), UT_UINT, NULL);
    utRegisterField("Class", &dvUnions.Class, sizeof(dvClass), UT_POINTER, "Class");
    utRegisterField("NextClassUnion", &dvUnions.NextClassUnion, sizeof(dvUnion), UT_POINTER, "Union");
    utRegisterField("FirstProperty", &dvUnions.FirstProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("LastProperty", &dvUnions.LastProperty, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterClass("Case", 5, &dvRootData.usedCase, &dvRootData.allocatedCase,
        &dvRootData.firstFreeCase, 160, 4, allocCase, destroyCase);
    utRegisterField("EntrySym", &dvCases.EntrySym, sizeof(utSym), UT_SYM, NULL);
    utRegisterField("Entry", &dvCases.Entry, sizeof(dvEntry), UT_POINTER, "Entry");
    utRegisterField("NextEntryCase", &dvCases.NextEntryCase, sizeof(dvCase), UT_POINTER, "Case");
    utRegisterField("Property", &dvCases.Property, sizeof(dvProperty), UT_POINTER, "Property");
    utRegisterField("NextPropertyCase", &dvCases.NextPropertyCase, sizeof(dvCase), UT_POINTER, "Case");
    allocRoots();
    allocModpaths();
    allocModules();
    allocLinks();
    allocSchemas();
    allocEnums();
    allocEntrys();
    allocTypedefs();
    allocClasss();
    allocPropertys();
    allocRelationships();
    allocUnions();
    allocCases();
}

